/* $Id: Comm.h,v 1.10 1997/10/01 19:37:00 proman Exp $ */
/*
 * File:       Comm.h
 * Author:     Pawel Roman (proman@npac.syr.edu)
 * Created:    long time ago
 * Revision:   $Revision: 1.10 $
 * Updated:    $Date: 1997/10/01 19:37:00 $
 * Copyright 1996/7 NPAC, Syracuse University
 *
 */

#ifndef COMM_H
#define COMM_H

#include <iostream.h>

#include <ace/Log_Msg.h>
#include <ace/Time_Value.h>
#include <ace/Trace.h>

#include <Inventor/fields/SoSFVec3f.h>
#include <Inventor/fields/SoSFRotation.h>

typedef float PosParams[3];
typedef float MoveParams[4];
typedef float CamParams[7];
typedef unsigned short WorldPrm;
typedef WorldPrm WorldParams[4];
typedef signed char LayerCode;

inline ostream&
operator<<( ostream& os, const PosParams& pos )
{
   return os << '[' << pos[0] << ", "<< pos[1] << ", "<< pos[2] << ']';
}

inline ostream&
operator<<( ostream& os, const MoveParams& mov )
{
   return os << '[' << mov[0] << ", "<< mov[1] << ", "<< mov[2] << ", " << mov[3] << ']';
}

inline ostream&
operator<<( ostream& os, const CamParams& cp )
{
   return os << '[' << cp[0] << ", " << cp[1] << ", " << cp[2] << "] ["
	     << cp[3] << ", " << cp[4]<< ", " << cp[5]<< ", " << cp[6] << ']';
}

inline ostream&
operator<<( ostream& os, const SbVec3f& vec )
{
   const float* ftab = vec.getValue();
   return os << '[' << ftab[0] << ", " << ftab[1] << ", "<< ftab[2] << ']';
}

inline ostream&
operator<<( ostream& os, const SoSFVec3f& trans )
{
   const float* ftab = trans.getValue().getValue();
   return os << '[' << ftab[0] << ", " << ftab[1] << ", "<< ftab[2] << ']';
}

inline ostream&
operator<<( ostream& os, const SoSFRotation& rot )
{
   const float* ftab = rot.getValue().getValue();
   return os << '[' << ftab[0] << ", " << ftab[1] << ", "<< ftab[2] << ", " << ftab[3] << ']';
}

inline ostream&
operator<<( ostream& os, const SbRotation& rot )
{
   const float* ftab = rot.getValue();
   return os << '[' << ftab[0] << ", " << ftab[1] << ", "<< ftab[2] << ", " << ftab[3] << ']';
}

#if defined (ACE_NDEBUG)
#define TRACE_NEW_2( __VPTR, __VTYPE )\
do { (__VPTR) = new __VTYPE; } while( 0 )
   
#define TRACE_DELETE( __VPTR )\
   do { delete __VPTR; } while( 0 )

#define TRACE_DELETE_TAB( __VPTR )\
      do { delete[] __VPTR; } while( 0 )
   
#define TRACE_REF( __SOBASE )\
	 do {  (__SOBASE)->ref(); } while( 0 )

#define TRACE_UNREF( __SOBASE )\
	    do {  (__SOBASE)->unref(); } while( 0 )
   
#define TRACE_UNREF_NODEL( __SOBASE )\
	       do {  (__SOBASE)->unrefNoDelete(); } while( 0 )
   
#else
#define TRACE_NEW_2( __VPTR, __VTYPE )\
		  do { (__VPTR) = new __VTYPE; \
		  int __ace_error = errno; \
		  ACE_Log_Msg *ace___ = ACE_Log_Msg::instance (); \
		  ace___->set( __FILE__, __LINE__, 0, __ace_error, ace___->restart(), ace___->msg_ostream() ); \
		  ace___->log( LM_DEBUG, "[%N:%l] NEW: %s %s=0x%x\n", #__VTYPE, #__VPTR, __VPTR ) ; \
		  } while( 0 )

#define TRACE_DELETE( __VPTR )\
		     do { int __ace_error = errno; \
		     ACE_Log_Msg *ace___ = ACE_Log_Msg::instance (); \
		     ace___->set( __FILE__, __LINE__, 0, __ace_error, ace___->restart(), ace___->msg_ostream() ); \
		     ace___->log( LM_DEBUG, "[%N:%l] DEL: %s=0x%x\n", #__VPTR, (__VPTR) ) ; \
		     delete (__VPTR); \
		     } while( 0 )
			   
#define TRACE_DELETE_TAB( __VPTR )\
			do { int __ace_error = errno;\
			ACE_Log_Msg *ace___ = ACE_Log_Msg::instance ();\
			ace___->set( __FILE__, __LINE__, 0, __ace_error, ace___->restart(), ace___->msg_ostream() );\
			ace___->log( LM_DEBUG, "[%N:%l] DEL_TAB: %s=0x%x\n", #__VPTR, (__VPTR) ) ;\
			delete[] (__VPTR);\
			} while( 0 )
			   
#define TRACE_REF( __SOBASE )\
			   do { (__SOBASE)->ref();\
			   int __ace_error = errno;\
			   ACE_Log_Msg *ace___ = ACE_Log_Msg::instance ();\
			   ace___->set( __FILE__, __LINE__, 0, __ace_error, ace___->restart(), ace___->msg_ostream() );\
			   ace___->log( LM_DEBUG, "[%N:%l] REF: %s(0x%x): %d\n", #__SOBASE, (__SOBASE), (__SOBASE)->getRefCount() );\
			   } while( 0 )

#define TRACE_UNREF( __SOBASE )\
			      do { int __ace_error = errno;\
			      ACE_Log_Msg *ace___ = ACE_Log_Msg::instance ();\
			      ace___->set( __FILE__, __LINE__, 0, __ace_error, ace___->restart(), ace___->msg_ostream() );\
			      ace___->log( LM_DEBUG, "[%N:%l] UNREF: %s(0x%x): %d\n", #__SOBASE, (__SOBASE), (__SOBASE)->getRefCount() );\
			      (__SOBASE)->unref();\
			      } while( 0 )

#define TRACE_UNREF_NODEL( __SOBASE )\
				 do { int __ace_error = errno;\
				 ACE_Log_Msg *ace___ = ACE_Log_Msg::instance ();\
				 ace___->set( __FILE__, __LINE__, 0, __ace_error, ace___->restart(), ace___->msg_ostream() );\
			      ace___->log( LM_DEBUG, "[%N:%l] UNREF: %s(0x%x): %d\n", #__SOBASE, (__SOBASE), (__SOBASE)->getRefCount() );\
				 (__SOBASE)->unrefNoDelete();\
				 } while( 0 )

#endif /*ACE_NDEBUG*/

#endif /*COMM_H*/
