/* $Id: DEM.C,v 1.19 1997/10/01 19:38:55 proman Exp $ */
/*
 * File:       DEM.C
 * Author:     Remek Trzaska (remek@npac.syr.edu)
 * Created:    long time ago
 * Revision:   $Revision: 1.19 $
 * Updated:    $Date: 1997/10/01 19:38:55 $
 * Copyright 1996/97 NPAC, Syracuse University
 *
 */

#include "DEM.h"

DEM::DEM(char *aName)
{
   ACE_TRACE( "(DEM::DEM(char*))" );
   strcpy(name, aName);
   data = NULL;
};

DEM::DEM(ELEV **aData, int aDim): dim(aDim)
{
   ACE_TRACE( "(DEM::DEM(ELEV**,int))" );
   allocMem();
   for(int i=0; i<dim; i++)
      for(int j=0; j<dim; j++)
	 data[i][j] = aData[i][j];
};

void 
DEM::setData(ELEV **newData)
{
   ACE_TRACE( "DEM::setData" );
   DeleteMatrix(data);
   data = newData;
}

void
DEM::allocMem()
{
   ACE_TRACE( "DEM::allocMem" );
   NewMatrix(data, dim, dim, ELEV);
}

void 
DEM::freeMem()
{
   ACE_TRACE( "DEM::freeMem" );
   if(data!=NULL)
      DeleteMatrix(data);
}

void
DEM::writeRect(int xfrom, int xto, int yfrom, int yto, char *fname)
{
   ACE_TRACE( "DEM::writeRect" );
   info("Writing the rectangle to a file...", 0);
   info(fname, 0);
  
   ofstream out(fname);
  
   out.put(0);
   out.put(0);
   out.put((xto-xfrom)/256);
   out.put((xto-xfrom)%256);
   out.put(0);
   out.put(0);
   out.put((yto-yfrom)/256);
   out.put((yto-yfrom)%256);

   for(int i=yfrom; i<yto; i++)
      for(int j=xto-1; j>=xfrom; j--) {
	 out.put(int(data[i][j])/256);
	 out.put(int(data[i][j])%256);
      }

   out.close();
   info("...done\n", 0);
  
}

////////////////////////////////////////

int
PGMDEM::read()
{
   ACE_TRACE( "PGMDEM::read" );
   int n, i, j;
   char buf[255];
   char c;

   FILE *f = fopen(name, "rb");
   if (f==NULL) fatal("Cannot open the data file!");
   // the pgm header -- P5 or sth
   fscanf(f, "%s", &buf);
   if ((c=fgetc(f))=='#') { //comment
      for(; (c=fgetc(f))!='\n'; ) ;
   } else ungetc(c, f);
   // the dimensions: number of columns and the rows, respectively
   fscanf(f, "%d %d", &dim, &dim);
   if ((c=fgetc(f))=='#') { //comment
      for(; (c=fgetc(f))!='\n'; ) ;
   } else ungetc(c, f);
   // the maximum value -- determines the data element size
   fscanf(f, "%d", &n);
   if (n!=255) 
      fatal("Can only handle 255-shades files");
   // and the last carridge return character
   fgetc(f); 

   // prepare the necessary memory space 
   allocMem();

   // and read the actual data
   for(i=0; i<dim; i++)
      for(j=0; j<dim; j++) 
	 if((data[i][j]=fgetc(f))==EOF)
	    fatal("File too short: ", name);
    
   // just to make sure the file is valid
   if (fgetc(f)!=EOF)
      fatal("File too long: ", name);
   fclose(f);
   return 0;
}

////////////////////////////////////////

int
URLDEM::read()
{
   ACE_TRACE( "(URLDEM::read)" );
   WebNode node( name );
   if( ! node )
   {
      ACE_ERROR(( LM_ERROR,
		  "[!](URLDEM::read) WebNode error: %s\n",
		  node.getErrMsg() ));
      return 1;
   }
   processWord(node);
   return 0;
}

void URLDEM::
processWord(WebNode& node)
{
   ACE_TRACE( "(URLDEM::processWord)" );
   char *buf = node.getBuffer();
   dim = ((buf[0]*256 + buf[1])*256 + buf[2])*256 + buf[3];

   buf = buf + 8;

   allocMem();

   int row, col;
   for(row=0; row<dim; row++) 
      for(col=0; col<dim; col++)
      {
	 int newrow = dim - col - 1;
	 int newcol = dim - row - 1;
	 data[newrow][newcol]= buf[2*(row*dim+col)]*256.0
	    +buf[2*(row*dim+col)+1];
      }
   TRACE_DELETE_TAB( node.getBuffer() );
}
