/* $Id: FixedVector.h,v 1.8 1997/09/25 20:31:47 remek Exp $ */
/*
 * File:       FixedVector.h
 * Author:     Pawel Roman (proman@npac.syr.edu)
 * Created:    long time ago
 * Revision:   $Revision: 1.8 $
 * Updated:    $Date: 1997/09/25 20:31:47 $
 * Copyright 1996/7 NPAC, Syracuse University
 *
 */

#ifndef _FIXEDVECTOR_H
#define _FIXEDVECTOR_H

template<class ITEM_TYPE>
class FixedVector
{
   public:
      enum VRESULT { V_OK, V_FAIL, V_NONEX, V_NOMEM };
      typedef unsigned char idx_type;
      typedef ITEM_TYPE value_type;
  
      FixedVector( idx_type a_size, const ITEM_TYPE& nonex );
      ~FixedVector( void );
      void addItem( idx_type idx, const ITEM_TYPE& item );
      ITEM_TYPE& removeItem( idx_type idx );
      ITEM_TYPE& operator[]( idx_type idx );
      operator const void* () const;
      
   private:
      ITEM_TYPE non_existent;
      ITEM_TYPE* store_ptr;
      VRESULT intStatus;
      idx_type size;
};

template<class ITEM_TYPE>
inline ITEM_TYPE& FixedVector<ITEM_TYPE>::
operator[]( idx_type idx )
{ 
   ACE_TRACE( "(FixedVector::operator[])" );
   if( idx < size )
      if( store_ptr[idx] != non_existent ) 
      {
	 intStatus = V_OK;
	 return store_ptr[idx];
      }
   ACE_ERROR(( LM_ERROR, "[!](FixedVector::operator[]) %d: does not exist\n", idx ));
   intStatus = V_NONEX;
   return non_existent;
}
  
template<class ITEM_TYPE>
inline FixedVector<ITEM_TYPE>::
operator const void* () const
{
   ACE_TRACE( "(FixedVector::operator const void*)" );
   return ( const void* )!intStatus;
}

template<class ITEM_TYPE>
FixedVector<ITEM_TYPE>::
FixedVector( idx_type a_size, const ITEM_TYPE& nonex ) 
   : size( a_size ), store_ptr( new ITEM_TYPE[a_size] )
{
   if( ! store_ptr )
   {
      ACE_ERROR(( LM_ERROR, "[!](FixedVector::FixedVector) No memory for store_ptr\n" ));
      intStatus = V_NOMEM;
      return;
   }
   non_existent = nonex;
   for( idx_type i=0; i<size; i++ )
      store_ptr[i] = non_existent;
   
   intStatus = V_OK;
}

template<class ITEM_TYPE>
FixedVector<ITEM_TYPE>::
~FixedVector( void ) 
{
   delete[] store_ptr;
}

template<class ITEM_TYPE>
void FixedVector<ITEM_TYPE>::
addItem( idx_type idx, const ITEM_TYPE& item )
{
   if( idx < size )
   {
#ifndef NDEBUG
      if( store_ptr[idx] != non_existent )
	 ACE_DEBUG(( LM_DEBUG, "overwritting previous item %d\n", idx ));
#endif
      store_ptr[idx] = item;
   }
   else
   {
      ACE_ERROR(( LM_ERROR, "[!](FixedVector::addItem) No room for %d\n", idx ));
      intStatus = V_FAIL;
   }
}

template<class ITEM_TYPE>
ITEM_TYPE& FixedVector<ITEM_TYPE>::
removeItem( idx_type idx )
{
   if( idx < size )
   {
      ITEM_TYPE& removed = store_ptr[idx];
      store_ptr[idx] = non_existent;
      return removed;
   }
   else
   {
      ACE_ERROR(( LM_ERROR, "[!](FixedVector::erase) %d: does not exist\n", idx ));
      intStatus = V_NONEX;
      return non_existent;
   }
}

#endif
