/* $Id: GeoPoint.C,v 1.9 1997/09/25 20:31:47 remek Exp $ */
/*
 * File         : GeoPoint.C
 * Author       : Paul Roman, changes by Remek Trzaska
 * Created      : Sun Jul 20 16:17:55 1997
 * Revision     : $Revision: 1.9 $
 * Updated      : $Date: 1997/09/25 20:31:47 $
 * Copyright: Northeast Parallel Architectures Center
 *            at Syracuse University 1997
 */

#include "GeoPoint.h"
#include "TagParser.h"

GeoPoint::
operator string( void ) const
{
   ACE_TRACE( "(GeoPoint::operator string)" );
   char buf[20];
   int lon_d = lon / 3600;
   int lon_m = ( lon % 3600 ) / 60;
   int lon_s = lon % 60;
   int lat_d = lat / 3600;
   int lat_m = ( lat % 3600 ) / 60;
   int lat_s = lat % 60;
   sprintf( buf, "%d:%d:%d%c%d:%d:%d", lon_d, lon_m, lon_s, TOKEN_SEPARATOR,
	    lat_d, lat_m, lat_s );
   return buf;
}

string GeoPoint::
fullString( void ) const
{
   ACE_TRACE( "(GeoPoint::fullString)" );
   char buf[32];
   int lon_d = lon / 3600;
   int lon_m = ( lon % 3600 ) / 60;
   int lon_s = lon % 60;
   int lat_d = lat / 3600;
   int lat_m = ( lat % 3600 ) / 60;
   int lat_s = lat % 60;
   sprintf( buf, "%d:%d:%d%c%d:%d:%d %d", lon_d, lon_m, lon_s, TOKEN_SEPARATOR,
	    lat_d, lat_m, lat_s, alt );
   return buf;
}

string GeoPoint::
longString( void ) const
{
   ACE_TRACE( "(GeoPoint::longString)" );
   char buf[32];
   int lon_d = lon / 3600;
   int lon_m = ( lon % 3600 ) / 60;
   int lon_s = lon % 60;
   int lat_d = lat / 3600;
   int lat_m = ( lat % 3600 ) / 60;
   int lat_s = lat % 60;
   sprintf( buf, "%d:%d:%d %c%c%d:%d:%d %c", lon_d, lon_m, lon_s,
	    ( lon_d > 0 )? 'W' : 'E', TOKEN_SEPARATOR,
	    lat_d, lat_m, lat_s, ( lat_d > 0 )? 'N' : 'S' );
   return buf;
}

const GeoPoint& GeoPoint::
dump( void ) const
{
   ACE_TRACE( "(GeoPoint::dump)" );
#ifndef ACE_NDEBUG
   int a_lon = abs( lon );
   int a_lat = abs( lat );
   int lon_d = a_lon / 3600;
   int lon_m = ( a_lon % 3600 ) / 60;
   int lon_s = a_lon % 60;
   int lat_d = a_lat / 3600;
   int lat_m = ( a_lat % 3600 ) / 60;
   int lat_s = a_lat % 60;
   ACE_DEBUG(( LM_DEBUG, "lon:[%c%d:%d:%d], lat:[%c%d:%d:%d], alt:%d\n",
	       lon < 0 ? '-' : '+', lon_d, lon_m, lon_s,
	       lat < 0 ? '-' : '+', lat_d, lat_m, lat_s, alt ));
#endif
   return *this;
}
