/* $Id: Global.h,v 1.16 1998/07/31 15:57:03 remek Exp $ */
/*
 * File:       Global.h
 * Author:     Remek Trzaska <remek@npac.syr.edu>
 * Created:    long time ago
 * Revision:   $Revision: 1.16 $
 * Updated:    $Date: 1998/07/31 15:57:03 $
 * Copyright 1996/97 NPAC, Syracuse University
 *
 */

#ifndef __GLOBAL_H
#define __GLOBAL_H

#undef DEBUG

#include <stdio.h>
#include <stdlib.h>
#include "Timer.h"
#include "Comm.h"

#define COOR int
#define CARD int
#define ELEV float

#define boolean int
#define TRUE 1
#define FALSE 0

const float sampleDistance = 1206/120.0;  //1 sample

void fatal(char *m, char *m1=NULL);

#ifdef DEBUG
void info(const char *m, long v=-1);
#else
#define info(__dummy1, __dummy2) { }
#endif //DEBUG

#define NewVector(v, n, type) do { if((v = new type[n])==NULL) fatal("Cannot allocate memory!"); } while ( 0 )

#define DeleteVector(v) do { delete[] v; v = NULL; } while ( 0 )

#if defined (ACE_NDEBUG)
#define NewMatrix(matr_,ysize_,xsize_,type_) {type_ *ptr_;\
  int j_;\
  matr_= new type_*[(ysize_)*sizeof(type_ *)];\
  ptr_= new type_[(ysize_)*(xsize_)*sizeof(type_)];\
  if(ptr_==NULL){\
    fprintf(stderr, "Cannot allocate memory\n"); exit(1);\
  }\
  for(j_=0;j_<(ysize_);j_++,ptr_+=(xsize_)) matr_[j_]=ptr_;}

#define DeleteMatrix(matr_) { delete[] *matr_;delete[] matr_;(matr_)=NULL; }

#else

#define NewMatrix(matr_,ysize_,xsize_,type_) {type_ *ptr_;\
  int j_;\
  matr_= new type_*[(ysize_)*sizeof(type_ *)];\
  ptr_= new type_[(ysize_)*(xsize_)*sizeof(type_)];\
  if(ptr_==NULL){\
    fprintf(stderr, "Cannot allocate memory\n"); exit(1);\
  }\
  int __ace_error = errno; \
  ACE_Log_Msg *ace___ = ACE_Log_Msg::instance (); \
  ace___->set( __FILE__, __LINE__, 0, __ace_error, ace___->restart(), ace___->msg_ostream() ); \
  ace___->log( LM_DEBUG, "[%N:%l] NEW_MTX: %s=0x%x\n", #matr_, matr_ ) ; \
  for(j_=0;j_<(ysize_);j_++,ptr_+=(xsize_)) matr_[j_]=ptr_;}

#define DeleteMatrix(matr_) {\
   int __ace_error = errno; \
   ACE_Log_Msg *ace___ = ACE_Log_Msg::instance (); \
   ace___->set( __FILE__, __LINE__, 0, __ace_error, ace___->restart(), ace___->msg_ostream() ); \
   ace___->log( LM_DEBUG, "[%N:%l] DEL_MTX: %s=0x%x\n", #matr_, matr_ ) ; \
   delete[] *matr_;delete[] matr_;(matr_)=NULL;}

#endif

#endif
