/* $Id: Message.h,v 1.11 1997/09/25 20:31:50 remek Exp $ */
/*
 * File:       Message.h
 * Author:     Pawel Roman (proman@npac.syr.edu)
 * Created:    long time ago
 * Revision:   $Revision: 1.11 $
 * Updated:    $Date: 1997/09/25 20:31:50 $
 * Copyright 1996/7 NPAC, Syracuse University
 *
 */

#ifndef _MESSAGE_H
#define _MESSAGE_H

#include "Comm.h"

template<class HEADER>
class Message
{
   public:
      typedef HEADER Header;
      typedef HEADER::SAP_TYPE SAP;
      
      enum MRESULT { OK_RES, END_RES, FAIL_RES };
      
      Message( const Header& hd );
      virtual ~Message( void ) {}
      virtual MRESULT read( const SAP&, const Header& )=0;
      virtual MRESULT write( const SAP& )=0;
      virtual MRESULT set( const void* )=0;
      virtual void execute( void )=0;
      operator const void* ( void ) const;

      static MRESULT readMsg( const SAP& sap,
			      char*& message,
			      const HEADER::LengthType& msg_len );

      static void decode( char*& message,
			  const HEADER::LengthType& msg_len );

   protected:
      Header header;
      int intStatus;
};

template<class HEADER>
inline Message<HEADER>::
Message( const HEADER& hd )
   : header( hd )
{
   ACE_TRACE( "(Message::Message)" );
}

template<class HEADER>
inline Message<HEADER>::
operator const void* ( void ) const
{
   ACE_TRACE( "(Message::operator const void*)" );
   return ( const void* )intStatus;
}

template<class HEADER>
inline Message<HEADER>::MRESULT Message<HEADER>::
readMsg( const SAP& sap, char*& message, const HEADER::LengthType& msg_len )
{
   ACE_TRACE( "(Message::readMsg)" );
   TRACE_NEW_2( message, char[msg_len+1] );
   if( ! message )
      ACE_ERROR_RETURN(( LM_ERROR, "[!](Message::readMsg) No memory for %d bytes\n",
			 msg_len+1 ), FAIL_RES );
   
   if( sap.recv_n( message, msg_len ) != msg_len )
      ACE_ERROR_RETURN(( LM_ERROR, "[!](Message::readMsg) Connection closed\n" ),
		       END_RES );

   return OK_RES;
}

template<class HEADER>
inline void Message<HEADER>::
decode( char*& message, const HEADER::LengthType& msg_len )
{
   ACE_TRACE( "(Message::decode)" );
   message[msg_len] = '\0';
   ACE_DEBUG(( LM_DEBUG, "[pre] message: %s\n", message ));
   for( int offset = 0; offset < msg_len; offset++ )
      if( message[offset] == ',' )
	 message[offset] = ' ';
   ACE_DEBUG(( LM_DEBUG, "[post] message: %s\n", message ));
}


#endif
