/* $Id: MsgHeader.h,v 1.12 1998/04/29 23:00:44 proman Exp $ */
/*
 * File:       MsgHeader.h
 * Author:     Pawel Roman (proman@npac.syr.edu)
 * Created:    long time ago
 * Revision:   $Revision: 1.12 $
 * Updated:    $Date: 1998/04/29 23:00:44 $
 * Copyright 1996/7 NPAC, Syracuse University
 *
 */

#ifndef _MSGHEADER_H
#define _MSGHEADER_H

#include "Comm.h"

template<class SAP>
class MsgHeader
{
   public:
      enum ChannelType { CONTROL_CHANNEL = 30, DATA_CHANNEL = 31 };
      typedef int LengthType;
      typedef unsigned char MsgID;
      typedef unsigned char VID;
      
      typedef SAP SAP_TYPE;

      enum { MSG_HEADER_SIZE=10 };
      enum { BROADCAST=255 };
      enum { IS_MASTER = 1, MASTER_INDEED = 10 };
      enum LINK_STATUS { LINK_OK, LINK_BROKEN, LINK_TIMEOUT };
      
      MsgHeader( const MsgID&,
		 const VID& = 0,
		 const LengthType& = 0,
		 const ChannelType& = DATA_CHANNEL );
      
      const ChannelType& channel( void ) const;
      const LengthType& length( void ) const;
      const MsgID& msgID( void ) const;
      const VID& vID( void ) const;
      VID& vID( void );
      void setVID( const VID& );
      void setLength( const LengthType& );
      void adjustLength( void );

      LINK_STATUS read( const SAP& );
      LINK_STATUS write( const SAP& );
      char* export( void );
      size_t size( void );
      
   private:
      char header_buf[MSG_HEADER_SIZE];
};      

template<class SAP>
inline MsgHeader<SAP>::
MsgHeader( const MsgID& id, const VID& vid, const LengthType& len, const ChannelType& ch )
{
   ACE_TRACE( "(MsgHeader::MsgHeader)" );
   *( ChannelType* )header_buf = ch;
   *( LengthType* )( header_buf + sizeof ch ) = len;
   *( MsgID* )( header_buf + sizeof ch + sizeof len ) = id;
   *( VID* )( header_buf + sizeof ch + sizeof len + sizeof id ) = vid;
}

template<class SAP>
inline MsgHeader<SAP>::LINK_STATUS MsgHeader<SAP>::
read( const SAP& sap )
{
   ACE_TRACE( "(MsgHeader::read)" );
   if( sap.recv_n( header_buf, sizeof header_buf ) == sizeof header_buf )
   {
      *( LengthType* )( header_buf + sizeof( ChannelType ) ) -=
	 sizeof( MsgID ) + sizeof( VID );

      ACE_HEX_DUMP(( LM_DEBUG, header_buf, sizeof header_buf, "header" ));
      return LINK_OK;
   }
   else
      ACE_ERROR_RETURN(( LM_ERROR, "[!](MsgHeader::read) Connection closed\n" ),
		       LINK_BROKEN );
}

template<class SAP>
inline MsgHeader<SAP>::LINK_STATUS MsgHeader<SAP>::
write( const SAP& sap )
{
   ACE_TRACE( "(MsgHeader::write)" );

   *( LengthType* )( header_buf + sizeof( ChannelType ) ) +=
      sizeof( MsgID ) + sizeof( VID );
      
   if( sap.send_n( header_buf, sizeof header_buf ) == sizeof header_buf )
   {
      *( LengthType* )( header_buf + sizeof( ChannelType ) ) -=
	 sizeof( MsgID ) + sizeof( VID );

      ACE_HEX_DUMP(( LM_DEBUG, header_buf, sizeof header_buf, "header" ));
      return LINK_OK;
   }
   else
   {
      *( LengthType* )( header_buf + sizeof( ChannelType ) ) -=
	 sizeof( MsgID ) + sizeof( VID );
      
      ACE_ERROR_RETURN(( LM_ERROR, "[!](MsgHeader::write) Connection closed\n" ),
		       LINK_BROKEN );
   }
}

template<class SAP>
inline const MsgHeader<SAP>::ChannelType& MsgHeader<SAP>::
channel( void ) const
{
   ACE_TRACE( "(MsgHeader::channel)" );
   ACE_DEBUG(( LM_DEBUG, "channel: %d\n",
	       *( const ChannelType* )header_buf ));

   return *( const ChannelType* )header_buf;
}

template<class SAP>
inline void MsgHeader<SAP>::
setLength( const LengthType& len )
{
   ACE_TRACE( "(MsgHeader::setLength)" );
   *( LengthType* )( header_buf + sizeof( ChannelType ) ) = len;
   ACE_DEBUG(( LM_DEBUG, "length: %d\n",
	       *( const LengthType* )( header_buf + sizeof( ChannelType ) ) ));
}

template<class SAP>
inline const MsgHeader<SAP>::LengthType& MsgHeader<SAP>::
length( void ) const
{
   ACE_TRACE( "(MsgHeader::length)" );
   ACE_DEBUG(( LM_DEBUG, "length: %d\n",
	       *( const LengthType* )( header_buf + sizeof( ChannelType ) ) ));

   return *( const LengthType* )( header_buf + sizeof( ChannelType ) );
}

template<class SAP>
inline const MsgHeader<SAP>::MsgID& MsgHeader<SAP>::
msgID( void ) const
{
   ACE_TRACE( "(MsgHeader::msgID)" );
   ACE_DEBUG(( LM_DEBUG, "msgID: %d\n", *( const MsgID* )( header_buf +
							   sizeof( ChannelType ) +
							   sizeof( LengthType ) ) ));
   
   return *( const MsgID* )( header_buf + sizeof( ChannelType ) + sizeof( LengthType ) );
}

template<class SAP>
inline const MsgHeader<SAP>::VID& MsgHeader<SAP>::
vID( void ) const
{
   ACE_TRACE( "(MsgHeader::vID)" );
   ACE_DEBUG(( LM_DEBUG, "vid: %d\n", *( const VID* )( header_buf +
						       sizeof( ChannelType ) +
						       sizeof( LengthType ) +
						       sizeof( MsgID ) ) ));
   
   return *( const VID* )( header_buf +
			   sizeof( ChannelType ) +
			   sizeof( LengthType ) +
			   sizeof( MsgID ) );
}

template<class SAP>
inline MsgHeader<SAP>::VID& MsgHeader<SAP>::
vID( void )
{
   ACE_TRACE( "(MsgHeader::vID)" );
   ACE_DEBUG(( LM_DEBUG, "vid: %d\n", *( const VID* )( header_buf +
						       sizeof( ChannelType ) +
						       sizeof( LengthType ) +
						       sizeof( MsgID ) ) ));
   
   return *( VID* )( header_buf +
			   sizeof( ChannelType ) +
			   sizeof( LengthType ) +
			   sizeof( MsgID ) );
}

template<class SAP>
inline void MsgHeader<SAP>::
setVID( const VID& vid )
{
   ACE_TRACE( "(MsgHeader::setVID)" );
   *( VID* )( header_buf +
	      sizeof( ChannelType ) +
	      sizeof( LengthType ) +
	      sizeof( MsgID ) ) = vid;
   
   ACE_DEBUG(( LM_DEBUG, "vid: %d\n", *( const VID* )( header_buf +
						       sizeof( ChannelType ) +
						       sizeof( LengthType ) +
						       sizeof( MsgID ) ) ));
}

template<class SAP>
inline char* MsgHeader<SAP>::
export( void )
{
   return header_buf;
}

template<class SAP>
inline size_t MsgHeader<SAP>::
size( void )
{
   return sizeof header_buf;
}


template<class SAP>
inline void MsgHeader<SAP>::
adjustLength( void )
{
   ACE_TRACE( "(MsgHeader::adjustLength)" );
   *( LengthType* )( header_buf + sizeof( ChannelType ) ) -=
     sizeof( MsgID ) + sizeof( VID );
}

#endif
