/* $Id: ProtocolInterface.h,v 1.14 1998/04/29 23:00:45 proman Exp $ */
/*
 * File:       ProtocolInterface.h
 * Author:     Pawel Roman (proman@npac.syr.edu)
 * Created:    long time ago
 * Revision:   $Revision: 1.14 $
 * Updated:    $Date: 1998/04/29 23:00:45 $
 * Copyright 1996/7 NPAC, Syracuse University
 *
 */

#ifndef _PROTOCOLINTERFACE_H
#define _PROTOCOLINTERFACE_H

template<class MESSAGE, class CONT>
class ProtocolInterface
{
   public: 
      typedef MESSAGE PMessage;
      typedef MESSAGE::SAP SAP;
      typedef MESSAGE::Header PHeader;
      typedef MESSAGE::Header::VID PVID;
      typedef MESSAGE::Header::MsgID PMsgID;
      typedef MESSAGE::Header::LengthType PLengthType;
      
      enum PMODE { P_MASTER, P_SLAVE, P_PEER };
      enum PSTATUS { P_OK, P_CLOSED, P_TIMEOUT, P_UNKNOWN, P_ECONT, P_NOTHANDLED };

      typedef PSTATUS MessageHandler( SAP&, PHeader&, void* );
      
      ProtocolInterface( const SAP& a_sap, int cont_size );
      ~ProtocolInterface( void ) {}
      PSTATUS addMsgType( const PMsgID& mid, MESSAGE* msg );
      PSTATUS removeMsgType( const PMsgID& mid );
      PSTATUS processAllMsgs( void );
      PSTATUS processMsg( void );
      PSTATUS processAllMsgs( const PVID& vid,
			      MessageHandler msgHandler,
			      void* handlerData = 0 );
      PSTATUS processMsg( const PVID& vid,
			  MessageHandler msgHandler,
			  void* handlerData = 0 );
      PSTATUS sendMsg( const PMsgID& mid );
      PSTATUS sendMsg( const PMsgID& mid, const PVID& vid );
      PSTATUS getMsgType( const PMsgID& mid, MESSAGE*& msg_type );
      PMODE askMode( void );
      PMODE getMode( void );
      SAP& getSAP( void ) { return sap; }
      const SAP& getSAP( void ) const { return sap; }
      
   protected:
      PSTATUS trashMsg( const PLengthType& msg_len );
      
      CONT msgTypeCont;
      PHeader header;
      SAP sap;
      PSTATUS link_status;
};

template<class MESSAGE, class CONT>
inline ProtocolInterface<MESSAGE, CONT>::
ProtocolInterface( const SAP& a_sap, int cont_size )
   : sap( a_sap ), msgTypeCont( cont_size, 0 ),
     header( PHeader::IS_MASTER, 0, 0, PHeader::CONTROL_CHANNEL )

{
   ACE_TRACE( "(ProtocolInterface::ProtocolInterface(const SAP&, int))" );
   ACE_DEBUG(( LM_DEBUG, "header size: %d\n", sizeof header ));
}

template<class MESSAGE, class CONT>
inline ProtocolInterface<MESSAGE, CONT>::PSTATUS
ProtocolInterface<MESSAGE, CONT>::
addMsgType( const PMsgID& mid, MESSAGE* msg )
{
   ACE_TRACE( "(ProtocolInterface::addMsgType)" );
   msgTypeCont.addItem( mid, msg );
   if( ! msgTypeCont )
      ACE_ERROR_RETURN(( LM_ERROR,
			 "[!](ProtocolInterface::addMsgType) %d : container error\n", mid ),
		       P_ECONT );
   
   return P_OK;
}

template<class MESSAGE, class CONT>
inline ProtocolInterface<MESSAGE, CONT>::PSTATUS
ProtocolInterface<MESSAGE, CONT>::
removeMsgType( const PMsgID& mid )
{
   ACE_TRACE( "(ProtocolInterface::removeMsgType)" );
   MESSAGE* removed = msgTypeCont.removeItem( mid );
   if( ! msgTypeCont )
      ACE_ERROR_RETURN(( LM_ERROR,
			 "[!](ProtocolInterface::addMsgType) %d : container error\n", mid ),
		       P_ECONT );
   TRACE_DELETE( removed );
   return P_OK;
}

template<class MESSAGE, class CONT>
inline ProtocolInterface<MESSAGE, CONT>::PSTATUS
ProtocolInterface<MESSAGE, CONT>::
getMsgType( const PMsgID& mid, MESSAGE*& msg_type )
{
   ACE_TRACE( "(ProtocolInterface::getMsgType)" );
   msg_type = msgTypeCont[mid];
   if( msgTypeCont )
      return P_OK;
   
   ACE_HEX_DUMP(( LM_DEBUG, header.export(), header.size(),  "Unknown header" ));
   ACE_ERROR_RETURN(( LM_ERROR, "[!](ProtocolInterface::getMsgType) Protocol violation\n" ),
		    P_UNKNOWN );
}

template<class MESSAGE, class CONT>
inline ProtocolInterface<MESSAGE, CONT>::PSTATUS
ProtocolInterface<MESSAGE, CONT>::
processAllMsgs( void )
{
   ACE_TRACE( "(ProtocolInterface::processAllMsgs)" );
   static char test_char;
#ifndef ACE_NLOGGING
   int msg_counter = 0;
#endif
   do
   {
      if( ( link_status = processMsg() ) != P_OK )
	 break;

#ifndef ACE_NLOGGING
      msg_counter++;
#endif
   }
   while( sap.recv( &test_char, sizeof test_char, MSG_PEEK, &ACE_Time_Value::zero ) == 1 );
   ACE_DEBUG(( LM_DEBUG, "msg counter: %d\n", msg_counter ));
   return link_status;
}

template<class MESSAGE, class CONT>
inline ProtocolInterface<MESSAGE, CONT>::PSTATUS
ProtocolInterface<MESSAGE, CONT>::
processMsg( void )
{
   ACE_TRACE( "(ProtocolInterface::processMsg)" );
   static MESSAGE* msg_type;
   if( header.read( sap ) == PHeader::LINK_OK )
   {
      msg_type = msgTypeCont[header.msgID()];
      if( msgTypeCont )
      {
	 if( msg_type->read( sap, header ) == PMessage::OK_RES )
	 {
	    msg_type->execute();
	    return P_OK;
	 }
	 else
	    return P_CLOSED;
      }
      else
	 return trashMsg( header.length() );
   }
   else
      return P_CLOSED;
}

template<class MESSAGE, class CONT>
inline ProtocolInterface<MESSAGE, CONT>::PSTATUS
ProtocolInterface<MESSAGE, CONT>::
processAllMsgs( const PVID& vid, MessageHandler msgHandler, void* handlerData )
{
   ACE_TRACE( "(ProtocolInterface::processAllMsgs)" );
   static char test_char;
#ifndef ACE_NLOGGING
   int msg_counter = 0;
#endif
   do
   {
      if( ( link_status = processMsg( vid, msgHandler, handlerData ) ) != P_OK )
	 break;
#ifndef ACE_NLOGGING
      msg_counter++;
#endif
   }
   while( sap.recv( &test_char, sizeof test_char, MSG_PEEK, &ACE_Time_Value::zero ) == 1 );
   ACE_DEBUG(( LM_DEBUG, "msg counter: %d\n", msg_counter ));
   return link_status;
}

template<class MESSAGE, class CONT>
inline ProtocolInterface<MESSAGE, CONT>::PSTATUS
ProtocolInterface<MESSAGE, CONT>::
processMsg( const PVID& vid, MessageHandler msgHandler, void* handlerData )
{
  ACE_TRACE( "(ProtocolInterface::processMsg)" );
  static MESSAGE* msg_type;
  // WARNING! ugly, but quick hack
  if( sap.recv_n( header.export(), header.size() - 1 ) == header.size() - 1 )
    if( ( header.channel() == PHeader::CONTROL_CHANNEL ) )
      {
	if( header.msgID() == 122 /*FOCUS*/ )
	  {
	    // Focus that window
	    msg_type = msgTypeCont[FOCUS];
	    msg_type->execute();
	  }
	else
	  {
	    if( ( header.length() - 1  ) > 0 )
	      return trashMsg( header.length() - 1 );
	  }
	return P_OK;
      }
    else
      {
	sap.recv_n( &header.vID(), sizeof header.vID() );
	header.adjustLength();
	//   if( header.read( sap ) == PHeader::LINK_OK )
	//   {
	if( header.vID() == vid )
	  {
	    msg_type = msgTypeCont[header.msgID()];
	    if( msgTypeCont )
	      {
		if( msg_type->read( sap, header ) == PMessage::OK_RES )
		  {
		    msg_type->execute();
		    return P_OK;
		  }
		else
		  return P_CLOSED;
	      }
	    else
	      return trashMsg( header.length() );
	  }
	else
	  return msgHandler( sap, header, handlerData );
      }
  else
    return P_CLOSED;
}

template<class MESSAGE, class CONT>
inline ProtocolInterface<MESSAGE, CONT>::PSTATUS
ProtocolInterface<MESSAGE, CONT>::
sendMsg( const PMsgID& mid )
{
   ACE_TRACE( "(ProtocolInterface::sendMsg)" );
   MESSAGE* msgType = msgTypeCont[mid];
   if( msgTypeCont )
      msgType->write( sap );
   else
      ACE_ERROR(( LM_ERROR,
		  "[!](ProtocolInterface::sendMsg) Message: %d doesn't exist\n", mid ));
      
   return P_OK;
}

template<class MESSAGE, class CONT>
inline ProtocolInterface<MESSAGE, CONT>::PSTATUS
ProtocolInterface<MESSAGE, CONT>::
sendMsg( const PMsgID& mid, const PVID& vid )
{
   ACE_TRACE( "(ProtocolInterface::sendMsg2)" );
   MESSAGE* msgType = msgTypeCont[mid];
   if( msgTypeCont )
   {
      PVID old_vid = msgType->setVID( vid );
      msgType->write( sap );
      msgType->setVID( old_vid );
   }
   else
      ACE_ERROR(( LM_ERROR,
		  "[!](ProtocolInterface::sendMsg) Message: %d doesn't exist\n", mid ));
   
   return P_OK;
}

template<class MESSAGE, class CONT>
inline ProtocolInterface<MESSAGE, CONT>::PSTATUS
ProtocolInterface<MESSAGE, CONT>::
trashMsg( const PLengthType& msg_len )
{
   ACE_TRACE( "(ProtocolInterface::trashMsg)" );
   const ACE_Time_Value timeout = 10;
   const unsigned short BUF_LEN = 64;
   char trash_buf[BUF_LEN];

   ACE_DEBUG(( LM_DEBUG, "%d * %d + %d\n", BUF_LEN, msg_len / BUF_LEN, msg_len % BUF_LEN ));
   for( PLengthType laps = msg_len / BUF_LEN; laps > 0; laps--)
      if( sap.recv_n( trash_buf, BUF_LEN, 0, &timeout ) != BUF_LEN )
	 ACE_ERROR_RETURN(( LM_ERROR, "[!](ProtocolInterface::trashMsg) Thrashing timeout\n" ),
			  P_TIMEOUT );

   if( sap.recv_n( trash_buf, msg_len % BUF_LEN, 0, &timeout ) != msg_len % BUF_LEN )
      ACE_ERROR_RETURN(( LM_ERROR, "[!](ProtocolInterface::trashMsg) Thrashing timeout\n" ),
		       P_TIMEOUT );
   else
      return P_OK;
}

template<class MESSAGE, class CONT>
inline ProtocolInterface<MESSAGE, CONT>::PMODE
ProtocolInterface<MESSAGE, CONT>::
askMode( void )
{
   ACE_TRACE( "(ProtocolInterface::askMode)" );
   if( header.write( sap ) == PHeader::LINK_OK )
      for( ; ; )
	 if( header.read( sap ) == PHeader::LINK_OK )
	    if( header.msgID() == PHeader::IS_MASTER )
	       if( header.vID() == PHeader::MASTER_INDEED )
		  return P_MASTER;
	       else
		  return P_SLAVE;
	    else
	       trashMsg( header.length() );
	 else
	    ACE_ERROR_RETURN(( LM_ERROR,
			       "[!](ProtocolInterface::askMode) header.read error\n" ),
			     P_SLAVE );
   else
      ACE_ERROR_RETURN(( LM_ERROR,
			 "[!](ProtocolInterface::askMode) header.write error\n" ),
		       P_SLAVE );
}

template<class MESSAGE, class CONT>
inline ProtocolInterface<MESSAGE, CONT>::PMODE
ProtocolInterface<MESSAGE, CONT>::
getMode( void )
{
   ACE_TRACE( "(ProtocolInterface::getMode)" );
   for( ; ; )
     if( header.read( sap ) == PHeader::LINK_OK )
       if( header.msgID() == PHeader::IS_MASTER )
	 if( header.vID() == PHeader::MASTER_INDEED )
	   return P_MASTER;
	 else
	   return P_SLAVE;
       else
	 trashMsg( header.length() );
     else
       ACE_ERROR_RETURN(( LM_ERROR,
			  "[!](ProtocolInterface::getMode) header.read error\n" ),
			P_SLAVE );
}

#endif
