/* $Id: RO3D.java,v 1.3 1997/09/25 20:31:52 remek Exp $ */
/*
 * File         : RO3D.java
 * Author       : Pawel Roman
 * Created      : Wed Aug 13 14:56:04 1997
 * Revision     : $Revision: 1.3 $
 * Updated      : $Date: 1997/09/25 20:31:52 $
 * Copyright: Northeast Parallel Architectures Center
 *            at Syracuse University 1997
 */

package stubs;

import consts.Const;
import messages.AppEventMessage;
import tango.simulator.TangoSimModule;
import tango.simulator.SimEvent;
import tango.simulator.SimEventConsumer;
import java.io.*;

public class RO3D extends TangoSimModule
{
  public static final byte NEWGROUND   = 10;
  public static final byte NEWAIRBORNE = 11;
  public static final byte ABUPDATE    = 12;
  public static final byte ABMOVE      = 13;
  public static final byte ABSHOW      = 14;
  public static final byte ABHIDE      = 15;
  public static final byte PAUSE       = 16;
  public static final byte RESUME      = 17;
  public static final byte CSUPDATE    = 18;
  
  public static final byte BASE        = 0;
  public static final byte SHIP        = 1;
  public static final byte RADAR       = 2;
  public static final byte SEMO        = 3;
  
  public static final byte AIRPLANE    = 0;
  public static final byte CHOPPER     = 1;
  
  public static final byte FRIEND      = 0;
  public static final byte FOE         = 1;
  public static final byte UFO         = 2;

  public static final byte OUTOFPROTO  = ( byte )255;
  public static final byte BROADCAST   = ( byte )255;

  public RO3D()
    {
      GisConst.init();
    }

  public void receive(messages.AppEventMessage m){};
  public void init(String host,int port,SimEventConsumer sec)
    {
      try
	{
	  start(host,port,53,"RO3D");
	}
      catch(IOException e)
	{
	  System.err.println(e);
	  e.printStackTrace(System.err);
	}
    }

  public void processSimEvent(SimEvent evt)
    {
      ByteArrayOutputStream bos=new ByteArrayOutputStream();
      DataOutputStream dos=new DataOutputStream( bos );
      try
	{
	  Location l;
	  String name;
	  String description;
	  System.out.println( "RO3D:"+evt.contents );
	  Tokenizer t = new Tokenizer( evt.contents );
	  int command=( ( Integer )( GisConst.hashtable.get( t.nextToken() ) ) ).intValue();
	  
	  switch(command)
	    {
	    case GisConst.PAUSE:
	      dos.writeByte( PAUSE );
	      dos.writeByte( BROADCAST );
	      break;

	    case GisConst.RESUME:
	      dos.writeByte( RESUME );
	      dos.writeByte( BROADCAST );
	      break;

	    case GisConst.NEWBASE:
	      name = t.nextToken();
	      l = new Location( t.nextToken() );
	      description = t.rest();
	      
	      dos.writeByte( NEWGROUND );
	      dos.writeByte( BROADCAST );
	      dos.writeByte( BASE );
	      dos.writeBytes( name );
	      dos.writeByte( 0 );
	      dos.writeDouble( -l.x );
	      dos.writeDouble( l.y );
	      dos.writeBytes( description );
	      dos.writeByte( 0 );
	      break;

	    case GisConst.NEWSHIP:
	      name = t.nextToken();
	      l = new Location( t.nextToken() );
	      description = t.rest();
	      
	      dos.writeByte( NEWGROUND );
	      dos.writeByte( BROADCAST );
	      dos.writeByte( SHIP );
	      dos.writeBytes( name );
	      dos.writeByte( 0 );
	      dos.writeDouble( -l.x );
	      dos.writeDouble( l.y );
	      dos.writeBytes( description );
	      dos.writeByte( 0 );
	      break;

	    case GisConst.NEWRADAR:
	      name = t.nextToken();
	      l = new Location( t.nextToken() );
	      description = t.rest();
	      
	      dos.writeByte( NEWGROUND );
	      dos.writeByte( BROADCAST );
	      dos.writeByte( RADAR );
	      dos.writeBytes( name );
	      dos.writeByte( 0 );
	      dos.writeDouble( -l.x );
	      dos.writeDouble( l.y );
	      dos.writeBytes( description );
	      dos.writeByte( 0 );
	      break;

	    case GisConst.NEWSEMO:
	      name = t.nextToken();
	      l = new Location( t.nextToken() );
	      description = t.rest();
	      
	      dos.writeByte( NEWGROUND );
	      dos.writeByte( BROADCAST );
	      dos.writeByte( SEMO );
	      dos.writeBytes( name );
	      dos.writeByte( 0 );
	      dos.writeDouble( -l.x );
	      dos.writeDouble( l.y );
	      dos.writeBytes( description );
	      dos.writeByte( 0 );
	      break;

	    case GisConst.NEWPLANE:
	      dos.writeByte( NEWAIRBORNE );
	      dos.writeByte( BROADCAST );
	      name = t.nextToken();

	      if( t.nextToken().trim().equalsIgnoreCase( "plane" ) )
		dos.writeByte( AIRPLANE );
	      else
		dos.writeByte( CHOPPER );

	      dos.writeBytes( name );
	      dos.writeByte( 0 );
	      description = t.rest();
	      dos.writeBytes( description );
	      dos.writeByte( 0 );
	      break;
	      
	    case GisConst.PLANE:
	      name = t.nextToken();
	      int cmd = ( ( Integer )( GisConst.hashtable.
				       get( "plane_"+t.nextToken().trim() ) ) ).intValue();
	      switch( cmd )
		{
		case GisConst.PLANE_UPDATE:
		  boolean clsUpdate = false;
		  String s = t.nextToken().trim();
		  try
		    {
		      l = new Location( s );
		      s = t.nextToken().trim();
		    }
		  catch( Exception e )
		    {
		      l = new Location( Math.PI, -Math.PI );
		      clsUpdate = true;
		    }
		  double altitude;
		  try
		    {
		      altitude = ( new Double( s ) ).doubleValue();
		      s = t.nextToken();
		    }
		  catch( Exception e )
		    {
		      altitude = -1;
		    }

		  byte classif;
		  if( s.equalsIgnoreCase( "friend" ) )
		    classif = FRIEND;
		  else if( s.equalsIgnoreCase( "foe" ) )
		    classif = FOE;
		  else
		    classif = UFO;
		  
		  if( clsUpdate )
		    dos.writeByte( CSUPDATE );
		  else
		    dos.writeByte( ABUPDATE );

		  dos.writeByte( BROADCAST );
		  dos.writeByte( classif );
		  dos.writeBytes( name );
		  dos.writeByte( 0 );

		  if( ! clsUpdate )
		    {
		      dos.writeDouble( -l.x );
		      dos.writeDouble( l.y );
		      dos.writeDouble( altitude );
		    }
		  break;

		case GisConst.PLANE_SHOW:
		  dos.writeByte( ABSHOW );
		  dos.writeByte( BROADCAST );
		  dos.writeBytes( name );
		  dos.writeByte( 0 );
		  break;
		  
		case GisConst.PLANE_HIDE:
		  dos.writeByte( ABHIDE );
		  dos.writeByte( BROADCAST );
		  dos.writeBytes( name );
		  dos.writeByte( 0 );
		  break;

		case GisConst.PLANE_MOVE:
		  dos.writeByte( ABMOVE );
		  dos.writeByte( BROADCAST );
		  dos.writeBytes( name );
		  dos.writeByte( 0 );
		  l = new Location( t.nextToken() );
		  dos.writeDouble( -l.x );
		  dos.writeDouble( l.y );
		  l = new Location( t.nextToken() );
		  dos.writeDouble( -l.x );
		  dos.writeDouble( l.y );
		  double time = ( new Double( t.nextToken() ) ).doubleValue();
		  dos.writeDouble( time );
		  break;

		default:
		  dos.writeByte( OUTOFPROTO );
		  dos.writeByte( BROADCAST );
		  break;
		}
	      break;
	      
	    default:
	      dos.writeByte( OUTOFPROTO );
	      dos.writeByte( BROADCAST );
	      break;
	    }
	  send( new AppEventMessage( Const.DATA, bos.toByteArray() ) );
	}
      catch( Exception e )
	{
	  System.out.println( e );
	  e.printStackTrace( System.out );
	}
    }
  
  public boolean accepts(String id)
    {
      return id!=null && (id.equals("RO3D") || id.equals("pause") || id.equals("resume"));
    }
    
}
