/* $Id: SimEngMessage.h,v 1.2 1997/09/25 20:31:53 remek Exp $ */
/*
 * File         : SimEngMessage.h
 * Author       : Pawel Roman
 * Created      : Wed Aug 20 13:40:08 1997
 * Revision     : $Revision: 1.2 $
 * Updated      : $Date: 1997/09/25 20:31:53 $
 * Copyright: Northeast Parallel Architectures Center
 *            at Syracuse University 1997
 */

#ifndef __SIMENGMESSAGE_H
#define __SIMENGMESSAGE_H

#include "ViewerMessage.h"
#include <ace/Message_Block.h>

template<class HEADER>
class SimEngMessage
   : public ViewerMessage<HEADER>
{
   public:
      SimEngMessage( const HEADER& hd, SoXtWebViewer* vw );
      MRESULT read( const SAP& sap, const Header& header );
      MRESULT write( const SAP& sap );
      MRESULT set( const void* params = NULL );

   protected:
      ACE_Message_Block* mb;
};

template<class HEADER>
inline SimEngMessage<HEADER>::
SimEngMessage( const HEADER& hd, SoXtWebViewer* vw )
   : ViewerMessage<HEADER>( hd, vw ), mb( NULL )
{ 
}


template<class HEADER>
inline SimEngMessage<HEADER>::MRESULT SimEngMessage<HEADER>::
read( const SAP& sap, const Header& header )
{
   ACE_TRACE( "(SimEngMessage::read)" );
   if( mb )
      mb->release();

   Header::LengthType msg_len = header.length();
   TRACE_NEW_2( mb, ACE_Message_Block( msg_len ) );
   if( mb )
      if( sap.recv_n( mb->base(), msg_len ) == msg_len )
      {
	 checkVID( header );
	 return OK_RES;
      }
      else
	 return END_RES;
   else
      return FAIL_RES;
}

template<class HEADER>
inline SimEngMessage<HEADER>::MRESULT SimEngMessage<HEADER>::
write( const SAP& sap ) 
{ 
   ACE_TRACE( "(SimEngMessage::write)" );
   if( mb )
   {
      header.setLength( mb->size() );
      if( header.write( sap ) == Header::LINK_OK )
	 if( sap.send_n( mb->base(), mb->size() ) == header.length() )
	    return OK_RES;
      
      return END_RES;
   }
   else
      return FAIL_RES;
}

template<class HEADER>
inline SimEngMessage<HEADER>::MRESULT SimEngMessage<HEADER>::
set( const void* msg ) 
{ 
   ACE_TRACE( "(SimEngMessage::set)" );
   if( mb )
      mb->release();
   
   mb = ( ( const ACE_Message_Block* )msg )->duplicate();
   return OK_RES;
}

#endif

