/* $Id: SoActiveTranslation.h,v 1.7 1997/09/25 20:31:54 remek Exp $ */
/*
 * File         : SoActiveTranslation.h
 * Author       : Pawel Roman
 * Created      : Mon Aug 25 14:27:50 1997
 * Revision     : $Revision: 1.7 $
 * Updated      : $Date: 1997/09/25 20:31:54 $
 * Copyright: Northeast Parallel Architectures Center
 *            at Syracuse University 1997
 */

#ifndef __SOACTIVETRANSLATION_H
#define __SOACTIVETRANSLATION_H

#include "Comm.h"

#include <Inventor/nodes/SoTransform.h>
#include <Inventor/engines/SoOneShot.h>
#include <Inventor/engines/SoInterpolate.h>
#include <math.h>

class SoActiveTranslation
   : public SoTransform
{
   public:
      SoActiveTranslation( void );
      ~SoActiveTranslation( void );
      SoMFVec3f& startPoint( void ) { return inter->input0; }
      SoMFVec3f& endPoint( void ) { return inter->input1; }
      SoSFTime& duration( void )  { return alpha->duration; }
      operator const void*( void ) { return ( const void* )intStatus; }
      void pause( void );
      void resume( void );
      void placeAt( const SbVec3f& pos );
      void setMoveParams( const SbVec3f& from,
			  const SbVec3f& to,
			  const float& time_delta );
      
   private:
      int intStatus;
      SoOneShot* alpha;
      SoInterpolateVec3f* inter;
};

inline void SoActiveTranslation::
setMoveParams( const SbVec3f& from,
	       const SbVec3f& to,
	       const float& time_delta )
{
   ACE_TRACE( "(SoActiveTranslation::setMoveParams)" );
   alpha->disable = TRUE;
   inter->input0.setValue( from );
   inter->input1.setValue( to );
   alpha->duration = time_delta;
   rotation = SbRotation( SbVec3f( 0, 0, -1 ), to - from );
   alpha->disable = FALSE;
   alpha->trigger.touch();
}

inline void SoActiveTranslation::
placeAt( const SbVec3f& pos )
{
   ACE_TRACE( "(SoActiveTranslation::placeAt)" );
   alpha->disable = TRUE;
   inter->input0.setValue( pos );
}

inline void SoActiveTranslation::
pause( void )
{
   ACE_TRACE( "(SoActiveTranslation::pause)" );
   inter->input0 = translation.getValue();
   alpha->duration = alpha->duration.getValue() * ( 1 - inter->alpha.getValue() );
   alpha->disable = TRUE;
}

inline void SoActiveTranslation::
resume( void )
{
   ACE_TRACE( "(SoActiveTranslation::resume)" );
   alpha->disable = FALSE;
   alpha->trigger.touch();
}
      
inline SoActiveTranslation::
~SoActiveTranslation( void )
{
   ACE_TRACE( "(SoActiveTranslation::~SoActiveTranslation)" );
   inter->alpha.disconnect();
   translation.disconnect();
   TRACE_UNREF( alpha );
   TRACE_UNREF( inter );
   ACE_DEBUG(( LM_DEBUG, "this=0x%x, ref=%d\n", this, getRefCount() ));
}

#endif
