/* $Id: SoChunk.h,v 1.15 1997/09/25 20:31:54 remek Exp $ */
/*
 * File:       SoChunk.h
 * Author:     Pawel Roman (proman@npac.syr.edu),
 *             Remek Trzaska (remek@npac.syr.edu)
 * Created:    long time ago
 * Revision:   $Revision: 1.15 $
 * Updated:    $Date: 1997/09/25 20:31:54 $
 * Copyright 1996/7 NPAC, Syracuse University
 *
 */

#ifndef _SOCHUNK_H
#define _SOCHUNK_H

#include <Inventor/nodes/SoSeparator.h>
#include <Inventor/nodes/SoTranslation.h>
#include <Inventor/nodes/SoTextureCoordinatePlane.h>

#include "GeoPoint.h"
#include "SoTerrain.h"
#include "SoURLTexture2.h"
#include "SoXtWebViewer.h"

class SoChunk : public SoSeparator
{
      enum 
      {
	 NORTH = 0x01, SOUTH = 0x02, WEST  = 0x04, EAST  = 0x08
      };
      int textureSlotsIndices[15]; //slots for possibly 9 small textures
      
      friend class SoXtWebViewer;
      
   public:
      SoChunk( const GeoPoint& gp, int dSize, float qual, int tSize,
	       float yS, unsigned char aBoundsTagParsed,
	       float maxWest, float maxNorth, float maxEast, float maxSouth);
      ~SoChunk( void );
      void setTranslation( const SbVec3f& trans );
      void setDEM( const string& loc , int real = TRUE);
      void switchTexTo( const string& loc );

      float maxAlt, minAlt;

   private:
      GeoPoint origin;
      float globalNorth;
      float globalWest;
      float globalEast;
      float globalSouth;

      SoTerrain* dem;

      SoGroup* extensionGroup;
      
      int demSize, texSize;
      float quality;
      float yScale;
      unsigned char boundsTagParsed;

      unsigned int onBoundary; //bitmask 

      void extendDEM();
      void extendTex(const string& loc);
      void addSmallTexture(const string& loc,
			   GeoPoint geoCoord,
			   SoNode* where, int rotate = FALSE);
};

#endif
