/*
 * File         : SoIsosurface.C
 * Author       : Remek Trzaska
 * Created      : Mon Feb  9 13:21:17 1998
 * Revision     : $Revision: 1.4 $
 * Updated      : $Date: 1998/04/23 20:13:36 $
 * Copyright: Northeast Parallel Architectures Center
 *            at Syracuse University 1998
 */

#include "SoIsosurface.h"
#include <Inventor/nodes/SoCoordinate3.h>
#include <Inventor/nodes/SoIndexedTriangleStripSet.h>
#include <Inventor/nodes/SoNormalBinding.h>
#include <Inventor/nodes/SoNormal.h>
#include <Inventor/nodes/SoMaterial.h>
#include <Inventor/nodes/SoShapeHints.h>
#include <iostream.h>

static const MAXFACES = 65000;

SoIsosurface::SoIsosurface() : SoSeparator()
{};

SoIsosurface::SoIsosurface(float* aGrid, int aSizeX, int aSizeY, int aSizeZ,
			   float aThreshold, float aDeltaX,
			   float aDeltaY, float aDeltaZ) :
   SoSeparator(), threshold(aThreshold), sizeX(aSizeX), sizeY(aSizeY),
   sizeZ(aSizeZ), deltaX(aDeltaX), deltaY(aDeltaY), deltaZ(aDeltaZ)
{
   //copy the grid
   grid = new float[sizeX * sizeY * sizeZ];
   memcpy(grid, aGrid, sizeX * sizeY * sizeZ * sizeof(float));
   doTheJob();
   
};

SoIsosurface::
~SoIsosurface()
{
   delete[] grid;
}


void
SoIsosurface::doTheJob()
{
   float vc[MAXFACES], vr[MAXFACES], vl[MAXFACES];
   float nx[MAXFACES], ny[MAXFACES], nz[MAXFACES];
   int vpts[2*MAXFACES];
   int numverts, numindexes, ipoly, itri;
   
   main_march(grid, sizeX, sizeZ, sizeY, 1, threshold, 1, 1, 1, MAXFACES,
	      vc, vr, vl, nx, ny, nz, 2*MAXFACES, vpts,
	      &numverts, &numindexes, &ipoly, &itri);

   if(numindexes > MAXFACES) numindexes = MAXFACES;

   if(numindexes <= 0 || numverts <= 0)
   {
      numverts = numindexes = 0;
   } else 
   {
      SoNormalBinding* normalBind= new SoNormalBinding;
      normalBind->value = SoNormalBinding::PER_VERTEX_INDEXED;
      addChild(normalBind);

      SoShapeHints* shints = new SoShapeHints;
      shints->faceType = SoShapeHints::UNKNOWN_FACE_TYPE;
      shints->vertexOrdering = SoShapeHints::UNKNOWN_ORDERING;
      shints->shapeType = SoShapeHints::SOLID;
      
      addChild(shints);

      SoMaterial* mat = new SoMaterial;
      mat->transparency = 0.01;
      mat->diffuseColor = SbVec3f(0.4, 0.4, 0.6);
//      mat->ambientColor = SbVec3f(0.6, 0.6, 0.6);
//      mat->specularColor = SbVec3f(0.6, 0.6, 0.6);
//      mat->emissiveColor = SbVec3f(0.6, 0.6, 0.6);
      
//      mat->shininess = 0.2;
      
      addChild(mat);

      SoCoordinate3* coords = new SoCoordinate3;
      SoNormal* normal = new SoNormal;
      for(int i=0; i<numverts; i++)
      {
	 coords->point.set1Value(i, SbVec3f(vc[i]*deltaX, vl[i]*deltaY, vr[i]*deltaZ));
	 normal->vector.set1Value(i, SbVec3f(nx[i], nz[i], ny[i]));
      }
      
      addChild(coords);
      addChild(normal);
      
      SoIndexedTriangleStripSet* sset = new SoIndexedTriangleStripSet;
      for(i=0; i<numindexes; i++)
	 sset->coordIndex.set1Value(i, vpts[i]);
      addChild(sset);
   }
};
