/* $Id: SoMovingObj.C,v 1.6 1997/09/25 20:31:55 remek Exp $ */
/*
 * File         : SoMovingObj.C
 * Author       : Pawel Roman (proman@npac.syr.edu)
 * Created      : Mon Aug 25 11:57:12 1997
 * Revision     : $Revision: 1.6 $
 * Updated      : $Date: 1997/09/25 20:31:55 $
 * Copyright: Northeast Parallel Architectures Center
 *            at Syracuse University 1997
 */

#include "SoMovingObj.h"

SbVec3f SoMovingObj::camDist( 0, 9, 9 );

void SoMovingObj::
update( const Class& cl )
{
   ACE_TRACE( "(SoMovingObj::update(Class))" );
   
   static const char FRIEND_SHAPE[] = "/usr/people/tangosim/applets/RO3D/shapes/friend.iv";
   static const char FOE_SHAPE[] = "/usr/people/tangosim/applets/RO3D/shapes/foe.iv";
   static const char UFO_SHAPE[] = "/usr/people/tangosim/applets/RO3D/shapes/ufo.iv";

//   static const char FRIEND_SHAPE[] = "/export/project/K136/CIV/CIVGIS/romek/browser/shapes/friend.iv";
//   static const char FOE_SHAPE[] = "/export/project/K136/CIV/CIVGIS/romek/browser/shapes/foe.iv";
//   static const char UFO_SHAPE[] = "/export/project/K136/CIV/CIVGIS/romek/browser/shapes/ufo.iv";

   if( classification != cl || shape == NULL )
   {
      classification = cl;
      const char* shapeFile;
      switch( classification )
      {
	 case FRIEND:
	 shapeFile = FRIEND_SHAPE;
	 break;
      
	 case FOE:
	 shapeFile = FOE_SHAPE;
	 break;
      
	 case UFO:
	 shapeFile = UFO_SHAPE;
	 break;
      }

      SoShape* newShape;
      SoInput inp;
      if( ! inp.openFile( shapeFile ) )
      {
	 ACE_ERROR(( LM_ERROR, "[!](SoMovingObj::update) File error: %s\n", shapeFile ));
	 newShape = new SoSphere;
      }
      else if( ( newShape = ( SoShape* )SoDB::readAll( &inp ) ) == NULL )
      {
	 ACE_ERROR(( LM_ERROR, "[!](SoMovingObj::update) Wrong format: %s\n", shapeFile ));
	 newShape = new SoSphere;
      }
      hide();
      if( shape )
	 TRACE_UNREF( shape );
      
      shape = newShape;
      TRACE_REF( shape );
   }
   show();
}

void SoMovingObj::
setMoveParams( const GeoPoint& beg_gp,
	       const GeoPoint& end_gp,
	       const float& time_delta )
{
   ACE_TRACE( "(SoMovingObj::setMoveParams)" );
   if( visible_ )
   {
      sprintf( speed_str_, " %.1f k", beg_gp.distance( end_gp ) / time_delta * 60 );
      SbVec3f from = beg_gp.dump().getScaledVec( viewer_->getScale() );
      SbVec3f to = end_gp.dump().getScaledVec( viewer_->getScale() );
      from[1] = to[1] = altitude;
      ACE_DEBUG(( LM_DEBUG, "from: [%.2f %.2f %.2f] to: [%.2f %.2f %.2f] in %.2f sec.\n",
		  from[0], from[1], from[2], to[0], to[1], to[2], time_delta ));
      pos_->setMoveParams( from, to, time_delta );
      if( cam_ )
      {
	 cam_->position = pos_->translation.getValue() + camDist;
	 cam_->pointAt( pos_->translation.getValue() );
      }
   }
   else
      ACE_ERROR(( LM_ERROR, "[!]((SoMovingObj::setMoveParams) %s is invisible\n",
		  name.c_str() ));
}
