/* $Id: SoPlane.C,v 1.5 1998/04/23 20:13:37 remek Exp $ */
//
// File:        SoPlane.C
// Author:      Remigiusz Trzaska
// e-mail:      remek@npac.syr.edu
// Created:     Wed Dec 11 1996

// Description:
//   An implementation of SoPlane Inventor node


#include "SoPlane.h"
#include <Inventor/nodes/SoPickStyle.h>
#include <iostream.h>

SoPlane::SoPlane() : SoPlaneable()
{}

// ugly hack for large files: divide x and z by 2
SoPlane::SoPlane(CFun aColorFunction,
		 int aSizeX,
		 int aSizeY,
		 int aSizeZ,
		 SoXtWebViewer* viewer,
		 int aMode,
		 float aDeltaX, float aDeltaY, float aDeltaZ):
   SoPlaneable(aSizeX/2, aSizeY, aSizeZ/2, viewer, aMode, aDeltaX, aDeltaY, aDeltaZ)
{
   colorFunction = aColorFunction;
   useFun = TRUE;
   setName("plane");
}

// ugly hack for large files: divide x and z by 2
SoPlane::SoPlane(float* aData,
		 int aSizeX,
		 int aSizeY,
		 int aSizeZ, 
		 SoXtWebViewer* viewer,
		 int aMode,
		 float aDeltaX, float aDeltaY, float aDeltaZ):
   SoPlaneable(aSizeX/2, aSizeY, aSizeZ/2, viewer, aMode, aDeltaX, aDeltaY, aDeltaZ)
{
   useFun = FALSE;
   data = new float[sizeX * sizeY * sizeZ];
   //replace memcpy with a loop
   //memcpy(data, aData, sizeX * sizeY * sizeZ * sizeof(float));
   aMin =  1000000;
   aMax = -1000000;

   for(int x=0; x<sizeX; x++)
      for(int y=0; y<sizeY; y++)
	 for(int z=0; z<sizeZ; z++)
	 {
	    float aValue = aData[y*(sizeZ*2)*(sizeX*2) + (2*x)*(sizeZ*2) + (2*z)];
	    data[y*sizeZ*sizeX + x*sizeZ + z] = aValue;
	    if(aValue < aMin)
	       aMin = aValue;
	    if(aValue > aMax)
	       aMax = aValue;
	 }


   setName("plane");

   //calculate the colormap
   //entries 0-25 are insignificant -- transparent cells
//   for(int i=0; i<26; colormap[i++].setValue(0, 0, 0))
//      ;
   
   for(int i=0; i<256; i++)
   {
      float a = i/356.0*2*3.1416;
      float red = 0.5*(-fcos(a/2.0)+1.0);
      float green = 0.5*(-fcos(a)+1.0);
      float blue = 1 - red;
      colormap[i].setValue(red, green, blue);
   }
}

SoPlane::~SoPlane()
{
   if(!useFun) delete data;
}

void
SoPlane::setData(float* aData, int aSizeX, int aSizeY, int aSizeZ)
{
   //ugly hack
   int reconstruct = (sizeX != aSizeX/2)||(sizeY != aSizeY)||(sizeZ != aSizeZ/2);
   
   sizeX = aSizeX/2; 
   sizeY = aSizeY; 
   sizeZ = aSizeZ/2;
   delete data;
   data = new float[sizeX * sizeY * sizeZ];
   //memcpy(data, aData, sizeX * sizeY * sizeZ * sizeof(float));

   aMin =  1000000;
   aMax = -1000000;
   for(int x=0; x<sizeX; x++)
      for(int y=0; y<sizeY; y++)
	 for(int z=0; z<sizeZ; z++)
	 {
	    float aValue = aData[y*(sizeZ*2)*(sizeX*2) + (2*x)*(sizeZ*2) + (2*z)];
	    data[y*sizeZ*sizeX + x*sizeZ + z] = aValue;
	    if(aValue < aMin)
	       aMin = aValue;
	    if(aValue > aMax)
	       aMax = aValue;
	 }

   

   if(reconstruct)
      construct();
   else
      updateValues();
}

void
SoPlane::constructContent()
{
   SoSeparator* scenSep = new SoSeparator;
   
   addChild(scenSep);

   SoPickStyle* style = new SoPickStyle;
   style->style = SoPickStyle::UNPICKABLE;

   scenSep->addChild(style);


   int i,j;

   SoMaterialBinding *binding = new SoMaterialBinding;
   binding->value = SoMaterialBinding::PER_VERTEX_INDEXED;
   scenSep->addChild(binding);

   SoShapeHints *hints = new SoShapeHints;
   hints->vertexOrdering = SoShapeHints::COUNTERCLOCKWISE;
   hints->faceType = SoShapeHints::CONVEX;
   scenSep->addChild(hints);
    
   int counter = 0;

   material = new SoMaterial;
   scenSep->addChild(material);
   material->shininess = 0;
   material->transparency = 0.2;
      
   translation = new SoTranslation;
   scenSep->addChild(translation);
   
   SoCoordinate3 *coor = new SoCoordinate3;
   scenSep->addChild(coor);

   int width;
   int height;
   
   switch (mode)
   {
      case XZ:
	 width = sizeX;
	 height = sizeZ;
	 break;
	 
      case XY:
	 width = sizeX;
	 height = sizeY;
	 break;
	 
      case YZ:
	 width = sizeZ;
	 height = sizeY;
	 break;
	 
   };
   
   for(i=0; i<height; i++)
      for(j=0; j<width; j++) {
	 switch (mode) {
	    case XZ:
	    coor->point.set1Value(counter, j*deltaX, 0, i*deltaZ);
//	    material->transparency.set1Value(counter, getTransparency(j, currY, i));
	    material->diffuseColor.set1Value(counter++, getColor(j, currY, i));
	    
	    break;
	    case XY:
	    coor->point.set1Value(counter, j*deltaX, i*deltaY, 0);
//	    material->transparency.set1Value(counter, getTransparency(j, i, currZ));
	    material->diffuseColor.set1Value(counter++, getColor(j, i, currZ));
	    break;
	    case YZ:
	    coor->point.set1Value(counter, 0, i*deltaY, j*deltaZ);
//	    material->transparency.set1Value(counter, getTransparency(currX, i, j));
	    material->diffuseColor.set1Value(counter++, getColor(currX, i, j));
	    break;
	 }
      }

   counter = 0;

   SoIndexedFaceSet *face = new SoIndexedFaceSet;
   scenSep->addChild(face);

   for(i=0; i<height-1; i++)
      for(j=0; j<width-1; j++) {
	 face->coordIndex.set1Value(counter++, i*width+j);
	 face->coordIndex.set1Value(counter++, i*width+(j+1));
	 face->coordIndex.set1Value(counter++, (i+1)*width+(j+1));
	 face->coordIndex.set1Value(counter++, (i+1)*width+j);
	 face->coordIndex.set1Value(counter++, -1);
      }

}

void
SoPlane::updateValues()
{
   int i, j;

   int counter = 0;

   int width;
   int height;
   
   switch (mode)
   {
      case XZ:
	 width = sizeX;
	 height = sizeZ;
	 break;
	 
      case XY:
	 width = sizeX;
	 height = sizeY;
	 break;
	 
      case YZ:
	 width = sizeZ;
	 height = sizeY;
	 break;
	 
   };
   for(i=0; i<height; i++)
      for(j=0; j<width; j++) {
	 switch (mode) {
	    case XZ:
//	    material->transparency.set1Value(counter, getTransparency(j, currY, i));
	    material->diffuseColor.set1Value(counter++, getColor(j, currY, i));
	    break;
	    case XY:
//	    material->transparency.set1Value(counter, getTransparency(j, i, currZ));
	    material->diffuseColor.set1Value(counter++, getColor(j, i, currZ));
	    break;
	    case YZ:
//	    material->transparency.set1Value(counter, getTransparency(currX, i, j));
	    material->diffuseColor.set1Value(counter++, getColor(currX, i, j));
	    break;
	 }
      }


}

const SbVec3f
SoPlane::getColor(int x, int y, int z)
{
   if (useFun)
      return (*colorFunction)(x, y, z);
   else
   {
      float scaled;
      
      if(aMax == aMin)
	 scaled = 0;
      else
	 scaled = (getData(x,y,z)-aMin)/(aMax-aMin)*255;
      return colormap[(int)scaled];
   }
}

const float
SoPlane::getTransparency(int x, int y, int z)
{
   float value = (getData(x,y,z)-aMin)/(aMax-aMin);
   if(value<0.1)
      return 1;
   else
      return (1-value)/4.0;
}

