/* $Id: SoPlane.h,v 1.4 1998/02/12 17:26:46 remek Exp $ */
//
// File:        SoPlane.h
// Author:      Remigiusz Trzaska
// e-mail:      remek@npac.syr.edu
// Created:     Wed Dec 11 1996
//
// Description: 
//   An inventor node for representing a colored plane

#ifndef _SOPLANE_H
#define _SOPLANE_H

#include "SoPlaneable.h"
#include <Inventor/nodes/SoMaterialBinding.h>
#include <Inventor/nodes/SoShapeHints.h>
#include <Inventor/nodes/SoIndexedFaceSet.h>


typedef SbVec3f (*CFun)(int, int, int) ; //a color generating function

class SoPlane: public SoPlaneable {
      CFun colorFunction;
      SbBool useFun;
      float *data; //the order data[x][y][z]
      SoMaterial* material;
      SbVec3f colormap[256];
      float aMax;
      float aMin;
      
   public:
      SoPlane();
      SoPlane(float *aData,
	      int aSizeX,
	      int aSizeY,
	      int aSizeZ,
	      SoXtWebViewer* viewer,
	      int aMode = XZ,
	      float aDeltaX=1.0, float aDeltaY=1.0, float aDeltaZ=1.0);
      SoPlane(CFun aColorFunction,
	      int aSizeX,
	      int aSizeY,
	      int aSizeZ, 
	      SoXtWebViewer* viewer,
	      int aMode = XZ,
	      float aDeltaX=1.0, float aDeltaY=1.0, float aDeltaZ=1.0);
      virtual ~SoPlane();
      void setColorFunction(CFun aNewFunction) {
	 colorFunction = aNewFunction;
      }
      void setData(float* aData, int aSizeX, int aSizeY, int aSizeZ);
      
   protected:
      virtual void constructContent();
      virtual void updateValues();
      
   private:
      const float getData(int x, int y, int z) {
	 float aValue = data[y*sizeZ*sizeX + x*sizeZ + z];
	 if (aValue > 10e30) aValue = 0.0;
	 return aValue;
      }
      const SbVec3f getColor(int x, int y, int z);
      const float getTransparency(int x, int y, int z);
      
}
;

#endif //SOPLANE_H
