/* $Id: SoPlaneable.C,v 1.4 1998/02/12 20:35:44 proman Exp $ */
//
// File:       <SoPlaneable.C>
// Author:     <Remigiusz Trzaska>
// e-mail:     <remek@npac.syr.edu>
// Created:    <Wed Jan 15 1997>
// Description:
// 


#include "SoPlaneable.h"
#include <iostream.h>
#include "SoXtWebViewer.h"

void
planeDraggerValueChangedCB(void *data, SoDragger *d)
{
   SoPlaneable *pl = (SoPlaneable*)data;
   SbVec3f v = ((SoTranslate1Dragger*)d)->translation.getValue();
   float newCoor = v[0];
   if(newCoor < 0)
      newCoor = 0;
   
   pl->moveYourself(newCoor);
}




SoPlaneable::SoPlaneable()
{
   mode = XZ;
   deltaX = deltaY = deltaZ = 1.0;
   currX = currY = currZ = 0;
}

SoPlaneable::SoPlaneable(int aSizeX,
			 int aSizeY,
			 int aSizeZ,
			 SoXtWebViewer* aViewer,
			 int aMode,
			 float aDeltaX, float aDeltaY, float aDeltaZ)
{
   deltaX = aDeltaX;
   deltaY = aDeltaY;
   deltaZ = aDeltaZ;
   sizeX = aSizeX;
   sizeY = aSizeY;
   sizeZ = aSizeZ;
   viewer = aViewer;
   setMode(aMode);
}

SoPlaneable::~SoPlaneable()
{
}

void
SoPlaneable::setMode(int aMode) 
{
   mode = aMode; 
   currX = currY = currZ = 0;
}

void 
SoPlaneable::setCurrentX(float newX)
{
   currX = newX;
   translation->translation.setValue(newX*deltaX, 0, 0);
   for(int i=0; i<4; i++)
   {
      SbVec3f dragTrans = draggers[i]->translation.getValue();
      draggers[i]->translation.setValue(newX*deltaX,
					dragTrans[1], dragTrans[2]);
   }
   updateValues();
}

void 
SoPlaneable::setCurrentY(float newY)
{
   currY = newY;
   translation->translation.setValue(0, newY*deltaY, 0);
   for(int i=0; i<4; i++)
   {
      SbVec3f dragTrans = draggers[i]->translation.getValue();
      draggers[i]->translation.setValue(newY*deltaY,
					dragTrans[1], dragTrans[2]);
   }
   updateValues();
}

void 
SoPlaneable::setCurrentZ(float newZ)
{
   currZ = newZ;
   translation->translation.setValue(0, 0, newZ*deltaZ);
   for(int i=0; i<4; i++)
   {
      SbVec3f dragTrans = draggers[i]->translation.getValue();
      draggers[i]->translation.setValue(newZ*deltaZ,
					dragTrans[1], dragTrans[2]);
   }
   updateValues();
}

void
SoPlaneable::construct()
{
   removeAllChildren();
   constructContent();
   addHandles();
}

void
SoPlaneable::addHandles()
{
   draggers[0] = addHandle(0, -1);
   draggers[1] = addHandle(1, 0);
   draggers[2] = addHandle(0, 1);
   draggers[3] = addHandle(-1, 0);
};

SoTranslate1Dragger*
SoPlaneable::addHandle(int widthDelta, int heightDelta)
{
   SoSeparator* dragSep = new SoSeparator;
   addChild(dragSep);

   float dw;
   float dh;

   SoTranslation *translation = new SoTranslation;
   dragSep->addChild(translation);

   SoRotation* rotation = new SoRotation;
   dragSep->addChild(rotation);
   switch(mode) {
      case(XZ):
	 dw = (widthDelta+1)*(sizeX-1)/2*deltaX;
	 dh = (heightDelta+1)*(sizeZ-1)/2*deltaZ;
	 rotation->rotation.setValue(0, 0, 1, 1);
	 translation->translation.setValue(dw, 0, dh);
      break;
      case(XY):
	 dw = (widthDelta+1)*(sizeX-1)/2*deltaX;
	 dh = (heightDelta+1)*(sizeY-1)/2*deltaY;
	 rotation->rotation.setValue(0, 1, 0, -1);
	 translation->translation.setValue(dw, dh, 0);
      break;
      case(YZ):
	 dw = (widthDelta+1)*(sizeZ-1)/2*deltaZ;
	 dh = (heightDelta+1)*(sizeY-1)/2*deltaY;
	 rotation->rotation.setValue(1, 0, 0, 1);
	 translation->translation.setValue(0, dh, dw);
      break;
   }
   SoTranslate1Dragger* dragger = new SoTranslate1Dragger;
   dragSep->addChild(dragger);
   dragger->addValueChangedCallback(planeDraggerValueChangedCB, this);
   
   SoSeparator* boxTranslator = new SoSeparator;
   boxTranslator->ref();
   SoCube* smallCube = new SoCube;
   switch(mode) {
      case(XZ):
	 smallCube->width = deltaY;
	 smallCube->height = deltaX;
	 smallCube->depth = deltaZ;
	 break;
      case(XY):
	 smallCube->width = deltaZ;
	 smallCube->height = deltaY;
	 smallCube->depth = deltaX;
	 break;
      case(YZ):
	 smallCube->width = deltaX;
	 smallCube->height = deltaZ;
	 smallCube->depth = deltaY;
	 break;
   }
   
   boxTranslator->addChild(smallCube);

   SoSeparator* activeBoxTranslator = new SoSeparator;
   activeBoxTranslator->ref();
   SoMaterial *activeBoxMaterial = new SoMaterial;
   activeBoxMaterial->diffuseColor.setValue(1, 1, 0);
   activeBoxTranslator->addChild(activeBoxMaterial);   
   activeBoxTranslator->addChild(smallCube);
   
   dragger->setPart("translator", boxTranslator);
   dragger->setPart("translatorActive", activeBoxTranslator);
   return dragger;
}

void SoPlaneable::
moveYourself(float newCoor)
{
   int mode = getMode();
   float upperLimit = (mode==SoPlaneable::XZ)?
      getSizeY() :
      (mode==SoPlaneable::XY)? getSizeZ() : getSizeX();

   float delta = (mode==SoPlaneable::XZ)?
      deltaY :
      (mode==SoPlaneable::XY)? deltaZ : deltaX;

   if(newCoor>(upperLimit-1)*delta) 
      newCoor=(upperLimit-1)*delta;

   //send here
   SoXtWebViewer::Protocol::PMessage* msg;
   if( viewer->getContrStatus() == SoXtWebViewer::MASTER )
     if( viewer->getProto().getMsgType( MOVEPL, msg ) == SoXtWebViewer::Protocol::P_OK )
       {
	 const char* buf[2] = { ( const char* )&newCoor, getName().getString() };
	 if( msg->set( buf ) == SoXtWebViewer::Protocol::PMessage::OK_RES )
	   viewer->getProto().sendMsg( MOVEPL );
       }

   if(delta!=0)
      newCoor = newCoor/delta;


   switch(mode) {
      case(SoPlaneable::XZ): setCurrentY(newCoor);
      break;
      case(SoPlaneable::XY): setCurrentZ(newCoor);
      break;
      case(SoPlaneable::YZ): setCurrentX(newCoor);
      break;
   }      
};
