/* $Id: SoStaticObj.h,v 1.4 1997/09/25 20:31:57 remek Exp $ */
/*
 * File         : SoStaticObj.h
 * Author       : Pawel Roman (proman@npac.syr.edu)
 * Created      : Wed Aug 20 13:37:58 1997
 * Revision     : $Revision: 1.4 $
 * Updated      : $Date: 1997/09/25 20:31:57 $
 * Copyright: Northeast Parallel Architectures Center
 *            at Syracuse University 1997
 */

#ifndef __SOSTATICOBJ_H
#define __SOSTATICOBJ_H

#include <STL/stlstring.h>

#include <Inventor/nodes/SoShape.h>
#include <Inventor/nodes/SoCone.h>
#include <Inventor/nodes/SoSeparator.h>
#include <Inventor/nodes/SoTranslation.h>

class SoStaticObj : public SoSeparator
{
   public:
      enum Type 
      {
	 BASE        = 0,
	 SHIP        = 1,
	 RADAR       = 2,
	 SEMO        = 3
      };
      
      SoStaticObj( const string& obj_name,
		   SoTranslation* obj_trans,
		   const Type& obj_type );

      ~SoStaticObj( void );
      const string& getName( void );
      
   private:
      string name;
      SoTranslation* position;
      SoShape* shape;
      Type type;
};

inline SoStaticObj::
SoStaticObj( const string& obj_name,
	     SoTranslation* obj_trans,
	     const Type& obj_type )
   : name( obj_name ), position( obj_trans ), type( obj_type )
{
   ACE_TRACE( "(SoStaticObj::SoStaticObj)" );
   static const char BASE_SHAPE[] = "/usr/people/tangosim/applets/RO3D/shapes/base.iv";
   static const char RADAR_SHAPE[] = "/usr/people/tangosim/applets/RO3D/shapes/radar.iv";
   static const char SHIP_SHAPE[] = "/usr/people/tangosim/applets/RO3D/shapes/ship.iv";

   ACE_ASSERT( position != NULL );
   const char* shapeFile;
   SoInput inp;
   switch( type )
   {
      case BASE:
      shapeFile = BASE_SHAPE;
      break;

      case RADAR:
      shapeFile = RADAR_SHAPE;
      break;
      
      case SHIP:
      shapeFile = SHIP_SHAPE;
      break;

      default:
	 shapeFile = RADAR_SHAPE;
	 break;
   }

   if( ! inp.openFile( shapeFile ) )
   {
      ACE_ERROR(( LM_ERROR, "[!](SoStaticObj::SoStaticObj) File error: %s\n", shapeFile ));
      shape = new SoCone;
   }
   else if( ( shape = ( SoShape* )SoDB::readAll( &inp ) ) == NULL )
   {
      ACE_ERROR(( LM_ERROR, "[!](SoStaticObj::SoStaticObj) Wrong format: %s\n", shapeFile ));
      shape = new SoCone;
   }
   addChild( position );
   addChild( shape );
}

inline SoStaticObj::
~SoStaticObj( void )
{
   ACE_TRACE( "(SoStaticObj::~SoStaticObj)" );
   removeChild( position );
   removeChild( shape ); 
   ACE_DEBUG(( LM_DEBUG, "this=0x%x, ref=%d\n", this, getRefCount() ));
}

inline const string& SoStaticObj::
getName( void )
{
   ACE_TRACE( "(SoStaticObj::getName)" );
   return name;
}


#endif
