/* $Id: SoTerrain.C,v 1.18 1997/09/26 19:31:41 remek Exp $ */
/*
 * File:       SoTerrain.C
 * Author:     Remek Trzaska (remek@npac.syr.edu)
 * Created:    long time ago
 * Revision:   $Revision: 1.18 $
 * Updated:    $Date: 1997/09/26 19:31:41 $
 * Copyright 1996/97 NPAC, Syracuse University
 *
 */

#include "SoTerrain.h"

#define FACTOR 25

int trianglemain(int, OutFile*, int *, int *);

SoTerrain::~SoTerrain()
{
   ACE_TRACE( "(SoTerrain::~SoTerrain)" );
   DeleteMatrix( demData );
   ACE_DEBUG(( LM_DEBUG, "this=0x%x, ref=%d\n", this, getRefCount() ));
};

void
SoTerrain::build()
{
   ACE_TRACE( "(SoTerrain::build)" );
// tr will contain SoSeparator root with the whole terrain
   IVFile *tr;
   TRACE_NEW_2( tr, IVFile() );
   tr->writeInitialParams();
   tr->startListOfNodes();
   MeshSimplifier *ms;
   TRACE_NEW_2( ms, MeshSimplifier(demData, tr, dim, yScale) );
   ms->simplifyMesh(meshThresh);
   tr->finishListOfNodes();

   minAlt = tr->minAlt;
   maxAlt = tr->maxAlt;
   
   tr->startListOfFaces();
   trianglemain(ms->getNotRemovedCounter(), tr, ms->datax, ms->dataz);
   tr->finishListOfFaces();
   tr->root->setName("Level_0");
   
   addChild(tr->root);
   TRACE_DELETE( ms );
   TRACE_DELETE( tr );
   
#ifndef HIGHEST_LEVEL_ONLY
   
   IVFile *tr1;
   TRACE_NEW_2( tr1, IVFile() );
   tr1->writeInitialParams();
   tr1->startListOfNodes();
   MeshSimplifier *ms1;
   TRACE_NEW_2( ms1, MeshSimplifier(demData, tr1, dim, yScale) );
   ms1->simplifyMesh(meshThresh*3); //*5
   tr1->finishListOfNodes();
   tr1->startListOfFaces();
   trianglemain(ms1->getNotRemovedCounter(), tr1, ms1->datax, ms1->dataz);
   tr1->finishListOfFaces();
   tr1->root->setName("Level_1");
   addChild(tr1->root);
   TRACE_DELETE( ms1 );
   TRACE_DELETE( tr1 );


   IVFile *tr2;
   TRACE_NEW_2( tr2, IVFile() );
   tr2->writeInitialParams();
   tr2->startListOfNodes();
   MeshSimplifier *ms2;
   TRACE_NEW_2( ms2, MeshSimplifier(demData, tr2, dim, yScale) );
   ms2->simplifyMesh(meshThresh*10); //*20
   tr2->finishListOfNodes();
   tr2->startListOfFaces();
   trianglemain(ms2->getNotRemovedCounter(), tr2, ms2->datax, ms2->dataz);
   tr2->finishListOfFaces();
   tr2->root->setName("Level_2");
   addChild(tr2->root);
   TRACE_DELETE( ms2 );
   TRACE_DELETE( tr2 );

#endif // HIGHEST_LEVEL_ONLY

//    SoGroup* square = new SoGroup;
//    SoCoordinate3* squareCoords = new SoCoordinate3;
//    squareCoords->point.set1Value(0, SbVec3f(
//       0, demData[0][0]/FACTOR, 0));
//    squareCoords->point.set1Value(1, SbVec3f(
//       dim-1, demData[0][dim-1]/FACTOR, 0));
//    squareCoords->point.set1Value(2, SbVec3f(
//       dim-1, demData[dim-1][dim-1]/FACTOR, dim-1));
//    squareCoords->point.set1Value(3, SbVec3f(
//       0, demData[dim-1][0]/FACTOR, dim-1));
//    square->addChild(squareCoords);

//    SoFaceSet* squareFace=new SoFaceSet;
//    squareFace->numVertices.setValue(4);
   
//    square->addChild(squareFace);

//    addChild(square);

};

void SoTerrain::buildCoordinates()
{};

void SoTerrain::buildFaceSet()
{};


void SoTerrain::
fakeDEM()
{
   ACE_TRACE( "(SoTerrain::fakeDEM)" );
   dim = degSize/60.0 * 1201;
   NewMatrix(demData, dim, dim, ELEV);
    
   for(int i=0; i<dim; i++)
      for(int j=0; j<dim; j++)
	 demData[i][j] = 0.0;

};
