/* $Id: SoTerrain.h,v 1.22 1997/09/25 20:31:58 remek Exp $ */
/*
 * File:       SoTerrain.h
 * Author:     Remek Trzaska (remek@npac.syr.edu)
 * Created:    long time ago
 * Revision:   $Revision: 1.22 $
 * Updated:    $Date: 1997/09/25 20:31:58 $
 * Copyright 1996/7 NPAC, Syracuse University
 *
 */

#ifndef __SOTERRAIN_H
#define __SOTERRAIN_H

#include <Inventor/nodes/SoSeparator.h>
#include <Inventor/nodes/SoLOD.h>
#include <Inventor/nodes/SoCoordinate3.h>
#include <Inventor/nodes/SoIndexedFaceSet.h>

#include <iostream.h>
#include "DEM.h"
#include "VRML.h"


class SoTerrain : public SoLOD
{
   public:
      int dim;
      float meshThresh;
      float minAlt, maxAlt;
      
      SoTerrain(char *name, float aMeshThresh, float yS, int dSize);
      SoTerrain(ELEV **aDEMData, float aMeshThresh, int aDim);
      SoTerrain(DEM *aDEM, float aMeshThresh);

      void build();
      void buildCoordinates();
      void buildFaceSet();

      void prepareListsOfBoundaries();
      ELEV getElev(int i, int j) 
      {
	 return demData[i][j];
      }

   protected:
      ~SoTerrain();

   private:
      float yScale;
      ELEV **demData;
      int degSize;
      void fakeDEM();
};

// {{{ constructors

inline SoTerrain::
SoTerrain(char *name, float aMeshThresh, float yS, int dSize)
   : SoLOD(), meshThresh( aMeshThresh), yScale( yS ), degSize (dSize)
{
   ACE_TRACE( "(SoTerrain::SoTerrain(char* float, float, int))" );
   char ext[4];
   int len = strlen(name);
   ext[0] = name[len-3];
   ext[1] = name[len-2];
   ext[2] = name[len-1];
   ext[3] = 0;
   DEM *df;
   TRACE_NEW_2( df, URLDEM((char*)name) );
   if(!df->read())
   {
      dim = df->dim;
      NewMatrix(demData, dim, dim, ELEV);
      for(int i=0; i<dim; i++)
	 for(int j=0; j<dim; j++)
	    demData[i][j] = df->data[i][j];
   }
   else fakeDEM();
   
   range.set1Value(0, 5000);
   range.set1Value(1, 7000);

   TRACE_DELETE(df);
   TRACE_REF(this);
};

inline SoTerrain::
SoTerrain(ELEV **aDEMData, float aMeshThresh, int aDim)
   : SoLOD()
{
   ACE_TRACE( "(SoTerrain::SoTerrain(ELEV**, float, int))" );
   dim = aDim;
   meshThresh = aMeshThresh;
   
   NewMatrix(demData, aDim, aDim, ELEV);
   
   for(int i=0; i<aDim; i++)
      for(int j=0; j<aDim; j++)
	 demData[i][j] = aDEMData[i][j];

   TRACE_REF(this);
   
};


inline SoTerrain::
SoTerrain(DEM *aDEM, float aMeshThresh): SoLOD()
{
   ACE_TRACE( "(SoTerrain::SoTerrain(DEM*, float))" );
   demData = aDEM->data;
   dim = aDEM->dim;
   meshThresh = aMeshThresh;
   
   TRACE_REF(this);
};

// }}}
#endif
