/* $Id: SoXtLayerList.C,v 1.8 1997/09/25 20:32:00 remek Exp $ */
/*
 * File:       SoXtLayerList.C
 * Author:     Pawel Roman (proman@npac.syr.edu)
 * Created:    long time ago
 * Revision:   $Revision: 1.8 $
 * Updated:    $Date: 1997/09/25 20:32:00 $
 * Copyright 1996/97 NPAC, Syracuse University
 *
 */

#include "SoXtLayerList.h"
#include "SoXtWebViewer.h"

#include <Xm/DialogS.h>
#include <Xm/BulletinB.h>
#include <Xm/List.h>

SoXtLayerList::
SoXtLayerList( const vector<string>& layers,
	       SoXtWebViewer* vw,
	       Widget parent,
	       const char* name,
	       SbBool buildInsideParent )
   : SoXtComponent( parent, name, buildInsideParent ), viewer( vw )
{
   ACE_TRACE( "(SoXtLayerList::SoXtLayerList)" );
   int nCount = layers.size();
   XmStringTable str_list = ( XmStringTable )XtMalloc( nCount * sizeof ( XmString* ) );
   for( int i=0; i < nCount; i++ )
      str_list[i] = XmStringCreateSimple( ( char* )layers[i].c_str() );
   
   Widget shell = XmCreateDialogShell( parent, "LayerList", NULL, 0 );
   Arg args[2];
   XtSetArg( args[0], XmNdefaultPosition, False );
   XtSetArg( args[1], XmNnoResize, True );
   manager = XmCreateBulletinBoard( shell, "BBoard", args, 2 );
   XtAddCallback( manager, XmNmapCallback, ( XtCallbackProc )mapCB, parent );
   Widget list = XtVaCreateManagedWidget( "LayerList",
					  xmListWidgetClass,     manager,
					  XmNvisibleItemCount,   nCount,
					  XmNitemCount,          nCount,
					  XmNitems,              str_list,
					  NULL );
   XtAddCallback( list, XmNdefaultActionCallback, ( XtCallbackProc )selectedCB, this );
   for( i=0; i<nCount; i++ )
      XmStringFree( str_list[i] );

   XtFree( ( char* )str_list );
}

void SoXtLayerList::
mapCB( Widget dialog, Widget adj_widget )
{
   ACE_TRACE( "(SoXtLayerList::mapCB)" );
   static const int dFrameWidth = 15;
   Position x, y;
   Dimension w;
   
   XtVaGetValues( adj_widget, XmNx, &x, XmNy, &y, NULL );
   XtVaGetValues( XtParent( dialog ), XmNwidth, &w, NULL );
   XtVaSetValues( XtParent( dialog ), XmNx, x-w-dFrameWidth, XmNy, y, NULL );
   ACE_DEBUG(( LM_DEBUG, "x=%d, y=%d, w=%d, x-w=%d\n", x, y, w, x-w-dFrameWidth ));
}

void SoXtLayerList::
selectedCB( Widget /*list*/,
	    SoXtLayerList* pLList,
	    XmListCallbackStruct* cbs )
{
   ACE_TRACE( "(SoXtLayerList::selectedCB)" );
   pLList->hide();
   ACE_DEBUG(( LM_DEBUG, "Selected no. %d\n", cbs->item_position ) );
   pLList->viewer->switchLayerTo( cbs->item_position-1 );
}
