/* $Id: SoXtVariableList.C,v 1.2 1998/02/12 02:31:32 remek Exp $ */
/*
 * File:       SoXtVariableList.C
 * Author:     Pawel Roman (proman@npac.syr.edu)
 * Created:    long time ago
 * Revision:   $Revision: 1.2 $
 * Updated:    $Date: 1998/02/12 02:31:32 $
 * Copyright 1996/97 NPAC, Syracuse University
 *
 */

#include "SoXtVariableList.h"
#include "SoXtWebViewer.h"

#include <Xm/DialogS.h>
#include <Xm/BulletinB.h>
#include <Xm/List.h>

SoXtVariableList::
SoXtVariableList( const vector<string>& variables,
	       SoXtWebViewer* vw,
	       Widget parent,
	       const char* name,
	       SbBool buildInsideParent )
   : SoXtComponent( parent, name, buildInsideParent ), viewer( vw )
{
   ACE_TRACE( "(SoXtVariableList::SoXtVariableList)" );
   int nCount = variables.size();
   XmStringTable str_list = ( XmStringTable )XtMalloc( nCount * sizeof ( XmString* ) );
   for( int i=0; i < nCount; i++ )
      str_list[i] = XmStringCreateSimple( ( char* )variables[i].c_str() );
   
   Widget shell = XmCreateDialogShell( parent, "VariableList", NULL, 0 );
   Arg args[2];
   XtSetArg( args[0], XmNdefaultPosition, False );
   XtSetArg( args[1], XmNnoResize, True );
   manager = XmCreateBulletinBoard( shell, "BBoard", args, 2 );
   XtAddCallback( manager, XmNmapCallback, ( XtCallbackProc )mapCB, parent );
   Widget list = XtVaCreateManagedWidget( "VariableList",
					  xmListWidgetClass,     manager,
					  XmNvisibleItemCount,   nCount,
					  XmNitemCount,          nCount,
					  XmNitems,              str_list,
					  NULL );
   XtAddCallback( list, XmNdefaultActionCallback, ( XtCallbackProc )selectedCB, this );
   for( i=0; i<nCount; i++ )
      XmStringFree( str_list[i] );

   XtFree( ( char* )str_list );
}

SoXtVariableList::
~SoXtVariableList( void )
{
   ACE_TRACE( "(SoXtVariableList::~SoXtVariableList)" );
   XtDestroyWidget( XtParent( manager ) );
}

void SoXtVariableList::
mapCB( Widget dialog, Widget adj_widget )
{
   ACE_TRACE( "(SoXtVariableList::mapCB)" );
   static const int dFrameWidth = 15;
   Position x, y;
   Dimension w;
   
   XtVaGetValues( adj_widget, XmNx, &x, XmNy, &y, NULL );
   XtVaGetValues( XtParent( dialog ), XmNwidth, &w, NULL );
   XtVaSetValues( XtParent( dialog ), XmNx, x-w-dFrameWidth, XmNy, y, NULL );
   ACE_DEBUG(( LM_DEBUG, "x=%d, y=%d, w=%d, x-w=%d\n", x, y, w, x-w-dFrameWidth ));
}

void SoXtVariableList::
selectedCB( Widget /*list*/,
	    SoXtVariableList* pLList,
	    XmListCallbackStruct* cbs )
{
   ACE_TRACE( "(SoXtVariableList::selectedCB)" );
   pLList->hide();
   ACE_DEBUG(( LM_DEBUG, "Selected no. %d\n", cbs->item_position ) );
   pLList->performAction(pLList, cbs->item_position-2);
//   pLList->viewer->switchVariableTo( cbs->item_position-2 );
}
