/* $Id: SoXtWebViewer.C,v 1.62 1998/04/23 20:13:38 remek Exp $ */
/*
 * File:       SoXtWebViewer.C
 * Author:     Pawel Roman (proman@npac.syr.edu) and
 *             Remek Trzaska (remek@npac.syr.edu)
 * Created:    long time ago
 * Revision:   $Revision: 1.62 $
 * Updated:    $Date: 1998/04/23 20:13:38 $
 * Copyright 1996/97 NPAC, Syracuse University
 *
 */

#include "SoXtWebViewer.h"

#include "msgCAMP.h"
#include "msgNURL.h"
#include "msgSLAY.h"
#include "msgNWRL.h"
#include "msgSYNCR.h"
#include "msgSYNCI.h"
#include "msgQSYNC.h"
#include "msgDRAG.h"
#include "msgNGRND.h"
#include "msgNAIRB.h"
#include "msgABUPD.h"
#include "msgABMOVE.h"
#include "msgSHOW.h"
#include "msgHIDE.h"
#include "msgPAUSE.h"
#include "msgRESUME.h"
#include "msgFOLLOW.h"
#include "msgCSUPD.h"

#ifdef V5DSTUFF
#include "msgANIMATE.h"
#include "msgSHOWSLICE.h"
#include "msgSHOWISO.h"
#include "msgSHOWWIND.h"
#include "msgMOVEPL.h"
#endif //V5DSTUFF

#include "WebNode.h"
#include "RectArea.h"
#include "TagParser.h"
#include "SoStaticObj.h"
#ifdef V5DSTUFF
#include "SoPlane.h"
#include "SoIsosurface.h"
#include "SoVectors.h"
#endif
#include <Inventor/Xt/SoXt.h>
#include <Inventor/SoPickedPoint.h>
#include <Inventor/nodes/SoSelection.h>
#include <Inventor/nodes/SoWWWInline.h>
#include <Inventor/nodes/SoCamera.h>
#include <Inventor/nodes/SoInfo.h>
#include <Inventor/nodes/SoFont.h>
#include <Inventor/nodes/SoText2.h>
#include <Inventor/nodes/SoTranslation.h>
#include <Inventor/nodes/SoOrthographicCamera.h>
#include <Inventor/fields/SoSFRotation.h>
#include <Inventor/actions/SoWriteAction.h>
#include <Inventor/actions/SoRayPickAction.h>
#include <Inventor/actions/SoSearchAction.h>
#include <Inventor/sensors/SoFieldSensor.h>
#include <Inventor/sensors/SoAlarmSensor.h>
#include <Inventor/sensors/SoTimerSensor.h>
#include <Inventor/draggers/SoTranslate1Dragger.h>

#include <Xm/PushBG.h>
#include <Xm/MessageB.h>

#include <iostream.h>

#if defined(ACE_NDEBUG)
#define DUMP_REF_INFO( X )
#else
#define DUMP_REF_INFO( __SCENE ) \
do { SoSearchAction dumpAction; \
 dumpAction.setType( SoNode::getClassTypeId() ); \
 dumpAction.setInterest( SoSearchAction::ALL ); \
 dumpAction.apply( __SCENE ); \
 SoPathList& pathList = dumpAction.getPaths(); \
 int __ace_error = errno;\
 ACE_Log_Msg *ace___ = ACE_Log_Msg::instance (); \
 ace___->set( __FILE__, __LINE__, 0, __ace_error, ace___->restart(), ace___->msg_ostream() ); \
 ace___->log( LM_DEBUG, "--DUMP_REF_INFO--\n" ); \
 vector<SoNode*> nodeVec( pathList.getLength() ); \
 for( int i=0; i<nodeVec.size(); i++ ) \
    { nodeVec[i] = pathList[i]->getTail(); } \
 dumpAction.reset(); \
 for( i=0; i<nodeVec.size(); i++ ) \
    { ace___->log( LM_DEBUG, "%30s:(0x%x): %d\n", \
		   nodeVec[i]->getTypeId().getName().getString(), \
		   nodeVec[i], \
		   nodeVec[i]->getRefCount() ); } \
 ace___->log( LM_DEBUG, "--DUMP_REF_INFO--\n" ); \
} while( 0 )
#endif
//define instead of static const char to allow compiler to glue
//consecutive lines
#define DBURL "http://sandman.npac.syr.edu:8888/cgi-bin/Webdriver?MIval=gen_query"

static const float DBRADIUS = 0.25;
static const float MINIMALMAXYCOORD = 1000.0;

static const char FILE_PREFIX[] = "file://";
static const char HTTP_PREFIX[] = "http://";
#ifndef ACE_NLOGGING
static const char DUMP_LOG[] = "/tmp/GIS_dump.log";
#endif
#ifdef DUMPSCENEBUTTON
static const char DUMP_SCENE_FILE_NAME[] = "/tmp/scene.iv";
#endif //DUMPSCENEBUTTON

#ifdef V5DSTUFF
static void
timeChangedCB(void* data, SoSensor* aSensor)
{
   SoXtWebViewer* wv = (SoXtWebViewer*)data;
//   ((SoAlarmSensor*)aSensor)->setTimeFromNow(0.5);
//   aSensor->schedule();

   if(wv->doAnimation) {
      wv->timeSlice++;
      if(wv->timeSlice==wv->v5d_file.NumTimes-1) wv->timeSlice=0;

      SoNodeList aList;
      SoNode::getByName("plane", aList);

      for(int i=0; i<aList.getLength(); i++)
      {
	 SoPlane* plane = (SoPlane*)aList[i];
	 float* dataBuf = new float[wv->v5d_numRows * wv->v5d_numCols * wv->v5d_numLevels];
	 v5dReadGrid(&(wv->v5d_file), wv->timeSlice, wv->v5d_currentVar, dataBuf);
	 plane->setData(dataBuf, wv->v5d_numCols, wv->v5d_numLevels,
			wv->v5d_numRows);
	 delete dataBuf;
      }

      SoSwitch* isoSwitch = (SoSwitch*)SoNode::getByName("isosurfaces");
      if(isoSwitch)
	 if(isoSwitch->whichChild.getValue()!=SO_SWITCH_NONE)
	    isoSwitch->whichChild = wv->timeSlice;
      

      SoNodeList aList2;
      SoNode::getByName("vectors", aList2);

      for(i=0; i<aList2.getLength(); i++)
      {
 	 SoVectors* plane = (SoVectors*)aList2[i];
 	 float* xdataBuf = new float[wv->v5d_numRows * wv->v5d_numCols * wv->v5d_numLevels];
 	 v5dReadGrid(&(wv->v5d_file), wv->timeSlice, 1, xdataBuf);
 	 float* ydataBuf = new float[wv->v5d_numRows * wv->v5d_numCols * wv->v5d_numLevels];
 	 v5dReadGrid(&(wv->v5d_file), wv->timeSlice, 2, ydataBuf);
 	 float* zdataBuf = new float[wv->v5d_numRows * wv->v5d_numCols * wv->v5d_numLevels];
 	 v5dReadGrid(&(wv->v5d_file), wv->timeSlice, 3, zdataBuf);
 	 plane->setData(xdataBuf, ydataBuf, zdataBuf,
			wv->v5d_numCols, wv->v5d_numLevels, wv->v5d_numRows);
 	 delete xdataBuf;
 	 delete ydataBuf;
	 delete zdataBuf;
      }
   }
} 

#endif

SoXtWebViewer::
SoXtWebViewer( const IPC_SAP& ipc_sap,
	       ViewerID id,
	       ContrStatus stat,
	       Widget parent, 
	       const char *name, 
	       SbBool buildInsideParent, 
	       SoXtFullViewer::BuildFlag flag, 
	       SoXtViewer::Type type )
   : SoXtWalkViewer( parent, name, buildInsideParent, flag, type, TRUE ),
     contrStatus( stat ), proto( ipc_sap, ENDMARK ), vid( id ),
     movoSensor( NULL ), world_scale( 1.0, 1.0, 1.0 ), update( true ),
     wrlType( NONE ), layer( -1 ), followed( NULL ), world_origin( 0, 0, 0 ),
     layerRep( NULL ), chunkRep( NULL ), olDialog( NULL ), layerList( NULL ),
     dbList( NULL ), movoList( NULL ), synchronousMode( true ),
     boundsTagParsed( false )
{ 
   ACE_TRACE( "(SoXtWebViewer::SoXtWebViewer)" );

   olDialog = new SoXtOLDialog( this, getWidget() );

   vector<string> dumbVec( 1 );
   dumbVec[0] = string( "(none)" );
   TRACE_NEW_2( movoList, SoXtMovoList( dumbVec, this, getWidget() ) );
   TRACE_NEW_2( layerList, SoXtLayerList( dumbVec, this, getWidget() ) );
   TRACE_NEW_2( layerRep, vector<string>( 1 ) );
   ( *layerRep )[0] = string( "none" );
   
   TRACE_NEW_2( chunkRep, ChunkVec( 0 ) ) ;
   if( contrStatus == SLAVE )
   {
      setTitle( "GIS Slave Browser" );
      setIconTitle( "Slave Browser" );
      setViewing( FALSE );
   } 
   else 
   {
      setTitle( "GIS Master Browser" );
      setIconTitle( "Master Browser" );
      setViewing( TRUE );
   }

   Arg args[2];
   XtSetArg( args[0], XmNshadowThickness, 3 );
   XtSetArg( args[1], XmNhighlightThickness, 0 );
   Widget appButtonParent = getAppPushButtonParent();
   Widget appButton;
   
   if( contrStatus == MASTER )
   {
      appButton = XmCreatePushButtonGadget( appButtonParent, "L", args, 2 );
      XtAddCallback( appButton, XmNactivateCallback,
		     ( XtCallbackProc )changeLayerCB, this ); 
      addAppPushButton( appButton );
   }

   if( contrStatus == SLAVE )
   {
      appButton = XmCreatePushButtonGadget( appButtonParent, "S", args, 2 );
      XtAddCallback( appButton, XmNactivateCallback,
		     ( XtCallbackProc )toggleSynchModeCB, this ); 
      addAppPushButton( appButton );
   }
   
#ifdef V5DSTUFF
   if ( contrStatus == MASTER )
   {
      TRACE_NEW_2( variableList, SoXtVariableListForSlices( dumbVec, this, getWidget() ) );
      TRACE_NEW_2( variableListIso, SoXtVariableListForIsos( dumbVec, this, getWidget() ) );
      TRACE_NEW_2( variableRep, vector<string>( 1 ) );
      ( *layerRep )[0] = string( "none" );
   
      v5d_numVars = v5d_numLevels = v5d_numCols = v5d_numRows = 0;
      appButton = XmCreatePushButtonGadget( appButtonParent, "W", args, 2 );
      XtAddCallback( appButton, XmNactivateCallback, ( XtCallbackProc )switchWindCB, this ); 
      addAppPushButton( appButton );

      appButton = XmCreatePushButtonGadget( appButtonParent, "A", args, 2 );
      XtAddCallback( appButton, XmNactivateCallback, ( XtCallbackProc )toggleAnimationCB, this ); 
      addAppPushButton( appButton );

      appButton = XmCreatePushButtonGadget( appButtonParent, "V", args, 2 );
      XtAddCallback( appButton, XmNactivateCallback, ( XtCallbackProc )nextVariableCB, this ); 
      addAppPushButton( appButton );

      appButton = XmCreatePushButtonGadget( appButtonParent, "I", args, 2 );
      XtAddCallback( appButton, XmNactivateCallback, ( XtCallbackProc )isosurfaceCB, this ); 
      addAppPushButton( appButton );

   }
//   SoAlarmSensor* doItNowSensor =
//      new SoAlarmSensor(timeChangedCB, this);
//   doItNowSensor->setTimeFromNow(0.5);
   SoTimerSensor* doItNowSensor =
      new SoTimerSensor(timeChangedCB, this);
   doItNowSensor->setInterval(0.33);
   
   doItNowSensor->schedule();
   
#endif //V5DSTUFF

#ifdef DUMPSCENEBUTTON
   appButton = XmCreatePushButtonGadget( appButtonParent, "D", args, 2);
   XtAddCallback(appButton,
		 XmNactivateCallback,
		 (XtCallbackProc)dumpSceneCB, this);
   addAppPushButton(appButton);
#endif //DUMPSCENEBUTTON

//Inline fetching callback
   SoWWWInline::setFetchURLCallBack( ( SoWWWInlineFetchURLCB& )fetchInlineCB, this );
   proto.addMsgType( CAMP, new msgCAMP<Header>( this ) );
   proto.addMsgType( NURL, new msgNURL<Header>( this ) );
   proto.addMsgType( SLAY, new msgSLAY<Header>( this ) );
   proto.addMsgType( NWRL, new msgNWRL<Header>( this ) );
   proto.addMsgType( SYNCR, new msgSYNCR<Header>( this ) );
   proto.addMsgType( SYNCI, new msgSYNCI<Header>( this ) );
   proto.addMsgType( QSYNC, new msgQSYNC<Header>( this ) );
   proto.addMsgType( DRAG, new msgDRAG<Header>( this ) );

#ifdef V5DSTUFF
   proto.addMsgType( ANIMATE, new msgANIMATE<Header>( this ) );
   proto.addMsgType( SHOWSLICE, new msgSHOWSLICE<Header>( this ) );
   proto.addMsgType( SHOWISO, new msgSHOWISO<Header>( this ) );
   proto.addMsgType( SHOWWIND, new msgSHOWWIND<Header>( this ) );
   proto.addMsgType( MOVEPL, new msgMOVEPL<Header>( this ) );
#endif //V5DSTUFF
   const char* mode = getenv( "GIS_MODE" );
   
   if( mode )
   {
      if( ! strcasecmp( "RADAR_OPERATOR", mode ) )
      {
	 proto.addMsgType( NEWGROUND, new msgNGRND<Header>( this ) );
	 proto.addMsgType( NEWAIRBORNE, new msgNAIRB<Header>( this ) );
	 proto.addMsgType( ABUPDATE, new msgABUPD<Header>( this ) );
	 proto.addMsgType( ABMOVE, new msgABMOVE<Header>( this ) );
	 proto.addMsgType( ABSHOW, new msgSHOW<Header>( this ) );
	 proto.addMsgType( ABHIDE, new msgHIDE<Header>( this ) );
	 proto.addMsgType( ABPAUSE, new msgPAUSE<Header>( this ) );
	 proto.addMsgType( ABRESUME, new msgRESUME<Header>( this ) );
	 proto.addMsgType( ABFOLLOW, new msgFOLLOW<Header>( this ) );
	 proto.addMsgType( CSUPDATE, new msgCSUPD<Header>( this ) );
	 
	 appButton = XmCreatePushButtonGadget( appButtonParent, "F", args, 2 );
	 XtAddCallback( appButton, XmNactivateCallback,
			( XtCallbackProc )showMovoListCB, this ); 
	 addAppPushButton( appButton );
//       we do not know the reason for commenting out the next statement
//       but it is nevertheless
//	 contrStatus = RO;
	 
      }
      ACE_DEBUG(( LM_DEBUG, "GIS_MODE: %s\n", mode ));
   }

   XtAppAddInput( SoXt::getAppContext(),
		  ipc_sap.get_handle(),
		  ( XtPointer )XtInputReadMask,
		  ( XtInputCallbackProc )handle_input,
		  ( XtPointer )this );

   mode = getenv( "GIS_CLIPPING" );
   if( mode )
   {
      if( sscanf( mode, "%f,%f", &nearDistance, &farDistance ) == 2 )
	 setAutoClipping( FALSE );
      
      ACE_DEBUG(( LM_DEBUG, "GIS_CLIPPING: %s\n", mode ));
   }

   mode = getenv( "GIS_STEREO" );
   if( mode )
   {
      float factor;
      if( sscanf( mode, "%f", &factor ) == 1 )
      {
	 setStereoOffset( factor * getStereoOffset() );
	 setStereoViewing( TRUE );
      }
      ACE_DEBUG(( LM_DEBUG, "GIS_STEREO: %s\n", mode ));
   }

   setTransparencyType(SoGLRenderAction::DELAYED_ADD);
   
}

void SoXtWebViewer::
sendSynchro( const ViewerID& vid )
{
   ACE_TRACE( "(SoXtWebViewer::sendSynchro)" );
   proto.sendMsg( SYNCI, vid );
   Protocol::PMessage* msg;
   if( proto.getMsgType( DRAG, msg ) == Protocol::P_OK )
   {
      SoSearchAction dragAction;
      dragAction.setType( SoTranslate1Dragger::getClassTypeId() );
      dragAction.setInterest( SoSearchAction::ALL );
      dragAction.apply( getSceneGraph() );
      SoPathList& pathList = dragAction.getPaths();
      for( int i=0; i<pathList.getLength(); i++ )
      {
	 int dnum;
	 SoTranslate1Dragger* dragger = ( SoTranslate1Dragger* )pathList[i]->getTail();
	 sscanf( dragger->getName().getString(), "dragger_%d", &dnum );
	 msgDRAG<Header>::DraggerUD drag_info( dnum, dragger->translation.getValue()[0] );
	 ACE_DEBUG(( LM_DEBUG, "dragger no %d at pos %f\n", drag_info.name_, drag_info.pos_ ));
	 msg->set( &drag_info );
	 proto.sendMsg( DRAG );
      }
      dragAction.reset();
   }
   proto.sendMsg( CAMP, vid );
}

void SoXtWebViewer::
sendQuickSynchro( void )
{
   ACE_TRACE( "(SoXtWebViewer::sendQuickSynchro)" );
   proto.sendMsg( CAMP );
}

void SoXtWebViewer::
buildNewWorld( WorldPrm* wrlParams )
{
   ACE_TRACE( "(SoXtWebViewer::buildNewWorld)" );
   SceneBuilder sc( GeoPoint( wrlParams[0]*60, wrlParams[1]*60, 0 ),
		    wrlParams[2], wrlParams[3], 15 );
   
   setNewGraph( sc.getScene() );
   ACE_DEBUG(( LM_DEBUG, "REFCOUNT sceneGraphRoot: %d\n",
	       getGraphRoot()->getRefCount() ));
   
   worldType( SoXtWebViewer::DYNAMIC );
   memcpy( &wrlPrms, wrlParams, sizeof wrlPrms );
}

void SoXtWebViewer::
switchLayerTo( int nLayer )
{
   ACE_TRACE( "(SoXtWebViewer::switchLayerTo)" );
   ACE_DEBUG(( LM_DEBUG, "Layer ref: %s\n", ( *layerRep )[nLayer].c_str() ));
   layer = nLayer;
   if( ( contrStatus == MASTER )&& update )
      proto.sendMsg( SLAY );

   ACE_DEBUG(( LM_DEBUG, "starting iteration\n" ));
   ChunkVec::iterator it = chunkRep->begin();
   while( it != chunkRep->end() )
   {
      string tex = gis_root + "/Textures/" + ( *layerRep )[nLayer] + "/";
      ACE_DEBUG(( LM_DEBUG, "tex ref: %s\n", tex.c_str() ));
      ( *it )->switchTexTo( tex );
      it++;
   }
}

void SoXtWebViewer::
setChunks( ChunkVec* cvec )
{
   ACE_TRACE( "(SoXtWebViewer::setChunks)" );
   TRACE_DELETE( chunkRep );
   chunkRep = cvec;
   ChunkVec::iterator it = chunkRep->begin();
   while( it != chunkRep->end() )
   {
      GeoPoint gp = world_origin - ( *it )->origin;
      gp.dump();
      ( *it )->setTranslation( gp.getScaledVec( world_scale ) );
      ( *it )->setDEM( gis_root + "/DEM/" , boundsTagParsed );

      if( maxYCoord < ( *it )->maxAlt )
	 maxYCoord = ( *it )->maxAlt;

      (( SoSeparator* )getSceneGraph())->addChild( *it );
      ACE_DEBUG(( LM_DEBUG, "SoChunk: this=0x%x, ref=%d\n", *it, ( *it )->getRefCount() ));
      it++;
   }

   minYCoord = maxYCoord;
   if( maxYCoord < MINIMALMAXYCOORD ) 
      maxYCoord = MINIMALMAXYCOORD;
   else 
      maxYCoord *= 3.0;

}

void SoXtWebViewer::
cleanupSceneGraph()
{
   //remove dragger's callbacks
   SoSearchAction findDraggersAction;
   findDraggersAction.setFind(SoSearchAction::TYPE);
   findDraggersAction.setInterest(SoSearchAction::ALL);
   findDraggersAction.setType(SoTranslate1Dragger::getClassTypeId());
   findDraggersAction.apply(getSceneGraph());
   SoPathList results = findDraggersAction.getPaths();
   if(results.getLength()>0)
      for(int i=0; i<results.getLength(); i++)
      {
	 SoPath* singlePath = results[i];
	 SoTranslate1Dragger* singleDragger =
	    (SoTranslate1Dragger*)(singlePath->getTail());
	 singleDragger->removeMotionCallback(( SoDraggerCB& )draggerMovedCB,
					     this );
      }
}

void SoXtWebViewer::
cleanup( void )
{
   ACE_TRACE( "(SoXtWebViewer::cleanup)" );
   AreaVec::iterator it = areaRep.begin();
   while( it != areaRep.end() )
   {
      TRACE_DELETE( *it );
      it++;
   }
   areaRep.erase( areaRep.begin(), areaRep.end() );
   statoRep.erase( statoRep.begin(), statoRep.end() );
   movoRep.erase( movoRep.begin(), movoRep.end() );
   followed = NULL;
   TRACE_DELETE( movoSensor );
   movoSensor = NULL;
   cleanupBounds();
   movoList->clear();
   movoList->add( string( "(none)" ) );

   
}

void SoXtWebViewer::
cleanupBounds()
{
   ACE_TRACE( "(SoXtWebViewer::cleanupBounds)" );
   maxWest = maxNorth = -1e30;
   maxSouth = maxEast = 1e30;
   
   minXCoord = minZCoord = -1000000;
   maxXCoord = maxZCoord = 1000000;
   
   maxYCoord = -1000000;
   minYCoord = 1000000;
   
   boundsTagParsed = 0;
}


void SoXtWebViewer::
recalculateBounds( GeoPoint corner, int degSize )
{
   ACE_TRACE( "(SoXtWebViewer::recalculateBounds)" );
   float west = corner.getLon();
   float north = corner.getLat();
   float east = corner.getLon() - GeoPoint(0, degSize, 0, 0, 0, 0, 0).getLon();
   float south = corner.getLat() - GeoPoint(0, 0, 0, 0, degSize, 0, 0).getLat();

   if(west > maxWest)
      maxWest = west;
   if(north > maxNorth)
      maxNorth = north;
   if(east < maxEast)
      maxEast = east;
   if(south < maxSouth)
      maxSouth = south;

   minXCoord = (world_origin.getLon() - maxWest + 0.05) * world_scale.lon_deg_size;
   maxXCoord = (world_origin.getLon() - maxEast - 0.05) * world_scale.lon_deg_size;

   maxZCoord = (world_origin.getLat() - maxSouth - 0.05) * world_scale.lat_deg_size;
   minZCoord = (world_origin.getLat() - maxNorth + 0.05) * world_scale.lat_deg_size;

}


int SoXtWebViewer::
findArea( const GeoPoint& point )
{
   ACE_TRACE( "(SoXtWebViewer::findArea)" );
   AreaVec::iterator it = areaRep.begin();
   while( it != areaRep.end() )
   {
      if( ( *it )->IsInside( point ) )
      {
	 ACE_DEBUG(( LM_DEBUG, "(SoXtWebViewer::findArea) Area found\n" ));

//	 sendURL((*it)->ref().c_str());
	 ( *it )->dump();
	 return 1;
      }
      it++;
   }
   ACE_DEBUG(( LM_DEBUG, "(SoXtWebViewer::findArea) No area found\n" ));
   return 0;
}

static const int NumFeatures = 6;

static const char* feature_name[] = 
{
   "airport",
   "hospital",
   "lake",
   "military",
   "reserve",
   "school"
};

//GIS { 0d0a name \t id \t distance 0d0a }*
int findTheNearest(char* aBuffer, int bufLen, char* name)
{
   char* buffer = new char[bufLen+1];
   char* savePointer = buffer;
   
   memcpy(buffer, aBuffer, bufLen);
   buffer[bufLen] = 0;
   
   ACE_TRACE( "findTheNearest" );
   int shortestDistance = 9999;

   buffer += 5; //GIS0d0a
   while (strchr(buffer, 13)!=NULL)
   {
      buffer += 2; //0d0a
      char aName[256];
      long anId;
      int aDistance;

      char* nameEndDx = strchr(buffer, '\t');
      if( nameEndDx )
	 (*nameEndDx) = 0;
      else
	 break;
      
      strcpy(aName, buffer);
      buffer = nameEndDx + 1;
      
      sscanf(buffer, "%ld\t%d", &anId, &aDistance);
      if(aDistance < shortestDistance)
      {
	 shortestDistance = aDistance;
	 strcpy(name, aName);
      }
      buffer = strchr(buffer, 13) + 2;
   }

   delete savePointer;
   
   return (shortestDistance==9999)? -1: shortestDistance;
}

struct AnswerUD
{
      AnswerUD( SoXtWebViewer* wv, int k ) 
	 : viewer( wv ), key( k ) {}

      SoXtWebViewer* viewer;
      int key;
};

void SoXtWebViewer::
answerCB( WebNode* dbLink, AnswerUD* ud )
{
   ACE_TRACE( "(SoXtWebViewer::answerCB)" );
   if( *dbLink )
   {
      char nameBuffer[512];
      ACE_HEX_DUMP(( LM_DEBUG, dbLink->getBuffer(), dbLink->getBufSize(), "answerCB" ));
      int distance = ( dbLink->getBufSize() > 7 ) ?
	 findTheNearest( dbLink->getBuffer(), dbLink->getBufSize(), nameBuffer ) : -1;
      
      char dbInfo[1024];
      if( distance<0 ) //none found
	 sprintf( dbInfo, "No features of type %s around\n",
		  feature_name[ud->key] );
      else
      {
	 char distanceString[256];
	 switch ( distance )
	 {
	    case 0: sprintf( distanceString, "just there");
	       break;
	    case 1: sprintf( distanceString, "1 mile away");
	       break;
	    default: sprintf( distanceString, "%d miles away", distance);
	 }
	 sprintf( dbInfo, "Nearest %s: %s, %s\n",
		  feature_name[ud->key], nameBuffer, distanceString );
      }
      
      ud->viewer->dbList->add( dbInfo );
      TRACE_DELETE_TAB( dbLink->getBuffer() );
   }
   else
      SoXt::createSimpleErrorDialog( ud->viewer->getWidget(), "Downloading Error",
				     ( char* )dbLink->getErrMsg(),
				     ( char* )dbLink->getURL().getString() );
   TRACE_DELETE( dbLink );
   TRACE_DELETE( ud );
}

void SoXtWebViewer::
redraw( void )
{
   ACE_TRACE( "(SoXtWebViewer::redraw)" );
#ifdef CONSTRAIN_CAMERA_POSITION
   if( boundsTagParsed )
   {
      SbVec3f camPos = getCamera()->position.getValue();

      if( camPos[0] < minXCoord )
	 camPos[0] = minXCoord;
      if( camPos[0] > maxXCoord )
	 camPos[0] = maxXCoord;
      if( camPos[1] < minYCoord )
	 camPos[1] = minYCoord;
      if( camPos[1] > maxYCoord )
	 camPos[1] = maxYCoord;
      if( camPos[2] < minZCoord )
	 camPos[2] = minZCoord;
      if( camPos[2] > maxZCoord )
	 camPos[2] = maxZCoord;

      getCamera()->position.setValue( camPos );
   }
#endif //CONSTRAIN_CAMERA_POSITION
   if( ( contrStatus == MASTER )&& update && !followed )
      proto.sendMsg( CAMP );

   SoXtWalkViewer::redraw();
}

//in a loop query the database for all the interesting features;
//for each locate the nearest one and store it
int SoXtWebViewer::
queryDataBase( const GeoPoint& point )
{
   ACE_TRACE( "(SoXtWebViewer::queryDataBase)" );
#ifdef SEND_DBQUERY_TO_NETSCAPE
   char buf[512];
   char* basse = getenv("GIS_DBQUERY_LIST");
   if( ! basse )
      basse = "http://sandman.npac.syr.edu:8888/cgi-bin/Webdriver?";
   

       

   sprintf(buf, "%s%s%f%s%f",
	   basse,
	   "MIval=querylist&select_lat=", point.getLat(), "&select_lon=",
	   -point.getLon());
   sendURL(buf);
#endif // SEND_DBQUERY_TO_NETSCAPE

   if( dbList )
      TRACE_DELETE( dbList );
   
   TRACE_NEW_2( dbList, SoXtDBInfoList( vector<string>( 0 ),
					this,
					getWidget() ) );
   if( ! dbList )
      ACE_ERROR_RETURN(( LM_ERROR,
			 "[!](SoXtWebViewer::queryDataBase) No memory for dbList\n" ),
		       1 );

   dbList->show();
   for(int i=0; i<NumFeatures; i++)
   {
      char query[1024];
      char* basse = getenv("GIS_DB_DBURL");
      if(!basse) basse = DBURL;
      
      sprintf(query,
	      "%s&select_lon=-%f&select_lat=%f&radius=%f"
	      "&feature=%s", basse,
	      point.getLon(), point.getLat(), DBRADIUS, feature_name[i]);

      AnswerUD* ans;
      TRACE_NEW_2( ans, AnswerUD( this, i ) );
      if( ! ans )
	 ACE_ERROR_RETURN(( LM_ERROR,
			    "[!](SoXtWebViewer::queryDataBase) No memory for ans\n" ),
			  1 );
      
      WebNode* dbLink;
      TRACE_NEW_2( dbLink, WebNode( query, ( WebNode::CallbackFunc* )answerCB, ans ) );
      if( ! dbLink )
      {
	 TRACE_DELETE( ans );
	 ACE_ERROR_RETURN(( LM_ERROR,
			    "[!](SoXtWebViewer::queryDataBase) No memory for dbLink\n" ),
			  1 );
      }
   }
   return 0;
}
 
SoPath* SoXtWebViewer::
pickObjectCB( SoXtWebViewer* viewer , const SoPickedPoint* ppoint )
{
   ACE_TRACE( "(SoXtWebViewer::pickObjectCB)" );
   ACE_DEBUG(( LM_DEBUG, "(SoXtWebViewer::pickObjectCB) Picked node: %s\n",
	       ppoint->getPath()->getTail()->getTypeId().getName().getString() ));

   GeoPoint picked = viewer->world_origin -
      GeoPoint( ppoint->getPoint(), viewer->world_scale );
   
   SoSearchAction myAction;
   myAction.setName("GIS_Slider");
   myAction.setInterest(SoSearchAction::LAST);
   myAction.apply(ppoint->getPath());
   if(myAction.getPath() == NULL) //it is not a slider
   {
      viewer->findArea( picked );
#ifdef QUERY_DATABASE
      viewer->queryDataBase( picked );
#endif //QUERY_DATABASE
      viewer->findObject( ppoint );
   }
   myAction.reset();
   return NULL;
}

void SoXtWebViewer::
draggerMovedCB( SoXtWebViewer* viewer, SoTranslate1Dragger* dragger )
{
   ACE_TRACE( "(SoXtWebViewer::draggerMovedCB)" );
   if( viewer->contrStatus == MASTER )
   {
      int dnum;
      
      sscanf( dragger->getName().getString(), "dragger_%d", &dnum );
      msgDRAG<Header>::DraggerUD drag_info( dnum, dragger->translation.getValue()[0] );
      ACE_DEBUG(( LM_DEBUG, "dragger no %d at pos %f\n", drag_info.name_, drag_info.pos_ ));
      Protocol::PMessage* msg;
      if( viewer->proto.getMsgType( DRAG, msg ) == Protocol::P_OK )
      {
	 msg->set( &drag_info );
	 viewer->proto.sendMsg( DRAG );
      }
   }
}

int SoXtWebViewer::
enablePick( void )
{
   ACE_TRACE( "(SoXtWebViewer::enablePick)" );
   SoSelection* selectionRoot = ( SoSelection* )getSceneGraph();
   if( selectionRoot->isOfType( SoSelection::getClassTypeId() ) )
   {
      selectionRoot->deselectAll();
      selectionRoot->policy = SoSelection::SINGLE;
      selectionRoot->setPickMatching( FALSE );
      selectionRoot->setPickFilterCallback( ( SoSelectionPickCB* )&pickObjectCB, this );
   }
   else
   {
      SoXt::createSimpleErrorDialog( getWidget(), "Picking Initialization Warrning",
				     "Picking not enabled",
				     "SoSelection should be a graph root" );
      ACE_ERROR_RETURN(( LM_ERROR,
			 "[!](SoXtWebViewer::enablePick) %s found instead of Selection\n",
			 selectionRoot->getTypeId().getName().getString() ), 0 );
   }
   return 1;
}

SoSeparator* SoXtWebViewer::
makeScene( SoInput& sceneInput )
{
   ACE_TRACE( "(SoXtWebViewer::makeScene)" );
   SoSeparator* newScene = SoDB::readAll( &sceneInput );
   if( ! newScene )
   {
      SoXt::createSimpleErrorDialog( getWidget(), "Scene Making Error",
				     "Invalid data format" );
      ACE_ERROR_RETURN(( LM_ERROR,
			 "[!](SoXtWebViewer::makeScene) Scene making error\n" ), NULL );
   }

   return newScene;
}

SoSeparator* SoXtWebViewer::
readOverNet( const char* newURL )
{
   ACE_TRACE( "(SoXtWebViewer::readOverNet)" );
   WebNode wNode = newURL;
   if( ! wNode )
   {
      string err_msg = wNode.getErrMsg();
      err_msg += ": ";
      err_msg += wNode.getErrReason();
      SoXt::createSimpleErrorDialog( getWidget(), "Network Error",
				     ( char* )err_msg.c_str(), ( char* )newURL );
      ACE_ERROR_RETURN(( LM_ERROR,
			 "[!](SoXtWebViewer::readOverNet) WebNode error\n" ), NULL );
   }

#ifndef ACE_NLOGGING
   ofstream log( DUMP_LOG );
   log.write( wNode.getBuffer(), wNode.getBufSize() );
#endif
   
   SoInput sceneInput;
   sceneInput.setBuffer( wNode.getBuffer(), wNode.getBufSize() );
   SoSeparator* newGraph = makeScene( sceneInput );
   TRACE_DELETE_TAB( wNode.getBuffer() );
   return newGraph;
}

SoSeparator* SoXtWebViewer::
readFromFile( const char* newURL )
{
   ACE_TRACE( "(SoXtWebViewer::readFromFile)" );
   URL url = newURL;
   if( !url )
   {
      SoXt::createSimpleErrorDialog( getWidget(), "Reading File Error",
				     "Invalid URL", ( char* )newURL );
      ACE_ERROR_RETURN(( LM_ERROR,
			 "[!](SoXtWebViewer::readFromFile) Invalid URL: %s\n", newURL ),
		       NULL );
   }
   
   SoInput sceneInput;
   if( ! sceneInput.openFile( url.getResource().getString() ) )
   {
      SoXt::createSimpleErrorDialog( getWidget(), "File Error",
				     "File not found", ( char* )newURL );
      ACE_ERROR_RETURN(( LM_ERROR,
			 "[!](SoXtWebViewer::readFromFile) File error: %s\n", newURL ),
		       NULL );
   }
   return makeScene( sceneInput );
}

SoSeparator* SoXtWebViewer::
loadScene( const char* newURL )
{
   ACE_TRACE( "(SoXtWebViewer::loadScene)" );
   SoSeparator* newScene;
   if( strncmp( newURL, FILE_PREFIX, sizeof FILE_PREFIX - 1 ) )
      newScene = readOverNet( newURL );
   else
      newScene = readFromFile( newURL );
   if( !newScene )
      ACE_ERROR_RETURN(( LM_ERROR,
			 "[!](SoXtWebViewer::loadScene) Cannot load the scene: %s\n",
			 newURL ), NULL );
   TRACE_REF( newScene );
   return newScene;
}


#ifdef V5DSTUFF

void SoXtWebViewer::
setV5DFile( const string& new_v5dFile )
{
   ACE_TRACE( "(SoXtWebViewer::setV5DFile)" );
   v5dstruct *opened = v5dOpenFile(new_v5dFile.c_str(), &v5d_file);
   if(opened==NULL)
   {
      char messageTxt[512];
      sprintf(messageTxt, "%s is not a valid V5D file!", new_v5dFile.c_str());
      SoXt::createSimpleErrorDialog( getWidget(), "V5D File Error",
				     messageTxt );
      ACE_ERROR(( LM_ERROR,
		  "[!](SoXtWebViewer::setV5DFile) incorect V5D file\n" ));
      return;
   }

   v5d_numVars = v5d_file.NumVars;
   v5d_numLevels = v5d_file.Nl[1];
   v5d_numRows = v5d_file.Nr;
   v5d_numCols = v5d_file.Nc;
   windShown = 0;
   timeSlice = 0;
   oldIsoIndex = -1;

   //variables
   vector<string>* lref;
   vector<string>* ldesc;
   TRACE_NEW_2( lref, vector<string>( v5d_numVars+1 ) );
   TRACE_NEW_2( ldesc, vector<string>( v5d_numVars+1 ) );
   (*lref)[0] = string("-1");
   (*ldesc)[0] = string("(none)");
   
   for( int i=1; i <= v5d_numVars; i++ )
   {
      char buf[5];
      sprintf(buf, "%d", i-1);
      ( *lref )[i] = string( buf );
      ( *ldesc )[i] = string( v5d_file.VarName[i-1]);
      ACE_DEBUG(( LM_DEBUG, "Variable: %s as %s\n", ( *ldesc )[i].c_str(),
		  ( *lref )[i].c_str() ));
   }
   makeVariableList( lref, ldesc );
}

void SoXtWebViewer::
showWind(int onOrOff)
{
   if(onOrOff==windShown)
      ACE_DEBUG(( LM_DEBUG, "Wind: conflict on/off\n"));
      
   if(!windShown)
   {
      windShown = TRUE;
      //find the delta[XYZ]
      float real_minXCoord = (world_origin.getLon() - maxWest) *
	 world_scale.lon_deg_size;
      float real_maxXCoord = (world_origin.getLon() - maxEast) *
	 world_scale.lon_deg_size;
   
      float real_maxZCoord = (world_origin.getLat() - maxSouth) *
	 world_scale.lat_deg_size;
      float real_minZCoord = (world_origin.getLat() - maxNorth) *
	 world_scale.lat_deg_size;
   
      float real_maxYCoord = maxYCoord;
      float real_minYCoord = minYCoord;
   
      float deltaX = (real_maxXCoord - real_minXCoord) / float(v5d_numCols);
      float deltaY = (real_maxYCoord - real_minYCoord) / float(v5d_numLevels);
      float deltaZ = (real_maxZCoord - real_minZCoord) / float(v5d_numRows);

      //read U, V, W
      float* bufU = new float[v5d_numCols*v5d_numRows*v5d_numLevels];
      v5dReadGrid(&v5d_file, 0, 1, bufU);
      float* bufV = new float[v5d_numCols*v5d_numRows*v5d_numLevels];
      v5dReadGrid(&v5d_file, 0, 2, bufV);
      float* bufW = new float[v5d_numCols*v5d_numRows*v5d_numLevels];
      v5dReadGrid(&v5d_file, 0, 3, bufW);


      SoVectors* vectors = new SoVectors(bufU, bufV, bufW,
					 v5d_numCols,
					 v5d_numLevels,
					 v5d_numRows,
					 this,
					 SoPlaneable::YZ,
					 deltaX, deltaY, deltaZ);
      vectors->construct();
      ((SoSeparator*)getSceneGraph())->addChild(vectors);

      delete bufU;
      delete bufV;
      delete bufW;
   }
   else
   {
      SoNode* windNode = SoNode::getByName("vectors");
      if(windNode)
	 ((SoGroup*)getSceneGraph())->removeChild(windNode);
      windShown = FALSE;
   }
   
}

void SoXtWebViewer::
switchVariableTo(int index)
{
   //if index is equal to v5d_numVars, remove all SoPlane objects
   Protocol::PMessage* msg;
   if( contrStatus == MASTER )
      if( proto.getMsgType( SHOWSLICE, msg ) == Protocol::P_OK )
      {
	 signed char tmp_index = index;
	 msg->set( &tmp_index );
	 proto.sendMsg( SHOWSLICE );
      }
  
   if(index == -1)
   {
      removePlanes();
      v5d_currentVar = -1;
      return;
   }

   SoNodeList aList;
   SoNode::getByName("plane", aList);
   
   //if no variable is displayed yet, add SoPlane object to the scene
   if(aList.getLength()==0)
   {
      //find the delta[XYZ]
      float real_minXCoord = (world_origin.getLon() - maxWest) *
	 world_scale.lon_deg_size;
      float real_maxXCoord = (world_origin.getLon() - maxEast) *
	 world_scale.lon_deg_size;

      float real_maxZCoord = (world_origin.getLat() - maxSouth) *
	 world_scale.lat_deg_size;
      float real_minZCoord = (world_origin.getLat() - maxNorth) *
	 world_scale.lat_deg_size;

      float real_maxYCoord = maxYCoord;
      float real_minYCoord = minYCoord;

      float deltaX = (real_maxXCoord - real_minXCoord) / float(v5d_numCols/2);
      float deltaY = (real_maxYCoord - real_minYCoord) / float(v5d_numLevels);
      float deltaZ = (real_maxZCoord - real_minZCoord) / float(v5d_numRows/2);
      
      float* buf = new float[v5d_numCols*v5d_numRows*v5d_numLevels];
      //remek time
      v5dReadGrid(&v5d_file, timeSlice, index, buf);

      //slices need some vertical offset in order to not
      //collide with ground
      SoTranslation* elevationTrans = new SoTranslation;
      elevationTrans->translation.setValue(0, minYCoord, 0);
//      SoSeparator* vertSliceGroup = new SoSeparator;
//       SoPlane* plane0 = new SoPlane(buf,
// 				    v5d_numCols,
// 				    v5d_numLevels,
// 				    v5d_numRows,
// 				    SoPlaneable::YZ, deltaX, deltaY*2, deltaZ);
//       plane0->construct();

//       vertSliceGroup->addChild(elevationTrans);
//       vertSliceGroup->addChild(plane0);
//       vertSliceGroup->setName("Vplane");
      
//       ((SoSeparator*)getSceneGraph())->addChild(vertSliceGroup);
      
      SoSeparator* horizSliceGroup = new SoSeparator;
      SoPlane* plane1 = new SoPlane(buf,
				    v5d_numCols,
				    v5d_numLevels,
				    v5d_numRows,
				    this,
				    SoPlaneable::XZ, deltaX, deltaY*2, deltaZ);
      plane1->construct();
            
      horizSliceGroup->addChild(elevationTrans);
      horizSliceGroup->addChild(plane1);
      horizSliceGroup->setName("Hplane");

      ((SoSeparator*)getSceneGraph())->addChild(horizSliceGroup);

      delete buf;
   }
   //if there are some, just set their data buffers
   else
   {
      float* buf = new float[v5d_numCols*v5d_numRows*v5d_numLevels];
      v5dReadGrid(&v5d_file, 0, index, buf);
      for(int i=0; i<aList.getLength(); i++)
	 ((SoPlane*)aList[i])->setData(buf,
				       v5d_numCols,
				       v5d_numLevels,
				       v5d_numRows);
      delete buf;
   }

   if(index>=0)
   {
      char buf[512];
      sprintf(buf, "Displaying variable: %s (%d of %d)",
	      v5d_file.VarName[index],
	      index+1,
	      v5d_numVars);
      SoXt::createSimpleErrorDialog(getWidget(), "Information", buf );
      
      v5d_currentVar = index;
   }
}

void SoXtWebViewer::
removePlanes()
{
   SoSeparator* root = (SoSeparator*)getSceneGraph();
   root->removeChild(SoNode::getByName("Hplane"));
//   root->removeChild(SoNode::getByName("Vplane"));
}

void SoXtWebViewer::
showIsosurface(int varIndex, float threshold )
{
   switch(varIndex)
   {
      
      case 1: threshold = 30;
	 break;
      case 2: threshold = 11.3;
	 break;
      case 3: threshold = 0;
	 break;
      case 0:
      case 4:
	 return;
      case 5: threshold = 0.01;
	 break;
      case 6: threshold = 0.1;
	 break;
   };
   
   Protocol::PMessage* msg;
   if( contrStatus == MASTER )
      if( proto.getMsgType( SHOWISO, msg ) == Protocol::P_OK )
      {
	 const char* prms[2] = 
	 {
	    ( const char* )&varIndex, ( const char* )&threshold 
	 };
	 msg->set( prms );
	 proto.sendMsg( SHOWISO );
      }

   int showNone = varIndex == -1 || varIndex >= v5d_numVars;
   SoSwitch* oldIso = (SoSwitch*)SoNode::getByName("isosurfaces");
   // isos not shown yet and do not show
   if(!oldIso && showNone)
      return;
   // isos not shown yet, so time to show them
   else if(!oldIso && !showNone)
      prepareIsosurfaces(varIndex, threshold)->whichChild = timeSlice;
   // isos already shown, so hide
   else if(oldIso && showNone)
   {
      oldIso->whichChild = SO_SWITCH_NONE;
      return;
   }
   // isos already shown, and no change to them
   else if(oldIso && oldIsoIndex == varIndex)
      oldIso->whichChild = timeSlice;
   // isos already shown, so switch to some other variable
   else if(oldIso && oldIsoIndex != varIndex)
   {
      ((SoGroup*)getSceneGraph())->removeChild(oldIso);
      prepareIsosurfaces(varIndex, threshold)->whichChild = timeSlice;
   }
   else 
      cerr << "Interesting combination!" << endl;
   oldIsoIndex = varIndex;

//    float* buf = new float[v5d_numCols*v5d_numRows*v5d_numLevels];
//    v5dReadGrid(&v5d_file, timeSlice, varIndex, buf);
   
//    //find the delta[XYZ]
//    float real_minXCoord = (world_origin.getLon() - maxWest) *
//       world_scale.lon_deg_size;
//    float real_maxXCoord = (world_origin.getLon() - maxEast) *
//       world_scale.lon_deg_size;
   
//    float real_maxZCoord = (world_origin.getLat() - maxSouth) *
//       world_scale.lat_deg_size;
//    float real_minZCoord = (world_origin.getLat() - maxNorth) *
//       world_scale.lat_deg_size;
   
//    float real_maxYCoord = maxYCoord;
//    float real_minYCoord = minYCoord;
   
//    float deltaX = (real_maxXCoord - real_minXCoord) / float(v5d_numCols);
//    float deltaY = (real_maxYCoord - real_minYCoord) / float(v5d_numLevels);
//    float deltaZ = (real_maxZCoord - real_minZCoord) / float(v5d_numRows);
   
//    SoSeparator* isoGroup = new SoSeparator;
//    SoTranslation* elevationTrans = new SoTranslation;
//    elevationTrans->translation.setValue(0, minYCoord, 0);
//    isoGroup->addChild(elevationTrans);
   
//    SoIsosurface* isos = new SoIsosurface(
//       buf,
//       v5d_numCols, v5d_numLevels, v5d_numRows, 0.1, deltaX, deltaY/4, deltaZ);
//    isoGroup->addChild(isos);
//    isoGroup->setName("isosurface");
//    ((SoSeparator*)getSceneGraph())->addChild(isoGroup);

}

SoSwitch* SoXtWebViewer::
prepareIsosurfaces(int varIndex, float threshold)
{
   SoSwitch* isoSwitch = new SoSwitch;
   isoSwitch->whichChild = SO_SWITCH_NONE;
   
   float* buf = new float[v5d_numCols*v5d_numRows*v5d_numLevels];
   //find the delta[XYZ]
   float real_minXCoord = (world_origin.getLon() - maxWest) *
      world_scale.lon_deg_size;
   float real_maxXCoord = (world_origin.getLon() - maxEast) *
      world_scale.lon_deg_size;

   float real_maxZCoord = (world_origin.getLat() - maxSouth) *
      world_scale.lat_deg_size;
   float real_minZCoord = (world_origin.getLat() - maxNorth) *
      world_scale.lat_deg_size;

   float real_maxYCoord = maxYCoord;
   float real_minYCoord = minYCoord;

   float deltaX = (real_maxXCoord - real_minXCoord) / float(v5d_numCols);
   float deltaY = (real_maxYCoord - real_minYCoord) / float(v5d_numLevels);
   float deltaZ = (real_maxZCoord - real_minZCoord) / float(v5d_numRows);

   for(int timeSlice = 0; timeSlice < v5d_file.NumTimes; timeSlice++)
   {
      v5dReadGrid(&v5d_file, timeSlice, varIndex, buf);
   
      SoSeparator* isoGroup = new SoSeparator;
      SoTranslation* elevationTrans = new SoTranslation;
      elevationTrans->translation.setValue(0, minYCoord, 0);
      isoGroup->addChild(elevationTrans);

      SoIsosurface* isos = new SoIsosurface(
	 buf,
	 v5d_numCols, v5d_numLevels, v5d_numRows, threshold, deltaX, deltaY, deltaZ);
      isoGroup->addChild(isos);
      isoGroup->setName("isosurface");
      isoSwitch->addChild(isoGroup);
   }

   delete[] buf;

   ((SoGroup*)getSceneGraph())->addChild(isoSwitch);
   isoSwitch->setName("isosurfaces");

   return isoSwitch;
}


#endif //V5DSTUFF

void SoXtWebViewer::
fetchInlineCB( const SbString& sb_newURL,
	       SoXtWebViewer* viewer,
	       SoWWWInline* inlNode )
{
   ACE_TRACE( "(SoXtWebViewer::fetchInlineCB)" );
   SoSeparator* newChild;
   const char* URL = viewer->sb_URL.getString();
   const char* newURL = sb_newURL.getString();
   ACE_DEBUG(( LM_DEBUG, "actual URL: %s\ninline URL: %s\n", URL, newURL ));
   if( strncmp( newURL, HTTP_PREFIX, sizeof HTTP_PREFIX - 1 )&&
       strncmp( newURL, FILE_PREFIX, sizeof FILE_PREFIX - 1 ) )
   {// making up full URL
      const char* pastSlashPtr = strrchr( URL, '/' );
      SbString sb_updURL( URL, 0, pastSlashPtr - URL );
      sb_updURL += sb_newURL;
      ACE_DEBUG(( LM_DEBUG, "updated URL: %s\n", sb_updURL ));
      newChild = viewer->loadScene( sb_updURL.getString() );
   }
   else
      newChild = viewer->loadScene( newURL );
      
   if( newChild )
   {
      inlNode->setChildData( newChild );
      TRACE_UNREF( newChild );
   }
   else
   {
      SoXt::createSimpleErrorDialog( viewer->getWidget(), "Fetching Inline Error",
				     "Cannot create inline node" );
      ACE_ERROR(( LM_ERROR, "[!](SoXtWebViewer::fetchInlineCB) Null newChild\n" ));
   }      
}

int SoXtWebViewer::
loadNewGraph( const char* newURL )
{
   ACE_TRACE( "(SoXtWebViewer::loadNewGraph)" );
   SbString sb_currURL = sb_URL;
   sb_URL = newURL;
   
   if( ( contrStatus == MASTER )&& update )
      proto.sendMsg( NURL );
   
   SoSeparator* newGraph = loadScene( newURL );
   if( newGraph )
   {
      wrlType = SoXtWebViewer::STATIC;
      return setNewGraph( newGraph );
   }

   sb_URL = sb_currURL;
   return 0;
}

int SoXtWebViewer::
setNewGraph( SoSeparator* newGraph )
{
   ACE_TRACE( "(SoXtWebViewer::setNewGraph)" );
   SoSeparator* oldGraph = ( SoSeparator* )getSceneGraph();
   if( oldGraph->isOfType( SoSeparator::getClassTypeId() ) )
   {
      // download needed textures
      StringRep texDB( newGraph->getNumChildren() );
      addTex( newGraph, texDB );

      cleanupSceneGraph();
      DUMP_REF_INFO( oldGraph );
      setSceneGraph( newGraph );
      TRACE_UNREF( oldGraph );
      setClipping();
      cleanupBounds();
      cleanup();
      movoList->clear();
      movoList->add( string( "(none)" ) );
#ifdef V5DSTUFF
      v5d_numVars = 0;

      doAnimation = FALSE;
      
#endif //V5DSTUFF
      TagParser tpar( *this );
      enablePick();
      return 1;
   }
   else
      ACE_ERROR_RETURN(( LM_ERROR,
			 "[!](SoXtWebViewer::loadNewGraph) Expected Separator not %s\n",
			 oldGraph->getTypeId().getName().getString() ), 0 );
}

void SoXtWebViewer::
handle_input( SoXtWebViewer* viewer, int*, XtInputId* )
{
   ACE_TRACE( "(SoXtWebViewer::handle_input)" );
   viewer->setAutoRedraw( FALSE ); 
   viewer->update = false;
   if( viewer->proto.processAllMsgs() == Protocol::P_CLOSED )
   {
      ACE_DEBUG(( LM_SHUTDOWN, "(SoXtWebViewer::handle_input) connection reset by peer\n" ));
      exit( 0 );
   }
   viewer->render();
   viewer->update = true;
   viewer->setAutoRedraw( TRUE ); 
}

void SoXtWebViewer::
showErrorMsg( char *title, char *message )
{
   Arg args[8];
   int nArgs = 0;
   XmString string = XmStringCreateSimple( title );
   XtSetArg( args[nArgs], XmNdialogTitle, string ); nArgs++;
   XtSetArg( args[nArgs], XmNdialogStyle, XmDIALOG_APPLICATION_MODAL ); nArgs++;
   XtSetArg( args[nArgs], XmNresizePolicy, XmRESIZE_ANY ); nArgs++;
   XtSetArg( args[nArgs], XmNnoResize, False ); nArgs++;
   XtSetArg( args[nArgs], XmNautoUnmanage, True ); nArgs++;
   XtSetArg( args[nArgs], XmNdefaultPosition, True ); nArgs++;
   XtSetArg( args[nArgs], XmNallowShellResize, False ); nArgs++;
   XmString msg = XmStringCreateLtoR( message, XmSTRING_DEFAULT_CHARSET );
   XtSetArg( args[nArgs], XmNmessageString, msg ); nArgs++;
     
   Widget errDialog = XmCreateErrorDialog( getWidget(), "ErrorDialog", args, nArgs );
   XmStringFree( string );
   XmStringFree( msg );
     
   Widget w = XmMessageBoxGetChild( errDialog, XmDIALOG_OK_BUTTON );
   XtUnmanageChild( w );
   w = XmMessageBoxGetChild( errDialog, XmDIALOG_HELP_BUTTON );
   XtUnmanageChild( w );
     
   XtAddCallback( errDialog, XmNcancelCallback, ( XtCallbackProc )errDialogCB, NULL );
   XtManageChild( errDialog );
   XSync( getDisplay(), False );
   XmUpdateDisplay( errDialog );
}

const char*
makeFullURL( const char* location, const char* filename )
{
   static char buf[MAXLEN];

   char* endPtr = strrchr( location, '/' )+1;
   int len = endPtr - location;
   memcpy( buf, location, len );
   strcpy( buf+len, filename );
   return buf;
}

#define TEX_DIR "/scratch/GIS_cache/"

void SoXtWebViewer::
addTex( SoGroup* root, StringRep& texDB )
{
   int chNum = root->getNumChildren();
   for( int idx=0; idx < chNum; idx++ )
   {
      if( root->getChild( idx )->isOfType( SoTexture2::getClassTypeId() ) )
      {
	 SoTexture2* tex = ( SoTexture2* )root->getChild( idx );
	 const char* fileName = tex->filename.getValue().getString();
	 if( strcmp( fileName, "" ) )
	 {
	    if( ! strncmp( fileName, TEX_DIR, 5 ) )
	       return;
		    
	    char shortName[128] = TEX_DIR;
	    if( texDB.search( fileName ) )
	    {
	       tex->filename.setValue( strcat( shortName, fileName ) );
	       return;
	    }
		    
	    texDB.add( fileName );
		    
	    if( strncmp( fileName, "http://", 7 ) )
	    {
	       strcat( shortName, fileName );
	       fileName = makeFullURL( sb_URL.getString(), fileName );
	    }
	    else
	       strcat( shortName, strrchr( fileName, '/' )+1 );
		    
	    WebNode node( fileName );
	    if( ! node )
	    {
	       SoXt::createSimpleErrorDialog( getWidget(), "Texture Error",
					      "I don't know what's wrong" );
	       return;
	    }
	    ofstream ofs( shortName );
	    ofs.write( node.getBuffer(), node.getBufSize() );
	    ofs.close();
	    TRACE_DELETE_TAB( node.getBuffer() );
	    tex->filename.setValue( shortName );
	 }
	 else
	    cerr << "addTex: empty filename\n";
      }
      else if( root->getChild( idx )->isOfType( SoGroup::getClassTypeId() ) )
	 addTex( ( SoGroup* )root->getChild( idx ), texDB );
   }
}

#ifdef DUMPSCENEBUTTON
void SoXtWebViewer::
dumpSceneCB( Widget, SoXtWebViewer* wv, XmPushButtonCallbackStruct )
{
   ACE_TRACE( "(SoXtWebViewer::dumpSceneCB)" );

   SoWriteAction dumpSceneAction;
   dumpSceneAction.getOutput()->openFile(DUMP_SCENE_FILE_NAME);
   dumpSceneAction.getOutput()->setBinary(FALSE);
   dumpSceneAction.apply(wv->getSceneGraph());
   dumpSceneAction.getOutput()->closeFile();
}

#endif //DUMPSCENEBUTTON

int SoXtWebViewer::
addStaticObj( SoStaticObj* stato )
{
   ACE_TRACE( "(SoXtWebViewer::addStaticObj)" );
   if( ! stato )
      ACE_ERROR_RETURN(( LM_ERROR,
			 "[!](SoXtWebViewer::addStaticObj) Null pointer received\n" ), -1 );

   const string& name = stato->getName();
   StaticObjMap::iterator it = statoRep.find( name );
   if( it != statoRep.end() )
      ACE_ERROR_RETURN(( LM_ERROR, "[!](SoXtWebViewer::addStaticObj) %s already exists\n",
			 name.c_str() ), -1 );
   
   getGraphRoot()->addChild( stato );
   statoRep[name] = stato;
   ACE_DEBUG(( LM_DEBUG , "Static object %s added\n", name.c_str() ));
   return 1;
}

int SoXtWebViewer::
addMovingObj( SoMovingObj* movo )
{
   ACE_TRACE( "(SoXtWebViewer::addMovingObj)" );
   if( ! movo )
      ACE_ERROR_RETURN(( LM_ERROR,
			 "[!](SoXtWebViewer::addMovingObj) Null pointer received\n" ), -1 );
   
   const string& name = movo->getName();
   if( getMovingObj( name ) != NULL )
      ACE_ERROR_RETURN(( LM_ERROR, "[!](SoXtWebViewer::addMovingObj) %s already exists\n",
			 name.c_str() ), -1 );
      
   getGraphRoot()->addChild( movo );
   movoRep[name] = movo;
   movoList->add( name );
   ACE_DEBUG(( LM_DEBUG , "Moving object %s added\n", name.c_str() ));
   return 1;
}

int SoXtWebViewer::
pointOnSurface( SbVec3f& vec )
{
   ACE_TRACE( "(SoXtWebViewer::pointOnSurface)" );
   SoRayPickAction act( getViewportRegion() );
   act.setRay( vec, SbVec3f( 0, -1 , 0 ) );
   act.apply( getGraphRoot() );
   SoPickedPoint* ppoint;
   if( ppoint = act.getPickedPoint() )
   {
      vec = ppoint->getPoint();
      ACE_DEBUG(( LM_DEBUG , "point[%.2f, %.2f, %.2f]\n", vec[0], vec[1], vec[2] ));
      return 1;
   }
   return 0;
}

void SoXtWebViewer::
followMovo( const string& movoID )
{
   ACE_TRACE( "(SoXtWebViewer::followMovo)" );
   if( contrStatus != SLAVE )
   {
      Protocol::PMessage* msg;
      if( proto.getMsgType( ABFOLLOW, msg ) == Protocol::P_OK )
      {
	 ACE_Message_Block* mb;
	 TRACE_NEW_2( mb, ACE_Message_Block( movoID.c_str(), strlen( movoID.c_str() ) + 1 ) );
	 msg->set( mb );
	 proto.sendMsg( ABFOLLOW );
      }
   }
   SoMovingObj* movo = getMovingObj( movoID );
   if( ! movo )
   {
      if( followed )
      {
	 followed->releaseCam();
	 followed = NULL;
	 TRACE_DELETE( movoSensor );
	 movoSensor = NULL;
	 ACE_ERROR(( LM_DEBUG, "(SoXtWebViewer::followMovo) follow mode stopped\n" ));
	 render();
 	 setAutoRedraw( TRUE ); 
 	 ACE_DEBUG(( LM_DEBUG, "Auto redraw enabled\n" ));
      }
      return;
   }
   if( ! movo->visible() )
   {
      SoXt::createSimpleErrorDialog( getWidget(), "Follow Mode Warning",
				     "Cannot follow invisible object referred to as",
				     ( char* )movoID.c_str() );
      ACE_DEBUG(( LM_DEBUG, "(SoXtWebViewer::followMovo) %s is invisible\n",
		  movoID.c_str() ));
      return;
   }
   if( movo == followed )
   {
      ACE_DEBUG(( LM_DEBUG, "(SoXtWebViewer::followMovo) %s already followed\n",
		  movoID.c_str() ));
      return;
   }
   if( followed )
      followed->releaseCam();

   followed = movo;
   movo->seizeCam( getCamera() );
   TRACE_DELETE( movoSensor );
   
   TRACE_NEW_2( movoSensor, SoFieldSensor( ( SoSensorCB& )movoSensorCB, movo ) );
   movoSensor->attach( &movo->getPosition()->translation );
   setAutoRedraw( FALSE ); 
   ACE_DEBUG(( LM_DEBUG, "Auto redraw disabled\n" ));
   render();
}

void SoXtWebViewer::
movoSensorCB( SoMovingObj* movo, SoSensor* )
{
   ACE_TRACE( "(SoXtWebViewer::movoSensorCB)" );
   movo->viewer_->setAutoRedraw( FALSE ); 
   movo->updateCam();
}

int SoXtWebViewer::
findObject( const SoPickedPoint* ppoint )
{
   ACE_TRACE( "(SoXtWebViewer::findObject)" );
   SoSearchAction myAction;
   myAction.setName("GIS_Object");
   myAction.setInterest(SoSearchAction::LAST);
   myAction.apply(ppoint->getPath());
   if(myAction.getPath() != NULL)
   {
      SoGroup* objectGroup = (SoGroup*)myAction.getPath()->getTail();
      if(objectGroup->isOfType(SoGroup::getClassTypeId()) &&
	 objectGroup->getChild(1) != NULL &&
	 objectGroup->getChild(1)->isOfType(SoInfo::getClassTypeId()))
      {
	 SoInfo* info = (SoInfo*)objectGroup->getChild(1);
//	 sendURL( info->string.getValue().getString());
	 return 1;
      }
   }
   return 0; // no objects found
}

void SoXtWebViewer::
sendURL( const char* url )
{
   ACE_TRACE( "(SoXtWebViewer::sendURL)" );
   char* netPath = getenv( "GIS_NETPATH" );
   if( ! netPath )
      netPath = "/usr/bin/X11/netscape";

   int childPID;

   if( ( childPID = fork() ) == -1 )
      ACE_DEBUG(( LM_DEBUG, "(SoXtWebViewer::sendURL) Cannot fork\n"));
   else if( childPID == 0 )
   {
      char buf[512];
      sprintf( buf, "openURL(%s,new-window)", url );
      
      if( execl( netPath, netPath, "-noraise", "-remote" , buf, NULL ) == -1 )
      {
	 ACE_DEBUG(( LM_DEBUG, "%s: %s\n", netPath, strerror( errno ) ));
	 if( execl( netPath, netPath, url, NULL )
	     == -1 )
	 {
	    ACE_DEBUG(( LM_DEBUG, "2nd try %s: %s\n",
			netPath, strerror( errno ) ));
	    exit( 0 );
	 }
      }
   }
}

void SoXtWebViewer::
pauseMovObjs( void )
{
   ACE_TRACE( "(SoXtWebViewer::pauseMovObjs)" );
   MovingObjMap::iterator it = movoRep.begin();
   while( it != movoRep.end() )
      ( *it++ ).second->pause();
}

void SoXtWebViewer::
resumeMovObjs( void )
{
   ACE_TRACE( "(SoXtWebViewer::resumeMovObjs)" );
   MovingObjMap::iterator it = movoRep.begin();
   while( it != movoRep.end() )
      ( *it++ ).second->resume();
}

void SoXtWebViewer::
toggleSynchModeCB( Widget, SoXtWebViewer* wv, XmPushButtonCallbackStruct )
{
   ACE_TRACE( "(SoXtWebViewer::toggleSynchModeCB)" );
   wv->synchronousMode = ! wv->synchronousMode;
   wv->setViewing( ! wv->synchronousMode );
   if( wv->synchronousMode )
      wv->proto.sendMsg( QSYNC );
}
