/* $Id: SoXtWebViewer.h,v 1.40 1998/04/23 20:13:39 remek Exp $ */
/*
 * File:       SoXtWebViewer.h
 * Author:     Pawel Roman (proman@npac.syr.edu)
 * Created:    long time ago
 * Revision:   $Revision: 1.40 $
 * Updated:    $Date: 1998/04/23 20:13:39 $
 * Copyright 1996/7 NPAC, Syracuse University
 *
 */

#ifndef _SOXTWEBVIEWER_H
#define _SOXTWEBVIEWER_H

#include "Area.h"
#include "Comm.h"
#include "FixedVector.h"
#include "GISdb.h"
#include "GeoPoint.h"
#include "ViewerMessage.h"
#include "MsgHeader.h"
#include "Proto.h"
#include "ProtocolInterface.h"
#include "SoChunk.h"
#include "SoXtDBInfoList.h"
#include "SoXtLayerList.h"
#include "SoXtOLDialog.h"
#include "strrep.h"
#include "SceneBuilder.h"
#include "SoXtMovoList.h"

#ifdef V5DSTUFF
#include "v5d.h"
#include "SoXtVariableListForSlices.h"
#include "SoXtVariableListForIsos.h"
#endif //V5DSTUFF


#ifndef TANGO
#include <ace/SOCK_Stream.h>
#else
#include <ace/LSOCK_Stream.h>
#endif

#include <map.h>
#include <STL/stlstring.h>

#include <Inventor/Xt/SoXt.h>
#include <Inventor/Xt/viewers/SoXtWalkViewer.h>
#include <Inventor/nodes/SoCamera.h>
#include <Inventor/nodes/SoSeparator.h>
#include <Inventor/nodes/SoSwitch.h>

#include <Xm/Xm.h>

struct AnswerUD;
class SoStaticObj;
class SoMovingObj;
class SoTranslate1Dragger;
class SoWWWInline;

class SoXtWebViewer : public SoXtWalkViewer
{
   public:
#ifndef TANGO
      typedef ACE_SOCK_Stream IPC_SAP;
#else
      typedef ACE_LSOCK_Stream IPC_SAP;
#endif
      typedef vector<string> LayerVec;
      typedef vector<SoChunk*> ChunkVec;
      typedef vector<Area*> AreaVec;
      typedef map<AreaInfo, string, less<AreaInfo> > AreaInfoMap;
      typedef map<string, SoStaticObj*, less<string> > StaticObjMap;
      typedef map<string, SoMovingObj*, less<string> > MovingObjMap;
      typedef MsgHeader<IPC_SAP> Header;
      typedef MsgHeader<IPC_SAP>::VID ViewerID;
      typedef ViewerMessage<Header> Message;
      typedef ProtocolInterface<Message, FixedVector<Message*> > Protocol;
      
      enum ContrStatus { MASTER, SLAVE, RO };
      enum WorldType { NONE, STATIC, DYNAMIC };
      
      SoXtWebViewer( const IPC_SAP& ipc_sap,
		     ViewerID id,
		     ContrStatus stat,
		     Widget parent = NULL,
		     const char *name = NULL, 
		     SbBool buildInsideParent = TRUE, 
		     SoXtFullViewer::BuildFlag flag = BUILD_ALL, 
		     SoXtViewer::Type type = BROWSER );

      virtual ~SoXtWebViewer( void ) { delete olDialog; }
      void redraw( void );
      void showErrorMsg( char *title, char *message );
      void setCamParams( const CamParams& cPrms );
      void getCamParams( CamParams& cPrms );
      int enablePick( void );
      int loadNewGraph( const char* URL );
      int setNewGraph( SoSeparator* newGraph );
      const char* getURL( void ) const;
      void setURL( const char* url );
      SoSeparator* getGraphRoot( void );
      void setOrigin( const GeoPoint& gp );
      const GeoPoint& getOrigin( void ) const;
      void setBounds( const GeoPoint& nw, const GeoPoint& se );
      void setScale( const CoordScale& cs );
      const CoordScale& getScale( void ) const;
      void makeLayerList( LayerVec* lref, LayerVec* ldesc );
      void switchLayerTo( int nLayer );
      void setGISRoot( const string& new_root );
      const string& getGISRoot( void ) const;
      void setChunks( ChunkVec* cvec );
      void addAreas( AreaVec& avec );
      void cleanupSceneGraph( void );
      void cleanup( void );
      void cleanupBounds( void );
      void recalculateBounds( GeoPoint corner, int degSize );
      unsigned char getBoundsTagParsed( void );
      void sendSynchro( const ViewerID& vid );
      void sendQuickSynchro( void );
      float getMaxWest( void ) const { return maxWest; }
      float getMaxNorth( void ) const { return maxNorth; }
      float getMaxEast( void ) const { return maxEast; }
      float getMaxSouth( void ) const { return maxSouth; }
      int getContrStatus( void ) const { return contrStatus; }
      void setCameraTilt( float tilt ) { tiltCamera( tilt ); }
      ViewerID getVID( void ) const { return vid; }
      void worldType( const WorldType& wt ) { wrlType = wt; } 
      const WorldType& worldType( void ) const { return wrlType;} 
      const WorldParams& getWorldParams( void ) const { return wrlPrms; }
      const LayerCode& getLayer( void ) const { return layer; }
      Protocol& getProto( void ) { return proto; }
      void buildNewWorld( WorldPrm* wrlParams );
      int addStaticObj( SoStaticObj* stato );
      int addMovingObj( SoMovingObj* movo );
      int pointOnSurface( SbVec3f& vec );
      int pointOnSurface( GeoPoint& gp );
      SoMovingObj* getMovingObj( const string& name );
      void followMovo( const string& name );
      void pauseMovObjs( void );
      void resumeMovObjs( void );
      void setClipping( void );
      bool synchMode( void );
      bool followMode( void );

      static void draggerMovedCB( SoXtWebViewer*, SoTranslate1Dragger* );

   protected:
      void openViewerHelpCard( void );
      int findArea( const GeoPoint& point );
      int findObject( const SoPickedPoint* ppoint );
      int queryDataBase( const GeoPoint& point );
      SoSeparator* makeScene( SoInput& sceneInput );
      SoSeparator* loadScene( const char* URL );
      SoSeparator* readOverNet( const char* URL );
      SoSeparator* readFromFile( const char* URL );
      void addTex( SoGroup* root, StringRep& texDB );

   private:
      static SoPath* pickObjectCB( SoXtWebViewer*, const SoPickedPoint* );
      static void errDialogCB( Widget w, XtPointer, XtPointer ) { XtDestroyWidget( w ); }
      static void fetchInlineCB( const SbString&, SoXtWebViewer*, SoWWWInline* );
      static void handle_input( SoXtWebViewer*, int*, XtInputId* );
      static void answerCB( WebNode*, AnswerUD* );
      static void changeLayerCB( Widget, SoXtWebViewer* , XmPushButtonCallbackStruct );
      static void toggleSynchModeCB( Widget, SoXtWebViewer* , XmPushButtonCallbackStruct );
      static void showMovoListCB( Widget, SoXtWebViewer*, XmPushButtonCallbackStruct );
      static void movoSensorCB( SoMovingObj*, SoSensor* );
      void sendURL(const char *anURL);

      bool synchronousMode;
      GeoPoint world_origin;
      CoordScale world_scale;
      AreaInfoMap areaInfoRep;
      StaticObjMap statoRep;
      MovingObjMap movoRep;
      AreaVec areaRep;
      Protocol proto;
      SbString sb_URL;
      string gis_root;
      WorldParams wrlPrms;
      LayerVec* layerRep;
      ChunkVec* chunkRep;
      SoXtOLDialog* olDialog;
      SoXtLayerList* layerList;
      SoXtDBInfoList* dbList;
      SoXtMovoList* movoList;
      SoFieldSensor* movoSensor;
      SoMovingObj* followed;
      ContrStatus contrStatus;
      WorldType wrlType;
      ViewerID vid;
      LayerCode layer;
      float nearDistance, farDistance;
      
#ifdef V5DSTUFF
   public:
      int v5d_numVars;
      int v5d_numLevels;
      int v5d_numCols;
      int v5d_numRows;
      int v5d_currentVar;
      int doAnimation;
      int windShown;
      int timeSlice;
      int oldIsoIndex;
      
      v5dstruct v5d_file;

      void setV5DFile( const string& new_v5dFile);
      typedef vector<string> VariableVec;
      void makeVariableList( VariableVec* lref, VariableVec* ldesc );
      void switchVariableTo( int nVariable );
      void showIsosurface( int index, float threshold );
      void showWind(int onOrOff);
      
   private:
      VariableVec* variableRep;
      SoXtVariableListForSlices* variableList;
      SoXtVariableListForIsos* variableListIso;
      
      static void switchWindCB( Widget, SoXtWebViewer* ,
			 XmPushButtonCallbackStruct );
      static void toggleAnimationCB( Widget, SoXtWebViewer*,
                         XmPushButtonCallbackStruct);
      static void nextVariableCB( Widget, SoXtWebViewer* ,
			 XmPushButtonCallbackStruct );
      static void isosurfaceCB( Widget, SoXtWebViewer*,
			 XmPushButtonCallbackStruct);
      void removePlanes();
      SoSwitch* prepareIsosurfaces(int varIndex, float threshold);
      
#endif //V5DSTUFF

#ifdef DUMPSCENEBUTTON
      static void dumpSceneCB( Widget, SoXtWebViewer*,
			       XmPushButtonCallbackStruct);
#endif //DUMPSCENEBUTTON
      
      bool update;
      int maxXCoord, minXCoord;
      int maxYCoord, minYCoord;
      int maxZCoord, minZCoord;
      float maxWest, maxEast, maxSouth, maxNorth;
      unsigned char boundsTagParsed;
      
};

inline const char* SoXtWebViewer::
getURL( void ) const
{
   ACE_TRACE( "(SoXtWebViewer::getURL)" );
   return sb_URL.getString();
}

inline void SoXtWebViewer::
setURL( const char* url )
{
   ACE_TRACE( "(SoXtWebViewer::setURL)" );
   sb_URL = url;
}

inline void SoXtWebViewer::
setGISRoot( const string& new_root )
{
   ACE_TRACE( "(SoXtWebViewer::setGISRoot)" );
   gis_root = new_root;
}

inline const string& SoXtWebViewer::
getGISRoot( void ) const
{
   ACE_TRACE( "(SoXtWebViewer::getGISRoot)" );
   return gis_root;
}

inline void SoXtWebViewer::
setCamParams( const CamParams& cPrms )
{
   ACE_TRACE( "(SoXtWebViewer::setCamParams)" );
   getCamera()->position.setValue( cPrms ); 
   getCamera()->orientation.setValue( &cPrms[3] ); 
}

inline void SoXtWebViewer::
getCamParams( CamParams& cPrms )
{
   ACE_TRACE( "(SoXtWebViewer::getCamParams)" );
   getCamera()->position.getValue().getValue( cPrms[0], cPrms[1], cPrms[2] );
   getCamera()->orientation.getValue().getValue( cPrms[3], cPrms[4], cPrms[5], cPrms[6] );
}

inline void SoXtWebViewer::
openViewerHelpCard( void )
{
   ACE_TRACE( "(SoXtWebViewer::openViewerHelpCard)" );
   if( contrStatus != SLAVE )
      olDialog->show();
   else
      SoXt::createSimpleErrorDialog( getWidget(), "Error",
				     "Slave viewers must not change the scene graph" );
}

inline void SoXtWebViewer::
changeLayerCB( Widget, SoXtWebViewer* wv, XmPushButtonCallbackStruct )
{
   ACE_TRACE( "(SoXtWebViewer::changeLayerCB)" );
   wv->layerList->show();
}

#ifdef V5DSTUFF
inline void SoXtWebViewer::
makeVariableList( VariableVec* lref, VariableVec* ldesc )
{
   ACE_TRACE( "(SoXtWebViewer::makeVariableList)" );
   TRACE_DELETE( variableList );
   TRACE_DELETE( variableListIso );
   TRACE_NEW_2( variableList, SoXtVariableListForSlices( *ldesc, this, getWidget() ) );
   TRACE_NEW_2( variableListIso, SoXtVariableListForIsos( *ldesc, this, getWidget() ) );
   TRACE_DELETE( ldesc );
   TRACE_DELETE( variableRep );
   variableRep = lref;
}

inline void SoXtWebViewer::
switchWindCB( Widget, SoXtWebViewer* wv, XmPushButtonCallbackStruct )
{
   ACE_TRACE( "(SoXtWebViewer::switchWindCB)" );
   if( wv->v5d_numVars > 0 ) wv->showWind(!(wv->windShown));
   Protocol::PMessage* msg;
   if( wv->contrStatus == MASTER )
     if( wv->proto.getMsgType( SHOWWIND, msg ) == Protocol::P_OK )
       wv->proto.sendMsg( SHOWWIND );
}

inline void SoXtWebViewer::
toggleAnimationCB( Widget, SoXtWebViewer* wv, XmPushButtonCallbackStruct )
{
   ACE_TRACE( "(SoXtWebViewer::toggleAnimation)" );
   if(wv->v5d_numVars > 0)
      wv->doAnimation = !(wv->doAnimation);

   Protocol::PMessage* msg;
   if( wv->contrStatus == MASTER )
     if( wv->proto.getMsgType( ANIMATE, msg ) == Protocol::P_OK )
       wv->proto.sendMsg( ANIMATE );
}

inline void SoXtWebViewer::
isosurfaceCB( Widget, SoXtWebViewer* wv, XmPushButtonCallbackStruct)
{
   ACE_TRACE( "(SoXtWebViewer::isosurfaceCB)");
   wv->variableListIso->show();
}


inline void SoXtWebViewer::
nextVariableCB( Widget, SoXtWebViewer* wv, XmPushButtonCallbackStruct )
{
   ACE_TRACE( "(SoXtWebViewer::nextVariableCB)" );
   wv->variableList->show();
}

#endif //V5DSTUFF

inline SoSeparator* SoXtWebViewer::
getGraphRoot( void )
{
   ACE_TRACE( "(SoXtWebViewer::getGraphRoot)" );
   return ( SoSeparator* )getSceneGraph();
}

inline void SoXtWebViewer::
setOrigin( const GeoPoint& gp )
{
   ACE_TRACE( "(SoXtWebViewer::setOrigin)" );
   world_origin = gp;
}

inline const GeoPoint& SoXtWebViewer::
getOrigin( void ) const
{
   ACE_TRACE( "(SoXtWebViewer::getOrigin)" );
   return world_origin;
}

inline void SoXtWebViewer::
setBounds( const GeoPoint& nw, const GeoPoint& se )
{
   ACE_TRACE( "(SoXtWebViewer::setBounds)" );
   maxWest = nw.getLon();
   maxNorth = nw.getLat();
   maxEast = se.getLon();
   maxSouth = se.getLat();
   boundsTagParsed = 1;
}

inline void SoXtWebViewer::
setScale( const CoordScale& cs )
{
   ACE_TRACE( "(SoXtWebViewer::setScale)" );
   world_scale = cs;
}

inline const CoordScale& SoXtWebViewer::
getScale( void ) const
{
   ACE_TRACE( "(SoXtWebViewer::getScale)" );
   return world_scale;
}

inline void SoXtWebViewer::
makeLayerList( LayerVec* lref, LayerVec* ldesc )
{
   ACE_TRACE( "(SoXtWebViewer::makeLayerList)" );
   TRACE_DELETE( layerList );
   TRACE_NEW_2( layerList, SoXtLayerList( *ldesc, this, getWidget() ) );
   TRACE_DELETE( ldesc );
   TRACE_DELETE( layerRep );
   layerRep = lref;
}

inline void SoXtWebViewer::
addAreas( AreaVec& avec )
{
   ACE_TRACE( "(SoXtWebViewer::addAreas)" );
   areaRep.insert( areaRep.end(), avec.begin(), avec.end() );
}

inline unsigned char SoXtWebViewer::
getBoundsTagParsed()
{
   return boundsTagParsed;
}

inline int SoXtWebViewer::
pointOnSurface( GeoPoint& gp )
{
   ACE_TRACE( "(SoXtWebViewer::pointOnSurface)" );
   SbVec3f vec = gp.getScaledVec( getScale() );
   if( pointOnSurface( vec ) )
   {
      gp = getOrigin() - GeoPoint( vec, getScale() );
      return 1;
   }
   else
      return 0;
}

inline SoMovingObj* SoXtWebViewer::
getMovingObj( const string& name )
{
   ACE_TRACE( "(SoXtWebViewer::getMovingObj)" );
   MovingObjMap::iterator it = movoRep.find( name );
   if( it == movoRep.end() )
      ACE_ERROR_RETURN(( LM_ERROR, "%s has *not* been found\n", name.c_str() ),
		       NULL );
   
   ACE_DEBUG(( LM_DEBUG , "%s has been found\n", name.c_str() ));
   return ( *it ).second;
}

inline void SoXtWebViewer::
showMovoListCB( Widget, SoXtWebViewer* wv, XmPushButtonCallbackStruct )
{
   ACE_TRACE( "(SoXtWebViewer::showMovoListCB)" );
   wv->movoList->show();
}

inline bool SoXtWebViewer::
synchMode( void )
{
   return synchronousMode;
}

inline void SoXtWebViewer::
setClipping( void )
{
   ACE_TRACE( "(SoXtWebViewer::setClipping)" );
   if( ! isAutoClipping() )
   {
      getCamera()->nearDistance = nearDistance;
      getCamera()->farDistance = farDistance;
   }
}

inline bool SoXtWebViewer::
followMode( void )
{
   return followed != NULL;
}

#endif
