/* $Id: VRML.C,v 1.11 1997/09/25 20:32:06 remek Exp $ */
/*
 * File:       VRML.C
 * Author:     Remek Trzaska (remek@npac.syr.edu)
 * Created:    long time ago
 * Revision:   $Revision: 1.11 $
 * Updated:    $Date: 1997/09/25 20:32:06 $
 * Copyright 1996/97 NPAC, Syracuse University
 *
 */

#include "VRML.h"
#include "Timer.h"

#define INT32_T int

int trianglemain(int, OutFile*, int *, int *);

OutFile::OutFile() 
{
   nodeCounter = faceCounter = -1;
   TRACE_NEW_2( root, SoSeparator );
   minAlt = 10e30;
   maxAlt = -10e30;
}

OutFile::~OutFile() 
{
}

void OutFile::writeInitialParams()
{
   SoShapeHints *h;
   TRACE_NEW_2( h, SoShapeHints );
   h->creaseAngle.setValue(3.14);

   root->addChild(h);
}

void OutFile::writeTextureData(DEM *df)
{
   char buf[256];
   SoTexture2 *texture;
   TRACE_NEW_2( texture, SoTexture2 );
   root->addChild(texture);
   strcpy(buf, df->name);
   strcat(buf, ".c.ppm");
   unsigned char *image;
   char c;
   int tdimx, tdimy;
   int i;

   FILE *f = fopen(buf, "rb");
   if (f==NULL) fatal("Cannot open the texture file!");
   // the pgm header -- P5 or sth
   fscanf(f, "%s", &buf);
   fgetc(f);

   if ((c=fgetc(f))=='#') { //comment
      for(; (c=fgetc(f))!='\n'; ) ;
   } else ungetc(c, f);

   // the dimensions: number of columns and the rows, respectively
   fscanf(f, "%d %d", &tdimx, &tdimy);
   fgetc(f);

   if ((c=fgetc(f))=='#') { //comment
      for(; (c=fgetc(f))!='\n'; ) ;
   } else ungetc(c, f);

   // the maximum value -- determines the data element size
   fscanf(f, "%d", &i);
   if (i!=255) {
      fatal("Can only handle rgb 3*255 files");
   }
   // and the last carridge return character
   fgetc(f); 

   // prepare the necessary memory space 
   NewVector(image, 3*tdimx*tdimy, unsigned char);

   // and read the actual data
   int j;
   //  if (tdimx<tdimy) 
   for(i=0; i<tdimy; i++) 
      for(j=0; j<tdimx; j++) {
	 int dx = 3*(i*tdimx + tdimx - 1 -j);
	 image[dx] = fgetc(f);
	 image[dx+1] = fgetc(f);
	 image[dx+2] = fgetc(f);
	 //      if((image[i]=fgetc(f))==EOF)
	 //	fatal("Texture file too short");
      }
  
    
   // just to make sure the file is valid
   //  if (fgetc(f)!=EOF)
   //    fatal("Texture file too long");
   fclose(f);

  


   //  texture->filename.setValue(buf);  ///!!!!! do poprawki!!!!!!!
   texture->image.setValue(SbVec2s(tdimx, tdimy), 3, image);
   TRACE_DELETE( image ); //last addition!!!!
  
   SoTexture2Transform *textr;
   TRACE_NEW_2( textr, SoTexture2Transform );
   root->addChild(textr);
   textr->scaleFactor.setValue(1, 1.0);
   textr->rotation.setValue(1.57);
}

void OutFile::startListOfNodes()
{
   TRACE_NEW_2( coor, SoCoordinate3 );
}

void OutFile::writeNode(COOR x, COOR y, ELEV z)
{
   coor->point.set1Value(++nodeCounter, SbVec3f(x, z, y));
   if(maxAlt < z)
      maxAlt = z;
   if(minAlt > z)
      minAlt = z;
}

void OutFile::finishListOfNodes()
{
   root->addChild(coor);
}

void OutFile::startListOfFaces()
{
   TRACE_NEW_2( face, SoIndexedFaceSet );
}

void OutFile::writeFace(int a, int b, int c)
{
   face->coordIndex.set1Value(++faceCounter,(INT32_T) a);
   face->coordIndex.set1Value(++faceCounter,(INT32_T) b);
   face->coordIndex.set1Value(++faceCounter,(INT32_T) c);
   face->coordIndex.set1Value(++faceCounter, SO_END_FACE_INDEX);
}

void OutFile::finishListOfFaces()
{
   root->addChild(face);
}

void OutFile::finishFile()
{
}

void OutFile::dumpToFile(char *fname)
{
   SoWriteAction myAct;
   FILE *iff;
   iff = fopen(fname, "w+");
   myAct.getOutput()->setBinary(FALSE);
   myAct.getOutput()->setFilePointer(iff);
   myAct.apply(root);
   fclose(iff);
}


void VRMLFile::writeHeader()
{
}

void IVFile::writeHeader()
{
}

