/* $Id: WebNode.h,v 1.15 1997/09/25 20:32:08 remek Exp $ */
/*
 * File:       WebNode.h
 * Author:     Pawel Roman (proman@npac.syr.edu)
 * Created:    long time ago
 * Revision:   $Revision: 1.15 $
 * Updated:    $Date: 1997/09/25 20:32:08 $
 * Copyright 1996/7 NPAC, Syracuse University
 *
 */

#ifndef __WEBNODE
#define __WEBNODE

//textures have to be rotated and mirrored in order to match 
#define TRANSFORM_TEXTURES

#include "URL.h"
#include "cache.h"
#include "Comm.h"

#ifndef __BOOL_DEFINED
#include <bool.h>
#endif
#include <iostream.h>

#include <ace/SOCK_Connector.h>
#include <ace/IOStream.h>

#include <X11/Intrinsic.h>

static const unsigned short MAXLEN = 512; // buffer length

class WebNode
{
   public:
      typedef void CallbackFunc( WebNode*, void* );
      

      WebNode( const char* location, CallbackFunc = NULL, void* = NULL,
	       int aRotate = FALSE);
      ~WebNode( void );
      char* getBuffer( void ) { bufGiven_ = true; return dataBuf_; }
      int getBufSize( void ) const { return dataSize_; }
      int isOK( void ) const { return ! intStatus_; }
      operator const void* ( void ) const { return ( const void* ) ! intStatus_; }
      const char* getErrMsg( void ) const;
      char* getErrReason( void ) const { return ( char* )buf_; }
      int getErrCode( void ) const { return intStatus_; }
      SbString getURL( void ) const { return *URL_; }
      void download( void );
      int recvHeader( void );
      void finish( void );
	 
   private:
      static void asynch_input( WebNode*, int*, XtInputId* );

      typedef ACE_IOStream<ACE_SOCK_Stream> ACE_SOCK_IOStream;
      void synch_input( void );
      int syncStreamBuf( void );
      void cleanUp( XtInputId id );

      int rotate;
      char buf_[MAXLEN];
      ACE_SOCK_IOStream* server_;
      CallbackFunc* doneCB_;
      void* userData_;
      URL* URL_;
      char* dataBuf_;
      int dataSize_;
      int intStatus_;
      ssize_t fetched_;
      bool hdRecved_;
      bool bufGiven_;
      
      void uncompressDEM();
      void uncompressTex();
      void processPPM();
#ifdef TRANSFORM_TEXTURES
      void transformTex();
#endif //TRANSFORM_TEXTURES

      static Cache cache_;
};

inline int WebNode::
syncStreamBuf( void )
{
   streambuf* strBuf = server_->rdbuf();
   int bytes_avail = strBuf->in_avail();
   ACE_DEBUG(( LM_DEBUG, "available %d byte(s)\n", bytes_avail ));
   strBuf->sgetn( dataBuf_, bytes_avail );
   return bytes_avail;
}


inline void WebNode::
cleanUp( XtInputId id )
{
   XtRemoveInput( id );
   doneCB_( this, userData_ );
}

#endif
