/* $Id: cache.C,v 1.8 1997/09/25 20:32:10 remek Exp $ */
/*
 * File:       cache.C
 * Author:     Pawel Roman (proman@npac.syr.edu)
 * Created:    long time ago
 * Revision:   $Revision: 1.8 $
 * Updated:    $Date: 1997/09/25 20:32:10 $
 * Copyright 1996/97 NPAC, Syracuse University
 *
 */

#include "cache.h"
#include "Comm.h"

#include <STL/stlstring.h>
#include <fstream.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/stat.h>

static const char CACHE_PREFIX[] = "GIS_";
static const char CACHE_INDEX[] = "GIS_index.db";
static const int  CF_MODE = 0666;

Cache::
Cache()
{
   if( !( cache_dir = getenv( "GIS_CACHE_DIR" ) ) )
   {
      cerr << "Cannot find GIS_CACHE_DIR\n";
      exit( 1 );
   }
   
   string idxfn( cache_dir );
   idxfn += '/';
   idxfn += CACHE_INDEX;
   index.open( idxfn.c_str(), ios::app | ios::in );
   if( ! index )
      cerr << "Cache init error: " << idxfn << endl;
   else
      chmod( idxfn.c_str(), CF_MODE );
}

int Cache::
isCached( const char* url )
{
   ACE_TRACE( "(Cache::isCached)" );
   struct stat file_stat;
   static char url_name[256];
   static char cfile_name[128];

   index.seekg( 0, ios::beg );
   while( ! index.eof() )
   {
      index >> url_name >> cfile_name;
      if( ! strcmp( url_name, url ) )
      {
	 if( stat( cfile_name, &file_stat ) < 0 )
	    ACE_ERROR_RETURN(( LM_ERROR, "[!](Cache::isCached) %s: %s\n",
			       cfile_name, strerror( errno ) ), 0 );
	 
	 buflen = file_stat.st_size;
	 TRACE_NEW_2( buffer, char[buflen] );
	 if( ! buffer )
	    ACE_ERROR_RETURN(( LM_ERROR,
			       "[!](Cache::isCached) No memory for buffer\n" ), 0 );

	 ifstream is( cfile_name );
	 is.read( buffer, buflen );
	 if( is )
	    return 1;
	 else
	 {
	    TRACE_DELETE_TAB( buffer );
	    ACE_ERROR_RETURN(( LM_ERROR, "[!](Cache::isCached) File error\n" ), 0 );
	 }
      }
   }
   index.clear();
   return 0;
}

void Cache::
cacheIt( const char* url, const char* buffer, int buflen )
{
   ACE_TRACE( "(Cache::cacheIt)" );
   char* tmp_name = tempnam( cache_dir, CACHE_PREFIX );
   index << url << '\t' << tmp_name << endl;
   ofstream os( tmp_name, ios::trunc );
   chmod( tmp_name, CF_MODE );
   os.write( buffer, buflen );
   free( tmp_name );
}

