/* $Id: msgABMOVE.h,v 1.4 1997/09/25 20:32:10 remek Exp $ */
/*
 * File         : msgABMOVE.h
 * Author       : Pawel Roman
 * Created      : Fri Aug 29 18:00:33 1997
 * Revision     : $Revision: 1.4 $
 * Updated      : $Date: 1997/09/25 20:32:10 $
 * Copyright: Northeast Parallel Architectures Center
 *            at Syracuse University 1997
 */

#ifndef __MSGABMOVE_H
#define __MSGABMOVE_H

#include "SimEngMessage.h"
#include "SoMovingObj.h"

template<class HEADER>
class msgABMOVE
   : public SimEngMessage<HEADER>
{
   public:
      msgABMOVE( SoXtWebViewer* vw );
      void execute( void );

   private:
      double beg_lon;
      double beg_lat;
      double end_lon;
      double end_lat;
      double time;
      char* name;
};

template<class HEADER>
inline msgABMOVE<HEADER>::
msgABMOVE( SoXtWebViewer* vw )
   : SimEngMessage<HEADER>( Header( ABMOVE, HEADER::BROADCAST ), vw )
{ 
   ACE_TRACE( "(msgABMOVE::msgABMOVE)" );
}

template<class HEADER>
inline void msgABMOVE<HEADER>::
execute( void )
{
   ACE_TRACE( "(msgABMOVE::execute)" );
   if( change )
   {
      ACE_HEX_DUMP(( LM_DEBUG, mb->base(), mb->size(), "ABMOVE" ));
      ACE_ASSERT( mb->size() > 1 );
      name = mb->base();
      size_t name_len = strlen( name ) + 1;
      memcpy( &beg_lon, name + name_len, sizeof beg_lon );
      memcpy( &beg_lat, name + name_len + sizeof beg_lon, sizeof beg_lat );
      memcpy( &end_lon, name + name_len + sizeof beg_lon + sizeof beg_lat, sizeof end_lon );
      memcpy( &end_lat, name + name_len + sizeof beg_lon + sizeof beg_lat + sizeof end_lon,
	      sizeof end_lat );
      memcpy( &time, name + name_len + sizeof beg_lon + sizeof beg_lat + sizeof end_lon +
	      sizeof end_lat, sizeof time );
      ACE_DEBUG(( LM_DEBUG, "name: %s, beg_pos[%f, %f], end_pos[%f, %f], time: %f\n",
		  name, beg_lon, beg_lat, end_lon, end_lat, time ));
      
      GeoPoint beg_gp = viewer->getOrigin().dump() - GeoPoint( beg_lon, beg_lat ).dump();
      GeoPoint end_gp = viewer->getOrigin() - GeoPoint( end_lon, end_lat ).dump();
      SoMovingObj* movo = viewer->getMovingObj( name );
      if( ! movo )
      {
	 ACE_ERROR(( LM_ERROR, "[!](msgABMOVE::execute) %s does not exist\n", name ));
	 return;
      }
      movo->setMoveParams( beg_gp, end_gp, time );
   }
}

#endif

