/* $Id: msgABUPD.h,v 1.3 1997/09/25 20:32:11 remek Exp $ */
/*
 * File         : msgABUPD.h
 * Author       : Pawel Roman
 * Created      : Mon Aug 25 16:04:46 1997
 * Revision     : $Revision: 1.3 $
 * Updated      : $Date: 1997/09/25 20:32:11 $
 * Copyright: Northeast Parallel Architectures Center
 *            at Syracuse University 1997
 */

#ifndef __MSGABUPD_H
#define __MSGABUPD_H

#include "SimEngMessage.h"
#include "SoMovingObj.h"

template<class HEADER>
class msgABUPD
   : public SimEngMessage<HEADER>
{
   public:
      msgABUPD( SoXtWebViewer* vw );
      void execute( void );

   private:
      double lon;
      double lat;
      double alt;
      char* name;
      SoMovingObj::Class cls;
};

template<class HEADER>
inline msgABUPD<HEADER>::
msgABUPD( SoXtWebViewer* vw )
   : SimEngMessage<HEADER>( Header( ABUPDATE, HEADER::BROADCAST ), vw )
{ 
   ACE_TRACE( "(msgABUPD::msgABUPD)" );
}

template<class HEADER>
inline void msgABUPD<HEADER>::
execute( void )
{
   ACE_TRACE( "(msgABUPD::execute)" );
   if( change )
   {
      ACE_HEX_DUMP(( LM_DEBUG, mb->base(), mb->size(), "ABUPDATE" ));
      ACE_ASSERT( mb->size() > 1 );
      cls = ( SoMovingObj::Class )*mb->base();
      name = mb->base() + 1;
      size_t name_len = strlen( name ) + 1;
      memcpy( &lon, name + name_len, sizeof lon );
      memcpy( &lat, name + name_len + sizeof lon, sizeof lat );
      memcpy( &alt, name + name_len + sizeof lon + sizeof lat, sizeof alt );
      ACE_DEBUG(( LM_DEBUG, "class: %d, name: %s, pos[%f, %f], alt: %f\n",
		  cls, name, lon, lat, alt ));
      
      GeoPoint gp = viewer->getOrigin().dump() - GeoPoint( lon, lat, alt ).dump();
      SoMovingObj* movo = viewer->getMovingObj( name );
      if( ! movo )
      {
	 ACE_ERROR(( LM_ERROR, "[!](msgABUPD::execute) %s does not exist\n", name ));
	 return;
      }
      movo->update( gp.dump().getScaledVec( viewer->getScale() ), cls );
   }
}

#endif
