/*
 * File         : msgANIMATE.h
 * Author       : Pawel Roman (proman@npac.syr.edu)
 * Created      : Tue Feb 10 14:06:22 1998
 * Revision     : $Revision: 1.2 $
 * Updated      : $Date: 1998/04/23 20:13:41 $
 * Copyright: Northeast Parallel Architectures Center
 *            at Syracuse University 1998
 */

#ifndef __MSGANIMATE_H
#define __MSGANIMATE_H

#include "ViewerMessage.h"

template<class HEADER>
class msgANIMATE
   : public ViewerMessage<HEADER>
{
   public:
      msgANIMATE( SoXtWebViewer* vw );
      MRESULT read( const SAP& sap, const Header& header );
      MRESULT write( const SAP& sap );
      MRESULT set( const void* params = NULL );
      void execute( void );

   private:
      char on_off;
};

template<class HEADER>
inline msgANIMATE<HEADER>::
msgANIMATE( SoXtWebViewer* vw )
   : ViewerMessage<HEADER>( Header( ANIMATE, vw->getVID(), sizeof on_off ), vw )
{ 
   ACE_TRACE( "(msgANIMATE::msgANIMATE)" );
}

template<class HEADER>
inline msgANIMATE<HEADER>::MRESULT msgANIMATE<HEADER>::
read( const SAP& sap, const Header& header )
{
   ACE_TRACE( "(msgANIMATE::read)" );
   if( sap.recv_n( &on_off, sizeof on_off ) == header.length() )
   {
      checkVID( header );
      return OK_RES;
   }
   else
      return END_RES;
}

template<class HEADER>
inline msgANIMATE<HEADER>::MRESULT msgANIMATE<HEADER>::
write( const SAP& sap ) 
{ 
   ACE_TRACE( "(msgANIMATE::write)" );
   on_off = viewer->timeSlice;
   if ( viewer->doAnimation ) on_off += 100;
   
   if( header.write( sap ) == Header::LINK_OK )
      if( sap.send_n( &on_off, sizeof on_off ) == header.length() )
	 return OK_RES;
   
   return END_RES;
}

template<class HEADER>
inline msgANIMATE<HEADER>::MRESULT msgANIMATE<HEADER>::
set( const void* msg ) 
{ 
   ACE_TRACE( "(msgANIMATE::set)" );
   on_off = *( char* )msg;
   return OK_RES;
}

template<class HEADER>
inline void msgANIMATE<HEADER>::
execute( void )
{
   ACE_TRACE( "(msgANIMATE::execute)" );
   if( change )
   {
      viewer->doAnimation = (on_off >= 100);
      viewer->timeSlice = (on_off >= 100)? on_off - 100 : on_off;
      viewer->timeSlice = viewer->timeSlice - 1;
      timeChangedCB(viewer, NULL);
   }
}

#endif

