/* $Id: msgCAMP.h,v 1.12 1997/09/25 20:32:11 remek Exp $ */
/*
 * File:       msgCAMP.h
 * Author:     Pawel Roman (proman@npac.syr.edu)
 * Created:    long time ago
 * Revision:   $Revision: 1.12 $
 * Updated:    $Date: 1997/09/25 20:32:11 $
 * Copyright 1996/7 NPAC, Syracuse University
 *
 */

#ifndef _MSGCAMP_H
#define _MSGCAMP_H

#include "ViewerMessage.h"
#include "Proto.h"

template<class HEADER>
class msgCAMP : public ViewerMessage<HEADER>
{
   public:
      msgCAMP( SoXtWebViewer* vw );
      MRESULT read( const SAP& sap, const Header& header );
      MRESULT write( const SAP& sap );
      MRESULT set( const void* params = NULL );
      void execute( void );
  
   private:
      CamParams cPrms;
      CamParams cPrms_old;
};

template<class HEADER>
inline msgCAMP<HEADER>::
msgCAMP( SoXtWebViewer* vw )
   : ViewerMessage<HEADER>( Header( CAMP, vw->getVID(), sizeof cPrms ), vw )
{ 
   ACE_TRACE( "(msgCAMP::msgCAMP)" );
}

template<class HEADER>
inline msgCAMP<HEADER>::MRESULT msgCAMP<HEADER>::
read( const SAP& sap, const Header& header )
{
   ACE_TRACE( "(msgCAMP::read)" );
   if( sap.recv_n( &cPrms, sizeof cPrms ) == header.length() )
   {
      checkVID( header );
      return OK_RES;
   }
   else
      return END_RES;
}

template<class HEADER>
inline msgCAMP<HEADER>::MRESULT msgCAMP<HEADER>::
write( const SAP& sap ) 
{ 
   ACE_TRACE( "(msgCAMP::write)" );
   viewer->getCamParams( cPrms );
   if( header.write( sap ) == Header::LINK_OK )
      if( sap.send_n( &cPrms, sizeof cPrms ) == header.length() )
	 return OK_RES;

   return END_RES;
}

template<class HEADER>
inline msgCAMP<HEADER>::MRESULT msgCAMP<HEADER>::
set( const void* ) 
{ 
   viewer->getCamParams( cPrms );
   return OK_RES;
}

template<class HEADER>
inline void msgCAMP<HEADER>::
execute( void )
{
   ACE_TRACE( "(msgCAMP::execute)" );
   if( viewer->synchMode() && ! viewer->followMode() )
   {
      if( ! change )
      {
	 viewer->getCamParams( cPrms_old );
	 cPrms[1] = cPrms_old[1];
   
	 static const SbVec3f defaultOrient( 0, 0, -1 );
	 static SbVec3f newOrient;
	 SbRotation newRotation( &cPrms[3] );
	 newRotation.multVec( defaultOrient, newOrient );
	 // angle to Y-axis
	 float newAlpha = acos( newOrient[1] / sqrt( newOrient[0]*newOrient[0] +
						     newOrient[1]*newOrient[1] +
						     newOrient[2]*newOrient[2] ) );

	 static SbVec3f oldOrient;
	 SbRotation oldRotation( &cPrms_old[3] );
	 oldRotation.multVec( defaultOrient, oldOrient );
	 float oldAlpha = acos( oldOrient[1] / sqrt( oldOrient[0]*oldOrient[0] +
						     oldOrient[1]*oldOrient[1] +
						     oldOrient[2]*oldOrient[2] ) );
   
	 viewer->setCamParams( cPrms );
	 viewer->setCameraTilt( newAlpha - oldAlpha );
      }
      else
	 viewer->setCamParams( cPrms );
   }
}

#endif
