/* $Id: msgDRAG.h,v 1.3 1997/09/25 20:32:12 remek Exp $ */
/*
 * File         : msgDRAG.h
 * Author       : Pawel Roman (proman@npac.syr.edu)
 * Created      : Fri Sep 12 11:29:40 1997
 * Revision     : $Revision: 1.3 $
 * Updated      : $Date: 1997/09/25 20:32:12 $
 * Copyright: Northeast Parallel Architectures Center
 *            at Syracuse University 1997
 */

#ifndef __MSGDRAG_H
#define __MSGDRAG_H

#include "ViewerMessage.h"
#include <ace/Message_Block.h>
#include <Inventor/draggers/SoTranslate1Dragger.h>

template<class HEADER>
class msgDRAG
   : public ViewerMessage<HEADER>
{
   public:
      struct DraggerUD
      {
	    DraggerUD( char name, float pos ) 
	       : name_( name ), pos_( pos ) {}

	    char name_;
	    float pos_;
      };

      msgDRAG( SoXtWebViewer* vw );
      MRESULT read( const SAP& sap, const Header& header );
      MRESULT write( const SAP& sap );
      MRESULT set( const void* params = NULL );
      void execute( void );

   private:
      char buf[5];
};

template<class HEADER>
inline msgDRAG<HEADER>::
msgDRAG( SoXtWebViewer* vw )
   : ViewerMessage<HEADER>( Header( DRAG, HEADER::BROADCAST, sizeof buf ), vw )
{ 
   ACE_TRACE( "(msgDRAG::msgDRAG)" );
}


template<class HEADER>
inline msgDRAG<HEADER>::MRESULT msgDRAG<HEADER>::
read( const SAP& sap, const Header& header )
{
   ACE_TRACE( "(msgDRAG::read)" );
   Header::LengthType msg_len = header.length();
   if( msg_len == sizeof buf )
      if( sap.recv_n( buf, sizeof buf ) == msg_len )
      {
	 checkVID( header );
	 return OK_RES;
      }
      else
	 return END_RES;
   else
      return FAIL_RES;
}

template<class HEADER>
inline msgDRAG<HEADER>::MRESULT msgDRAG<HEADER>::
write( const SAP& sap ) 
{ 
   ACE_TRACE( "(msgDRAG::write)" );
   if( header.write( sap ) == Header::LINK_OK )
      if( sap.send_n( buf, sizeof buf ) == header.length() )
	 return OK_RES;
   
   return END_RES;
}

template<class HEADER>
inline msgDRAG<HEADER>::MRESULT msgDRAG<HEADER>::
set( const void* msg ) 
{ 
   ACE_TRACE( "(msgDRAG::set)" );
   const DraggerUD* drag_info  = ( const DraggerUD* )msg;
   memcpy( buf, &drag_info->name_, sizeof drag_info->name_ );
   memcpy( buf + sizeof drag_info->name_, &drag_info->pos_, sizeof drag_info->pos_ );
   return FAIL_RES;
}

template<class HEADER>
inline void msgDRAG<HEADER>::
execute( void )
{
   ACE_TRACE( "(msgDRAG::execute)" );
   if( change )
   {
      char name_buf[32];
      sprintf( name_buf, "dragger_%d", *buf );
      SoTranslate1Dragger* dragger = ( SoTranslate1Dragger* )SoNode::getByName( name_buf );
      if( dragger )
      {
	 float pos;
	 memcpy( &pos, buf + 1, sizeof pos );
	 dragger->translation.setValue( pos, 0.0, 0.0 );
	 ACE_DEBUG(( LM_DEBUG, "dragger no %d at pos %f\n", *buf, pos ));
      }
   }
}

#endif

