/*
 * File         : msgMOVEPL.h
 * Author       : Pawel Roman
 * Created      : Thu Feb 12 13:36:50 1998
 * Revision     : $Revision: 1.1 $
 * Updated      : $Date: 1998/02/12 20:35:45 $
 * Copyright: Northeast Parallel Architectures Center
 *            at Syracuse University 1998
 */

#ifndef __MSGMOVEPL_H
#define __MSGMOVEPL_H

#include "ViewerMessage.h"

template<class HEADER>
class msgMOVEPL
: public ViewerMessage<HEADER>
{
 public:
  msgMOVEPL( SoXtWebViewer* vw );
  MRESULT read( const SAP& sap, const Header& header );
  MRESULT write( const SAP& sap );
  MRESULT set( const void* params = NULL );
  void execute( void );
  
 private:
  char* buf;
};

template<class HEADER>
inline msgMOVEPL<HEADER>::
msgMOVEPL( SoXtWebViewer* vw )
  : ViewerMessage<HEADER>( Header( MOVEPL, vw->getVID() ), vw ), buf( 0 )
{ 
  ACE_TRACE( "(msgMOVEPL::msgMOVEPL)" );
}

template<class HEADER>
inline msgMOVEPL<HEADER>::MRESULT msgMOVEPL<HEADER>::
read( const SAP& sap, const Header& header )
{
  ACE_TRACE( "(msgMOVEPL::read)" );
  TRACE_DELETE_TAB( buf );
  if( readMsg( sap, buf, header.length() ) == OK_RES )
    {
      buf[header.length()] = '\0';
      checkVID( header );
      return OK_RES;
    }
  else
    return END_RES;
}

template<class HEADER>
inline msgMOVEPL<HEADER>::MRESULT msgMOVEPL<HEADER>::
write( const SAP& sap ) 
{ 
  ACE_TRACE( "(msgMOVEPL::write)" );
  if( header.write( sap ) == Header::LINK_OK )
    if( sap.send_n( buf, header.length() ) == header.length() )
      return OK_RES;
   
  return END_RES;
}

template<class HEADER>
inline msgMOVEPL<HEADER>::MRESULT msgMOVEPL<HEADER>::
set( const void* cbuf ) 
{ 
  ACE_TRACE( "(msgMOVEPL::set)" );
  TRACE_DELETE_TAB( buf );
  size_t name_len = strlen( *( ( const char** )cbuf + 1 ) );
  TRACE_NEW_2( buf, char[sizeof( float ) + name_len] );
  if( ! buf )
    return FAIL_RES;
   
  ACE_DEBUG(( LM_DEBUG, "node name: %s, trans: %f\n", *( ( const char** )cbuf + 1 ),
	      **( float** )cbuf ));

  memcpy( buf, *( float** )cbuf, sizeof( float ) );
  memcpy( buf + sizeof( float ), *( ( const char** )cbuf + 1 ), name_len );
  header.setLength( sizeof( float ) + name_len );
  return OK_RES;
}

template<class HEADER>
inline void msgMOVEPL<HEADER>::
execute( void )
{
  ACE_TRACE( "(msgMOVEPL::execute)" );
  if( change && buf )
    {
      float trans;
      SoPlaneable* pl = ( SoPlaneable* )SoNode::getByName( buf + sizeof trans );
      if( pl )
	{
	  memcpy( &trans, buf, sizeof trans );
	  ACE_DEBUG(( LM_DEBUG, "node name: %s, trans: %f\n", buf + sizeof trans, trans ));
	  pl->moveYourself( trans );
	}
    }
}

#endif

