/* $Id: msgNGRND.h,v 1.2 1997/09/25 20:32:14 remek Exp $ */
/*
 * File         : msgNGRND.h
 * Author       : Pawel Roman (proman@npac.syr.edu)
 * Created      : Wed Aug 20 13:37:58 1997
 * Revision     : $Revision: 1.2 $
 * Updated      : $Date: 1997/09/25 20:32:14 $
 * Copyright: Northeast Parallel Architectures Center
 *            at Syracuse University 1997
 */

#ifndef __MSGNGRND_H
#define __MSGNGRND_H

#include "SimEngMessage.h"
#include "SoStaticObj.h"

template<class HEADER>
class msgNGRND
   : public SimEngMessage<HEADER>
{
   public:
      msgNGRND( SoXtWebViewer* vw );
      void execute( void );

   private:
      double lon;
      double lat;
      char* name;
      char* desc;
      SoStaticObj::Type type;
};

template<class HEADER>
inline msgNGRND<HEADER>::
msgNGRND( SoXtWebViewer* vw )
   : SimEngMessage<HEADER>( Header( NEWGROUND, HEADER::BROADCAST ), vw )
{ 
   ACE_TRACE( "(msgNGRND::msgNGRND)" );
}

template<class HEADER>
inline void msgNGRND<HEADER>::
execute( void )
{
   ACE_TRACE( "(msgNGRND::execute)" );
   if( change )
   {
      ACE_HEX_DUMP(( LM_DEBUG, mb->base(), mb->size(), "NEWGROUND" ));
      ACE_ASSERT( mb->size() > 1 );
      type = ( SoStaticObj::Type )*mb->base();
      name = mb->base() + 1;
      size_t name_len = strlen( name ) + 1;
      memcpy( &lon, name + name_len, sizeof lon );
      memcpy( &lat, name + name_len + sizeof lon, sizeof lat );
      desc = name + name_len + sizeof lon + sizeof lat;
      ACE_DEBUG(( LM_DEBUG, "type: %d, name: %s, pos[%f, %f], desc: %s\n",
		  type, name, lon, lat, desc ));
      
      SoTranslation* pos = new SoTranslation;
      if( pos )
      {
	 TRACE_REF( pos );
	 GeoPoint gp = viewer->getOrigin().dump() - GeoPoint( lon, lat ).dump();
	 SbVec3f vec = gp.dump().getScaledVec( viewer->getScale() );
	 if( viewer->pointOnSurface( vec ) )
	 {
	    pos->translation.setValue( vec );
	    viewer->addStaticObj( new SoStaticObj( name, pos, type ) );
	 }
	 else
	    ACE_ERROR(( LM_ERROR, "[!](msgNGRND::execute) No ground for obj\n" ));

	 TRACE_UNREF( pos );
      }
      else
	 ACE_ERROR(( LM_ERROR, "[!](msgNGRND::execute) No memory for pos\n" ));
   }
}

#endif

