/* $Id: msgNWRL.h,v 1.5 1997/09/25 20:32:15 remek Exp $ */
/*
 * File:       msgNWRL.h
 * Author:     Pawel Roman (proman@npac.syr.edu)
 * Created:    Thu Jul 24 14:33:37 1997
 * Revision:   $Revision: 1.5 $
 * Updated:    $Date: 1997/09/25 20:32:15 $
 * Copyright 1996/7 NPAC, Syracuse University
 *
 */

#ifndef _MSGNWRL_H
#define _MSGNWRL_H

#include "ViewerMessage.h"
#include "Proto.h"

template<class HEADER>
class msgNWRL : public ViewerMessage<HEADER>
{
   public:
      msgNWRL( SoXtWebViewer* vw );
      MRESULT read( const SAP& sap, const Header& header );
      MRESULT write( const SAP& sap );
      MRESULT set( const void* params = NULL );
      void execute( void );
      
   private:
      WorldParams wrlParams;
};

template<class HEADER>
inline msgNWRL<HEADER>::
msgNWRL( SoXtWebViewer* vw )
   : ViewerMessage<HEADER>( Header( NWRL, vw->getVID(), sizeof wrlParams ), vw )
{ 
   ACE_TRACE( "(msgNWRL::msgNWRL)" );
}

template<class HEADER>
inline msgNWRL<HEADER>::MRESULT msgNWRL<HEADER>::
read( const SAP& sap, const Header& header )
{
   ACE_TRACE( "(msgNWRL::read)" );
   if( sap.recv_n( wrlParams, sizeof wrlParams ) == header.length() )
   {
      ACE_DEBUG(( LM_DEBUG, "ulLat=%.2f, ulLon=%.2f, szLat=%d, szLon=%d\n",
		  wrlParams[0] / 60.0, wrlParams[1] / 60.0, wrlParams[2], wrlParams[3] ));
      
      checkVID( header );
      return OK_RES;
   }
   else
      return END_RES;
}

template<class HEADER>
inline msgNWRL<HEADER>::MRESULT msgNWRL<HEADER>::
write( const SAP& sap ) 
{ 
   ACE_TRACE( "(msgNWRL::write)" );
   if( header.write( sap ) == Header::LINK_OK )
      if( sap.send_n( &viewer->getWorldParams(), sizeof wrlParams ) == header.length() )
	 return OK_RES;

   return END_RES;
}

template<class HEADER>
inline msgNWRL<HEADER>::MRESULT msgNWRL<HEADER>::
set( const void* params ) 
{ 
   memcpy( wrlParams, params, sizeof wrlParams );
   return OK_RES;
}

template<class HEADER>
inline void msgNWRL<HEADER>::
execute( void )
{
   ACE_TRACE( "(msgNWRL::execute)" );
   if( change )
      viewer->buildNewWorld( wrlParams );
}

#endif
