/* $Id: msgQSYNC.h,v 1.2 1997/09/25 20:32:15 remek Exp $ */
/*
 * File         : msgQSYNC.h
 * Author       : Pawel Roman
 * Created      : Mon Sep 15 14:01:45 1997
 * Revision     : $Revision: 1.2 $
 * Updated      : $Date: 1997/09/25 20:32:15 $
 * Copyright: Northeast Parallel Architectures Center
 *            at Syracuse University 1997
 */

#ifndef __MSGQSYNC_H
#define __MSGQSYNC_H

#include "ViewerMessage.h"
#include "Proto.h"

template<class HEADER>
class msgQSYNC : public ViewerMessage<HEADER>
{
   public:
      msgQSYNC( SoXtWebViewer* vw );
      MRESULT read( const SAP& sap, const Header& header );
      MRESULT write( const SAP& sap );
      MRESULT set( const void* params = NULL );
      void execute( void );
  
   private:
      HEADER::VID vid;
};

template<class HEADER>
inline msgQSYNC<HEADER>::
msgQSYNC( SoXtWebViewer* vw )
   : ViewerMessage<HEADER>( Header( QSYNC, vw->getVID(), sizeof vid ), vw )
{ 
   ACE_TRACE( "(msgQSYNC::msgQSYNC)" );
}

template<class HEADER>
inline msgQSYNC<HEADER>::MRESULT msgQSYNC<HEADER>::
read( const SAP& sap, const Header& header )
{
   ACE_TRACE( "(msgQSYNC::read)" );
   if( sap.recv_n( &vid, sizeof vid ) == header.length() )
   {
      checkVID( header );
      return OK_RES;
   }
   else
      return END_RES;
}

template<class HEADER>
inline msgQSYNC<HEADER>::MRESULT msgQSYNC<HEADER>::
write( const SAP& sap ) 
{ 
   ACE_TRACE( "(msgQSYNC::write)" );
   vid = viewer->getVID();
   if( header.write( sap ) == Header::LINK_OK )
      if( sap.send_n( &vid, sizeof vid ) == header.length() )
	 return OK_RES;
   
   return END_RES;
}

template<class HEADER>
inline msgQSYNC<HEADER>::MRESULT msgQSYNC<HEADER>::
set( const void* viewerID ) 
{ 
   ACE_TRACE( "(msgQSYNC::set)" );
   vid = *( HEADER::VID* )viewerID;
   return OK_RES;
}

template<class HEADER>
inline void msgQSYNC<HEADER>::
execute( void )
{
   ACE_TRACE( "(msgQSYNC::execute)" );
   if( change )
      viewer->sendQuickSynchro();
}

#endif

