/*
 * File         : msgSHOWISO.h
 * Author       : Pawel Roman
 * Created      : Tue Feb 10 14:32:10 1998
 * Revision     : $Revision: 1.2 $
 * Updated      : $Date: 1998/02/12 02:32:30 $
 * Copyright: Northeast Parallel Architectures Center
 *            at Syracuse University 1998
 */

#ifndef __MSGSHOWISO_H
#define __MSGSHOWISO_H

#include "ViewerMessage.h"

template<class HEADER>
class msgSHOWISO
: public ViewerMessage<HEADER>
{
 public:
      msgSHOWISO( SoXtWebViewer* vw );
      MRESULT read( const SAP& sap, const Header& header );
      MRESULT write( const SAP& sap );
      MRESULT set( const void* params = NULL );
      void execute( void );
  
   private:
	char buf[5];
};

template<class HEADER>
inline msgSHOWISO<HEADER>::
msgSHOWISO( SoXtWebViewer* vw )
   : ViewerMessage<HEADER>( Header( SHOWISO, vw->getVID(), sizeof buf ), vw )
{ 
   ACE_TRACE( "(msgSHOWISO::msgSHOWISO)" );
}

template<class HEADER>
inline msgSHOWISO<HEADER>::MRESULT msgSHOWISO<HEADER>::
read( const SAP& sap, const Header& header )
{
   ACE_TRACE( "(msgSHOWISO::read)" );
   if( sap.recv_n( &buf, sizeof buf ) == header.length() )
   {
      checkVID( header );
      return OK_RES;
   }
   else
      return END_RES;
}

template<class HEADER>
inline msgSHOWISO<HEADER>::MRESULT msgSHOWISO<HEADER>::
write( const SAP& sap ) 
{ 
   ACE_TRACE( "(msgSHOWISO::write)" );
   if( header.write( sap ) == Header::LINK_OK )
      if( sap.send_n( &buf, sizeof buf ) == header.length() )
	 return OK_RES;
   
   return END_RES;
}

template<class HEADER>
inline msgSHOWISO<HEADER>::MRESULT msgSHOWISO<HEADER>::
set( const void* cbuf ) 
{ 
   ACE_TRACE( "(msgSHOWISO::set)" );
   buf[0] = **( int** )cbuf;
   memcpy( &buf[1], *( ( float** )cbuf + 1 ), sizeof( float ) );
   return OK_RES;
}

template<class HEADER>
inline void msgSHOWISO<HEADER>::
execute( void )
{
   ACE_TRACE( "(msgSHOWISO::execute)" );
   if( change )
     {
       float thresh;
       memcpy( &thresh, &buf[1], sizeof thresh );
       viewer->showIsosurface( buf[0], thresh );
     }
}

#endif

