/*
 * File         : msgSHOWSLICE.h
 * Author       : Pawel Roman (proman@npac.syr.edu)
 * Created      : Tue Feb 10 14:26:26 1998
 * Revision     : $Revision: 1.1 $
 * Updated      : $Date: 1998/02/10 21:20:56 $
 * Copyright: Northeast Parallel Architectures Center
 *            at Syracuse University 1998
 */

#ifndef __MSGSHOWSLICE_H
#define __MSGSHOWSLICE_H


#include "ViewerMessage.h"

template<class HEADER>
class msgSHOWSLICE
: public ViewerMessage<HEADER>
{
 public:
      msgSHOWSLICE( SoXtWebViewer* vw );
      MRESULT read( const SAP& sap, const Header& header );
      MRESULT write( const SAP& sap );
      MRESULT set( const void* params = NULL );
      void execute( void );
  
   private:
      signed char var;
};

template<class HEADER>
inline msgSHOWSLICE<HEADER>::
msgSHOWSLICE( SoXtWebViewer* vw )
   : ViewerMessage<HEADER>( Header( SHOWSLICE, vw->getVID(), sizeof var ), vw )
{ 
   ACE_TRACE( "(msgSHOWSLICE::msgSHOWSLICE)" );
}

template<class HEADER>
inline msgSHOWSLICE<HEADER>::MRESULT msgSHOWSLICE<HEADER>::
read( const SAP& sap, const Header& header )
{
   ACE_TRACE( "(msgSHOWSLICE::read)" );
   if( sap.recv_n( &var, sizeof var ) == header.length() )
   {
      checkVID( header );
      return OK_RES;
   }
   else
      return END_RES;
}

template<class HEADER>
inline msgSHOWSLICE<HEADER>::MRESULT msgSHOWSLICE<HEADER>::
write( const SAP& sap ) 
{ 
   ACE_TRACE( "(msgSHOWSLICE::write)" );
   if( header.write( sap ) == Header::LINK_OK )
      if( sap.send_n( &var, sizeof var ) == header.length() )
	 return OK_RES;
   
   return END_RES;
}

template<class HEADER>
inline msgSHOWSLICE<HEADER>::MRESULT msgSHOWSLICE<HEADER>::
set( const void* buf ) 
{ 
   ACE_TRACE( "(msgSHOWSLICE::set)" );
   var = *( signed char* )buf;
   return OK_RES;
}

template<class HEADER>
inline void msgSHOWSLICE<HEADER>::
execute( void )
{
   ACE_TRACE( "(msgSHOWSLICE::execute)" );
   if( change )
      viewer->switchVariableTo( var );
}

#endif

