/*
 * File         : msgSHOWWIND.h
 * Author       : Pawel Roman (proman@npac.syr.edu)
 * Created      : Tue Feb 10 14:24:22 1998
 * Revision     : $Revision: 1.1 $
 * Updated      : $Date: 1998/02/10 21:20:56 $
 * Copyright: Northeast Parallel Architectures Center
 *            at Syracuse University 1998
 */

#ifndef __MSGSHOWWIND_H
#define __MSGSHOWWIND_H


#include "ViewerMessage.h"

template<class HEADER>
class msgSHOWWIND
   : public ViewerMessage<HEADER>
{
   public:
      msgSHOWWIND( SoXtWebViewer* vw );
      MRESULT read( const SAP& sap, const Header& header );
      MRESULT write( const SAP& sap );
      MRESULT set( const void* params = NULL );
      void execute( void );

   private:
      char on_off;
};

template<class HEADER>
inline msgSHOWWIND<HEADER>::
msgSHOWWIND( SoXtWebViewer* vw )
   : ViewerMessage<HEADER>( Header( SHOWWIND, vw->getVID(), sizeof on_off ), vw )
{ 
   ACE_TRACE( "(msgSHOWWIND::msgSHOWWIND)" );
}

template<class HEADER>
inline msgSHOWWIND<HEADER>::MRESULT msgSHOWWIND<HEADER>::
read( const SAP& sap, const Header& header )
{
   ACE_TRACE( "(msgSHOWWIND::read)" );
   if( sap.recv_n( &on_off, sizeof on_off ) == header.length() )
   {
      checkVID( header );
      return OK_RES;
   }
   else
      return END_RES;
}

template<class HEADER>
inline msgSHOWWIND<HEADER>::MRESULT msgSHOWWIND<HEADER>::
write( const SAP& sap ) 
{ 
   ACE_TRACE( "(msgSHOWWIND::write)" );
   on_off = viewer->windShown;
   if( header.write( sap ) == Header::LINK_OK )
      if( sap.send_n( &on_off, sizeof on_off ) == header.length() )
	 return OK_RES;
   
   return END_RES;
}

template<class HEADER>
inline msgSHOWWIND<HEADER>::MRESULT msgSHOWWIND<HEADER>::
set( const void* msg ) 
{ 
   ACE_TRACE( "(msgSHOWWIND::set)" );
   on_off = *( char* )msg;
   return OK_RES;
}

template<class HEADER>
inline void msgSHOWWIND<HEADER>::
execute( void )
{
   ACE_TRACE( "(msgSHOWWIND::execute)" );
   if( change )
   {
     viewer->showWind( on_off );
   }
}

#endif

