/* $Id: msgSYNCI.h,v 1.4 1997/09/25 20:32:17 remek Exp $ */
/*
 * File         : msgSYNCI.h
 * Author       : Pawel Roman
 * Created      : Thu Aug  7 12:26:44 1997
 * Revision     : $Revision: 1.4 $
 * Updated      : $Date: 1997/09/25 20:32:17 $
 * Copyright: Northeast Parallel Architectures Center
 *            at Syracuse University 1997
 */

#ifndef __MSGSYNCI_H
#define __MSGSYNCI_H

#include "ViewerMessage.h"
#include "Proto.h"

#include <ace/Message_Block.h>

template<class HEADER>
class msgSYNCI : public ViewerMessage<HEADER>
{
   public:
      msgSYNCI( SoXtWebViewer* vw );
      MRESULT read( const SAP& sap, const Header& header );
      MRESULT write( const SAP& sap );
      MRESULT set( const void* params = NULL );
      void execute( void );
  
   private:
      ACE_Message_Block* mb;
      bool initialState;
};

template<class HEADER>
inline msgSYNCI<HEADER>::
msgSYNCI( SoXtWebViewer* vw )
   : ViewerMessage<HEADER>( Header( SYNCI, vw->getVID() ), vw ),
     initialState( true )
{ 
   ACE_TRACE( "(msgSYNCI::msgSYNCI)" );
}

template<class HEADER>
inline msgSYNCI<HEADER>::MRESULT msgSYNCI<HEADER>::
read( const SAP& sap, const Header& header )
{
   ACE_TRACE( "(msgSYNCI::read)" );
   Header::LengthType msg_len = header.length();
   TRACE_NEW_2( mb, ACE_Message_Block( msg_len ) );
   if( ! mb )
      ACE_ERROR_RETURN(( LM_ERROR, "[!](msgSYNCI::read) allocation error\n" ),
		       OK_RES );
   
   if( sap.recv_n( mb->base(), msg_len ) == msg_len )
   {
      checkVID( header );
      ACE_HEX_DUMP(( LM_DEBUG, mb->base(), mb->size(), "SYNCI" ));
      return OK_RES;
   }
   else
      return END_RES;
}

template<class HEADER>
inline msgSYNCI<HEADER>::MRESULT msgSYNCI<HEADER>::
write( const SAP& sap ) 
{ 
   ACE_TRACE( "(msgSYNCI::write)" );
   ACE_Message_Block mb;
   char marker;
   if( viewer->worldType() == SoXtWebViewer::DYNAMIC )
   {
      marker = NWRL;
      mb.size( sizeof( LayerCode ) + sizeof( HEADER::MsgID ) +
	       sizeof( WorldParams ) );
      mb.copy( ( const char* )&viewer->getLayer(), sizeof( LayerCode ) );
      mb.copy( ( const char* )&marker, sizeof marker );
      mb.copy( ( const char* )&viewer->getWorldParams(), sizeof( WorldParams ) );
   }
   else if( viewer->worldType() == SoXtWebViewer::STATIC )
   {
      marker = NURL;
      mb.size( sizeof( LayerCode ) + sizeof( HEADER::MsgID ) +
	       strlen( viewer->getURL() )+1 );
      mb.copy( ( const char* )&viewer->getLayer(), sizeof( LayerCode ) );
      mb.copy( ( const char* )&marker, sizeof marker );
      mb.copy( viewer->getURL(), strlen( viewer->getURL() )+1 );
   }
   else
   {
      marker = CAMP;
      mb.size( sizeof( LayerCode ) + sizeof( HEADER::MsgID ) );
      mb.copy( ( const char* )&viewer->getLayer(), sizeof( LayerCode ) );
      mb.copy( ( const char* )&marker, sizeof marker );
   }
   
   ACE_HEX_DUMP(( LM_DEBUG, mb.base(), mb.size(), "SYNCI" ));
   header.setLength( mb.size() );
   if( header.write( sap ) == Header::LINK_OK )
      if( sap.send_n( mb.base(), header.length() ) != mb.size() )
	 return OK_RES;

   return END_RES;
}

template<class HEADER>
inline msgSYNCI<HEADER>::MRESULT msgSYNCI<HEADER>::
set( const void* ) 
{ 
   ACE_TRACE( "(msgSYNCI::set)" );
   return OK_RES;
}

template<class HEADER>
inline void msgSYNCI<HEADER>::
execute( void )
{
   ACE_TRACE( "(msgSYNCI::execute)" );
   if( change && initialState && mb )
   {
      LayerCode layer = *mb->rd_ptr();
      mb->rd_ptr( 1 );
      if( *mb->rd_ptr() == NWRL )
      {
	 mb->rd_ptr( 1 );
	 viewer->buildNewWorld( ( WorldPrm* )mb->rd_ptr() );
      }
      else if( *mb->rd_ptr() == NURL )
      {
	 mb->rd_ptr( 1 );
	 viewer->loadNewGraph( mb->rd_ptr() );
      }

      if( layer != -1 )
	 viewer->switchLayerTo( layer );
      
      initialState = false;
   }
   TRACE_DELETE( mb );
}

#endif

