/* $Id: msgSYNCR.h,v 1.4 1997/09/25 20:32:17 remek Exp $ */
/*
 * Author       : Pawel Roman (proman@npac.syr.edu)
 * Created      : Mon Jul 28 18:59:32 1997
 * Revision     : $Revision: 1.4 $
 * Updated      : $Date: 1997/09/25 20:32:17 $
 * Copyright: Northeast Parallel Architectures Center
 *            at Syracuse University 1997
 */

#ifndef _MSGSYNCR_H
#define _MSGSYNCR_H

#include "ViewerMessage.h"
#include "Proto.h"

template<class HEADER>
class msgSYNCR : public ViewerMessage<HEADER>
{
   public:
      msgSYNCR( SoXtWebViewer* vw );
      MRESULT read( const SAP& sap, const Header& header );
      MRESULT write( const SAP& sap );
      MRESULT set( const void* params = NULL );
      void execute( void );
  
   private:
      HEADER::VID vid;
};

template<class HEADER>
inline msgSYNCR<HEADER>::
msgSYNCR( SoXtWebViewer* vw )
   : ViewerMessage<HEADER>( Header( SYNCR, vw->getVID(), sizeof vid ), vw )
{ 
   ACE_TRACE( "(msgSYNCR::msgSYNCR)" );
}

template<class HEADER>
inline msgSYNCR<HEADER>::MRESULT msgSYNCR<HEADER>::
read( const SAP& sap, const Header& header )
{
   ACE_TRACE( "(msgSYNCR::read)" );
   if( sap.recv_n( &vid, sizeof vid ) == header.length() )
   {
      checkVID( header );
      return OK_RES;
   }
   else
      return END_RES;
}

template<class HEADER>
inline msgSYNCR<HEADER>::MRESULT msgSYNCR<HEADER>::
write( const SAP& sap ) 
{ 
   ACE_TRACE( "(msgSYNCR::write)" );
   if( header.write( sap ) == Header::LINK_OK )
      if( sap.send_n( &vid, sizeof vid ) == header.length() )
	 return OK_RES;
   
   return END_RES;
}

template<class HEADER>
inline msgSYNCR<HEADER>::MRESULT msgSYNCR<HEADER>::
set( const void* viewerID ) 
{ 
   ACE_TRACE( "(msgSYNCR::set)" );
   vid = *( HEADER::VID* )viewerID;
   return OK_RES;
}

template<class HEADER>
inline void msgSYNCR<HEADER>::
execute( void )
{
   ACE_TRACE( "(msgSYNCR::execute)" );
   if( change )
      viewer->sendSynchro( vid );
}

#endif
