/* $Id: tweety.C,v 1.17 1997/09/25 20:32:19 remek Exp $ */
/*
 * File:       tweety.C
 * Author:     Pawel Roman (proman@npac.syr.edu)
 * Created:    long time ago
 * Revision:   $Revision: 1.17 $
 * Updated:    $Date: 1997/09/25 20:32:19 $
 * Copyright 1996/97 NPAC, Syracuse University
 *
 */

#include "Comm.h"

#include <sys/types.h>
#include <sys/time.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <unistd.h>
#include <bstring.h>
#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <string.h>

#include <Inventor/nodes/SoSelection.h>
#include <Inventor/nodes/SoWWWInline.h>
#include <Inventor/nodes/SoPerspectiveCamera.h>
#include <Inventor/nodes/SoText3.h>
#include <Inventor/Xt/SoXt.h>
#include <stdlib.h>
#include <fstream.h>

#include <ace/SOCK_Acceptor.h>
#include <ace/SOCK_Connector.h>

#include "SoXtWebViewer.h"

int
main( int argc, char* argv[] )
{
#ifndef ACE_NLOGGING
   char* log_dir;
   ofstream log;
   if( log_dir = getenv( "GIS_LOG_DIR" ) )
   {
      const int CF_MODE = 0666;
      const char* CACHE_PREFIX = "LOG_";
      char* tempfile = tempnam( log_dir, CACHE_PREFIX );
      log.open( tempfile );
      if( log )
      {
	 chmod( tempfile, CF_MODE );
	 ACE_LOG_MSG->set_flags( ACE_Log_Msg::OSTREAM );
	 ACE_LOG_MSG->msg_ostream( &log );
	 ACE_LOG_MSG->clr_flags( ACE_Log_Msg::STDERR );
	 ACE_LOG_MSG->start_tracing();
      }
      else
	 cerr << "Cannot open " << tempfile << endl;
   }
   else
      cerr << "Cannot find GIS_LOG_DIR\n";
#endif /* ! ACE_NLOGGING */
   SceneBuilder::init();
   signal( SIGCHLD, SIG_IGN );
   Widget myWindow = SoXt::init( argv[0] );
   if( myWindow == NULL )
   {
      cerr << "SoXt::init failed\n";
      exit(1); 
   }

   SoXtWebViewer::ContrStatus status;
   SoXtWebViewer::ViewerID vid;
#if defined( TANGO )
   status = *argv[3] == 'm' ? SoXtWebViewer::MASTER : SoXtWebViewer::SLAVE; 
   ACE_DEBUG(( LM_DEBUG, "--- %s: %d ---\n",
	       status == SoXtWebViewer::MASTER ? "master" : "slave", getpid() ));
   ACE_DEBUG(( LM_DEBUG, "sock: %d, vid: %d\n", atoi( argv[1] ), atoi( argv[2] ) ));
   ACE_LSOCK_Stream sock;
   sock.set_handle( atoi( argv[1] ) );
   vid = atoi( argv[2] );
#else /* ! TANGO */
   const u_short DEFAULT_PORT = 4444;
   const char* DEFAULT_HOST = "kopernik.npac.syr.edu";
   const SoXtWebViewer::ContrStatus DEFAULT_STATUS = SoXtWebViewer::SLAVE;
   ACE_SOCK_Stream sock;
   status = argc > 1 ? ( *argv[1] == 'm' ? SoXtWebViewer::MASTER : SoXtWebViewer::SLAVE )
      : DEFAULT_STATUS;
   u_short port = argc > 2 ? ACE_OS::atoi( argv[2] ) : DEFAULT_PORT;
   if( status == SoXtWebViewer::MASTER )
   {
      ACE_DEBUG(( LM_DEBUG, "--- master: %d ---\n", getpid() ));
      if( ACE_SOCK_Acceptor( ACE_INET_Addr( port ) ).accept( sock ) < 0 )
      {
	 perror( "ACE_SOCK_Acceptor" );
	 exit( 1 );
      }
   }
   else
   {
      ACE_DEBUG(( LM_DEBUG, "--- slave: %d ---\n", getpid() ));
      if( ACE_SOCK_Connector()
	  .connect( sock, ACE_INET_Addr( port, argc > 3 ? argv[3] : DEFAULT_HOST ) ) < 0 )
      {
	 perror( "ACE_SOCK_Connector" );
	 exit( 1 );
      }
   }
   vid = 1;
#endif /* TANGO */
   SoXtWebViewer* Viewer = new SoXtWebViewer( sock, vid, status, myWindow );
   SoSelection* sep = new SoSelection;
   TRACE_REF( sep );
   Viewer->setSceneGraph( sep );
   if( argc < 5 )
   {
      Viewer->setBackgroundColor( SbColor( 0.3, 0.4, 0.6 ) );
      SoPerspectiveCamera* aCamera = new SoPerspectiveCamera;
      sep->addChild( aCamera );
      SoText3* aText = new SoText3;
      aText->string.setValue( "3D GIS" );
      sep->addChild( aText );
      aCamera->viewAll( sep, Viewer->getViewportRegion() );
   }
   else
      Viewer->loadNewGraph( argv[4] );

   Viewer->show();
   SoXt::show( myWindow );
   SoXt::mainLoop();
   return 0;
}
